@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: arg.c 900 2008-01-05 01:13:26Z hubert@u.washington.e
 
 /*
  * ========================================================================
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -55,6 +55,10 @@ static char args_err_missing_pinerc[] =	N_("missing argument for option \"-piner
 #if	defined(DOS) || defined(OS2)
 static char args_err_missing_aux[] =		N_("missing argument for option \"-aux\"");
 #endif
+#ifdef SMIME
+static char args_err_missing_smimedir[] =	N_("missing argument for option \"-smimedir\"");
+static char args_err_non_abs_smimedir[] =	N_("argument to \"-smimedir\" should be fully-qualified");
+#endif /* SMIME */
 #ifdef	PASSFILE
 static char args_err_missing_passfile[] =	N_("missing argument for option \"-passfile\"");
 static char args_err_non_abs_passfile[] =	N_("argument to \"-passfile\" should be fully-qualified");
@@ -131,6 +135,10 @@ N_(" -nosplash \tDisable the PC-Alpine splash screen"),
 N_(" -erase_stored_passwords\tEliminate any stored passwords"),
 #endif
 
+#ifdef SMIME
+N_(" -smimedir <fully_qualified_path>\tdirectory where smime personal certificates are saved"),
+#endif /* SMIME */
+
 #ifdef	PASSFILE
 N_(" -passfile <fully_qualified_filename>\tSet the password file to something other"),
 N_("\t\tthan the default"),
@@ -274,6 +282,31 @@ Loop: while(--ac > 0)
 	      }
 #endif /* defined(APPLEKEYCHAIN) || (WINCRED > 0) */
 
+#ifdef SMIME
+	      else if(strcmp(*av, "smimedir") == 0){
+		  if(--ac){
+		      if((str = *++av) != NULL){
+			  if(!is_absolute_path(str)){
+			      display_args_err(_(args_err_non_abs_smimedir),
+					       NULL, 1);
+			      ++usage;
+			  }
+			  else{
+			      if(pine_state->smimedir)
+				fs_give((void **)&pine_state->smimedir);
+
+			      pine_state->smimedir = cpystr(str);
+			  }
+		      }
+		  }
+		  else{
+		      display_args_err(_(args_err_missing_smimedir), NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+#endif /* SMIME */
 #ifdef	PASSFILE
 	      else if(strcmp(*av, "passfile") == 0){
 		  if(--ac){
@@ -657,8 +690,10 @@ Loop: while(--ac > 0)
 				  path[sizeof(path)-1] = '\0';
 			      }
 			      else{
-				  getcwd(dir, sizeof(path));
-				  build_path(path, dir, str, sizeof(path));
+				  if(getcwd(dir, sizeof(path)) != NULL)
+				     build_path(path, dir, str, sizeof(path));
+				  else
+				    alpine_panic(_("getcwd() call failed while parsing argument (1)"));
 			      }
 
 			      /*
@@ -701,8 +736,10 @@ Loop: while(--ac > 0)
 				  path[sizeof(path)-1] = '\0';
 			      }
 			      else{
-				  getcwd(dir, sizeof(path));
-				  build_path(path, dir, str, sizeof(path));
+				  if(getcwd(dir, sizeof(path)) != NULL)
+				    build_path(path, dir, str, sizeof(path));
+				  else
+				    alpine_panic(_("getcwd() call failed while parsing argument (2)"));
 			      }
 
 			      if(pine_state->pconf)
@@ -1289,7 +1326,8 @@ pinerc_cmdline_opt(char *arg)
     /*--value is non-empty--*/
     if(*value == '"' && !v->is_list){
       value++;
-      for(p1 = value; *p1 && *p1 != '"'; p1++);
+      for(p1 = value; *p1 && *p1 != '"'; p1++)
+	;
         if(*p1 == '"')
           *p1 = '\0';
         else
