@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: colorconf.c 934 2008-02-23 00:44:29Z hubert@u.washin
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -106,7 +106,7 @@ color_config_screen(struct pine *ps, int edit_exceptions)
 					      : _("SETUP COLOR"),
 			      /* TRANSLATORS: Print something1 using something2.
 				 configuration is something1 */
-			      _("configuration"), 0)){
+			      _("configuration"), 0, NULL)){
       case 0:
 	break;
 
@@ -2708,7 +2708,7 @@ color_edit_screen(struct pine *ps, CONF_S **cl)
     name[0] = '\0';
     if(is_general){
 	p = srchstr(vtmp->name, "-foreground-color");
-	snprintf(name, sizeof(name), "%.*s", p ? MIN(p - vtmp->name, 30) : 30, vtmp->name);
+	snprintf(name, sizeof(name), "%.*s", p ? (int) MIN(p - vtmp->name, 30) : 30, vtmp->name);
 	name[sizeof(name)-1] = '\0';
 	if(islower((unsigned char)name[0]))
 	  name[0] = toupper((unsigned char)name[0]);
@@ -2716,7 +2716,7 @@ color_edit_screen(struct pine *ps, CONF_S **cl)
     else if(is_index){
 	p = srchstr(vtmp->name, "-foreground-color");
 	snprintf(name, sizeof(name), "%.*s Symbol",
-		p ? MIN(p - vtmp->name, 30) : 30, vtmp->name);
+		p ? (int) MIN(p - vtmp->name, 30) : 30, vtmp->name);
 	name[sizeof(name)-1] = '\0';
 	if(islower((unsigned char)name[0]))
 	  name[0] = toupper((unsigned char)name[0]);
@@ -2918,7 +2918,7 @@ color_edit_screen(struct pine *ps, CONF_S **cl)
     rv = conf_scroll_screen(ps, &screen, first_line,
 			    ew == Post ? _("SETUP COLOR EXCEPTIONS")
 				       : _("SETUP COLOR"),
-			    _("configuration"), 1);
+			    _("configuration"), 1, NULL);
 
     opt_screen = saved_screen;
     ps->mangled_screen = 1;
