@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: imap.c 1266 2009-07-14 18:39:12Z hubert@u.washington
 
 /*
  * ========================================================================
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2009 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -104,7 +104,7 @@ int   answer_cert_failure(int, MSGNO_S *, SCROLL_S *);
 #ifdef	LOCAL_PASSWD_CACHE
 int   read_passfile(char *, MMLOGIN_S **);
 void  write_passfile(char *, MMLOGIN_S *);
-int   preserve_prompt(void);
+int   preserve_prompt(char *);
 void  update_passfile_hostlist(char *, char *, STRLIST_S *, int);
 
 static	MMLOGIN_S	*passfile_cache = NULL;
@@ -115,7 +115,6 @@ int  save_password = 1;
 #ifdef	PASSFILE
 char  xlate_in(int);
 char  xlate_out(char);
-char *passfile_name(char *, char *, size_t);
 int  line_get(char *, size_t, char **);
 #endif	/* PASSFILE */
 
@@ -168,7 +167,7 @@ mm_notify(MAILSTREAM *stream, char *string, long int errflg)
 
     snprintf(ps_global->last_error, sizeof(ps_global->last_error), "%s : %.*s",
 	    (stream && stream->mailbox) ? stream->mailbox : "-no folder-",
-	    MIN(MAX_SCREEN_COLS, sizeof(ps_global->last_error)-70),
+	    (int) MIN(MAX_SCREEN_COLS, sizeof(ps_global->last_error)-70),
 	    string);
     ps_global->last_error[ps_global->ttyo ? ps_global->ttyo->screen_cols
 		    : sizeof(ps_global->last_error)-1] = '\0';
@@ -1990,57 +1989,6 @@ xlate_out(char c)
     else
       return(c);
 }
-
-
-char *
-passfile_name(char *pinerc, char *path, size_t len)
-{
-    struct stat  sbuf;
-    char	*p = NULL;
-    int		 i, j;
-
-    if(!path || !((pinerc && pinerc[0]) || ps_global->passfile))
-      return(NULL);
-
-    if(ps_global->passfile)
-      strncpy(path, ps_global->passfile, len-1);
-    else{
-	if((p = last_cmpnt(pinerc)) && *(p-1) && *(p-1) != PASSFILE[0])
-	  for(i = 0; pinerc < p && i < len; pinerc++, i++)
-	    path[i] = *pinerc;
-	else
-	  i = 0;
-
-	for(j = 0; (i < len) && (path[i] = PASSFILE[j]); i++, j++)
-	  ;
-	
-    }
-
-    path[len-1] = '\0';
-
-    dprint((9, "Looking for passfile \"%s\"\n",
-	   path ? path : "?"));
-
-#if	defined(DOS) || defined(OS2)
-    return((our_stat(path, &sbuf) == 0
-	    && ((sbuf.st_mode & S_IFMT) == S_IFREG))
-	     ? path : NULL);
-#else
-    /* First, make sure it's ours and not sym-linked */
-    if(our_lstat(path, &sbuf) == 0
-       && ((sbuf.st_mode & S_IFMT) == S_IFREG)
-       && sbuf.st_uid == getuid()){
-	/* if too liberal permissions, fix them */
-	if((sbuf.st_mode & 077) != 0)
-	  if(our_chmod(path, sbuf.st_mode & ~077) != 0)
-	    return(NULL);
-	
-	return(path);
-    }
-    else
-	return(NULL);
-#endif
-}
 #endif	/* PASSFILE */
 
 
@@ -2053,8 +2001,9 @@ line_get(char *tmp, size_t len, char **textp)
   char *s;
 
   tmp[0] = '\0';
-  if (*textp == NULL || strlen(*textp) >= len - 1
-	|| (s = strchr(*textp, '\n')) == NULL)
+  if (*textp == NULL 
+	|| (s = strchr(*textp, '\n')) == NULL
+	|| (s - *textp) > len - 1)
     return 0;
 
   *s = '\0';
@@ -2353,7 +2302,7 @@ read_passfile(pinerc, l)
 #else /* PASSFILE */
 
     char  tmp[MAILTMPLEN], *ui[5];
-    int   i, j, n;
+    int   i, j, n, rv = 0;
 #ifdef SMIME
     char tmp2[MAILTMPLEN];
     char *text = NULL, *text2 = NULL;
@@ -2384,18 +2333,48 @@ read_passfile(pinerc, l)
      * do not init smime here, because the .pinerc might not have been
      * read and we do not really know where the keys and certificates really
      * are.
+     * Remark: setupwdcert will produce a null ps_global->pwdcert only when
+     * it is called for the first time and there are certificates at all,
+     * or when it is called after the first time and the user refuses to 
+     * create a self-signed certificate. In this situation we will just
+     * let the user live in an insecure world, but no more passwords will
+     * be saved in the password file, and only those found there will be used.
      */
-    if(ps_global->pwdcert == NULL)
-       setup_pwdcert(&ps_global->pwdcert);
     tmp2[0] = '\0';
     fgets(tmp2, sizeof(tmp2), fp);
     fclose(fp);
     if(strcmp(tmp2, "-----BEGIN PKCS7-----\n")){
-       if(encrypt_file((char *)tmp, NULL, (PERSONAL_CERT *)ps_global->pwdcert))
-	  encrypted++;
+       /* there is an already existing password file, that is not encrypted
+        * and there is no key to encrypt it yet, go again through setup_pwdcert
+	* and encrypt it now.
+	*/
+       if(tmp2[0]){	/* not empty, UNencrypted password file */
+	 if(ps_global->pwdcert == NULL)
+            rv = setup_pwdcert(&ps_global->pwdcert);
+	 if((rv == 0 || rv == -5) && ps_global->pwdcert == NULL)
+	    ps_global->pwdcert = (void *) ALPINE_self_signed_certificate(NULL, 0, ps_global->pwdcertdir, MASTERNAME);
+	 if(ps_global->pwdcert == NULL){
+	    q_status_message(SM_ORDER, 3, 3, 
+		" Failed to create private key. Using UNencrypted Password file. ");
+	    save_password = 0;
+	 }
+	 else{
+	    if(rv == 1){
+	       q_status_message(SM_ORDER, 3, 3, 
+		" Failed to unlock private key. Using UNencrypted Password file. ");
+	       save_password = 0;	/* do not save more passwords */
+	    }
+	 }
+	if(ps_global->pwdcert != NULL
+	    && encrypt_file((char *)tmp, NULL, (PERSONAL_CERT *)ps_global->pwdcert))
+	       encrypted++;
+       }
     }
-    else
+    else {
+       if(ps_global->pwdcert == NULL)
+         rv = setup_pwdcert(&ps_global->pwdcert);
        encrypted++;
+    }
 
     /* 
      * if password file is encrypted we attemtp to decrypt. We ask the 
@@ -2412,10 +2391,15 @@ read_passfile(pinerc, l)
     if(encrypted){
 	text = text2 = decrypt_file((char *)tmp, &i, (PERSONAL_CERT *)ps_global->pwdcert);
 	switch(i){
+	   case -2: using_passfile = 0;
+		    break;
+
 	   case 1 : save_password = 1;
+		    using_passfile = 1;
 		    break;
 
 	   case -1: save_password = 0;
+		    using_passfile = 1;
 		    break;
 
 	   default: break;
@@ -2425,7 +2409,13 @@ read_passfile(pinerc, l)
        fp = our_fopen(tmp, "rb");	/* reopen to read data */
 #endif /* SMIME */
 
-    using_passfile = 1;
+    if(using_passfile == 0){
+#ifdef SMIME
+      if(text) fs_give((void **)&text);
+#endif /* SMIME */
+      return using_passfile;
+    }
+
 #ifdef SMIME
     for(n = 0; encrypted ? line_get(tmp, sizeof(tmp), &text2) 
 	: (fgets(tmp, sizeof(tmp), fp) != NULL); n++){
@@ -2608,7 +2598,7 @@ write_passfile(pinerc, l)
     }
 
 #ifdef SMIME
-   strncpy(tmp2, tmp, sizeof(tmp2)-1);
+   strncpy(tmp2, tmp, sizeof(tmp2));
    tmp2[sizeof(tmp2)-1] = '\0';
 #endif /* SMIME */
 
@@ -2643,13 +2633,21 @@ write_passfile(pinerc, l)
     fclose(fp);
 #ifdef SMIME
     if(text != NULL){
-       if(encrypt_file((char *)tmp2, text, ps_global->pwdcert) == 0){
-	 if((fp = our_fopen(tmp2, "wb")) != NULL){
-	   fputs(text, fp);
-	   fclose(fp);
-	 }
+       if(ps_global->pwdcert == NULL){
+	  q_status_message(SM_ORDER, 3, 3, "Attempting to encrypt password file");
+	  i = setup_pwdcert(&ps_global->pwdcert);
+	  if(i == 0 && ps_global->pwdcert == NULL)
+	    ps_global->pwdcert = (void *) ALPINE_self_signed_certificate(NULL, 0, ps_global->pwdcertdir, MASTERNAME);
        }
-       fs_give((void **)&text);
+       if(ps_global->pwdcert == NULL){	/* we tried but failed	*/
+	if(i == -1)
+	  q_status_message1(SM_ORDER, 3, 3, "Error: no directory %s to save certificates", ps_global->pwdcertdir);
+	else
+	  q_status_message(SM_ORDER, 3, 3, "Refusing to write non-encrypted password file");
+       }
+       else if(encrypt_file((char *)tmp2, text, ps_global->pwdcert) == 0)
+	  q_status_message(SM_ORDER, 3, 3, "Failed to encrypt password file");
+       fs_give((void **)&text);	/* do not save this text */
     }
 #endif /* SMIME */
 #endif /* PASSFILE */
@@ -2737,7 +2735,7 @@ get_passfile_user(pinerc, hostlist)
 
 
 int
-preserve_prompt(void)
+preserve_prompt(char *pinerc)
 {
 #ifdef	WINCRED
 # if	(WINCRED > 0)
@@ -2762,7 +2760,7 @@ preserve_prompt(void)
 #elif	APPLEKEYCHAIN
 
     int rc;
-    if(rc = macos_store_pass_prompt()){
+    if((rc = macos_store_pass_prompt()) != 0){
 	if(want_to(_("Preserve password for next login"),
 		   'y', 'x', NO_HELP, WT_NORM)
 	   == 'y'){
@@ -2782,10 +2780,17 @@ preserve_prompt(void)
     }
     return(0);
 #else /* PASSFILE */
-if(F_OFF(F_DISABLE_PASSWORD_FILE_SAVING,ps_global))
-    return(want_to(_("Preserve password on DISK for next login"), 
+    char tmp[MAILTMPLEN];
+    struct stat sbuf;
+
+    if(!passfile_name(pinerc, tmp, sizeof(tmp)) || our_stat(tmp, &sbuf) < 0)
+        return 0;
+
+    if(F_OFF(F_DISABLE_PASSWORD_FILE_SAVING,ps_global))
+	return(want_to(_("Preserve password on DISK for next login"), 
 		   'y', 'x', NO_HELP, WT_NORM)
 	   == 'y');
+    return(0);
 #endif /* PASSFILE */
 }
 
@@ -2925,10 +2930,10 @@ set_passfile_passwd(pinerc, passwd, user, hostlist, altflag, already_prompted)
     int	       altflag, already_prompted;
 {
     dprint((10, "set_passfile_passwd\n"));
-    if((passfile_cache || read_passfile(pinerc, &passfile_cache))
+    if(((already_prompted == 0 && preserve_prompt(pinerc))
+	   || already_prompted == 1)
        && !ps_global->nowrite_password_cache
-       && ((already_prompted == 0 && preserve_prompt())
-	   || already_prompted == 1)){
+       && (passfile_cache || read_passfile(pinerc, &passfile_cache))){
 	imap_set_passwd(&passfile_cache, passwd, user, hostlist, altflag, 0, 0);
 	write_passfile(pinerc, passfile_cache);
     }
