@@ -2,8 +2,8 @@
  * $Id: keymenu.h 1074 2008-06-04 00:08:43Z hubert@u.washington.edu $
  *
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -519,6 +519,8 @@ struct key_menu {
 #define	ATV_PRINT_KEY		20
 #define	ATV_REPLY_KEY		22
 #define	ATV_FORWARD_KEY		23
+#define ATV_DECRYPT_KEY		(ATV_PIPE_KEY + 14)
+#define ATV_SECURITY_KEY	(ATV_DECRYPT_KEY + 1)
 #define VIEW_ATT_KEY		 3
 #define VIEW_FULL_HEADERS_KEY	32
 #define	VIEW_VIEW_HANDLE	26
@@ -558,6 +560,8 @@ struct key_menu {
 #define TRUST_KEY		3
 #define PUBLIC_KEY		6
 #define PRIVATE_KEY		7
+#define DELETE_CERT_KEY		4	
+#define UNDELETE_CERT_KEY	5
 
 extern struct key_menu	cancel_keymenu,
 			ab_keymenu,
@@ -659,7 +663,10 @@ extern struct key_menu	cancel_keymenu,
 			smime_info_keymenu,
 			config_smime_manage_certs_menu_keymenu,
 			config_smime_manage_certs_work_keymenu,
-			smime_certificate_info_keymenu;
+			config_smime_manage_password_file_menu_keymenu,
+			smime_certificate_info_keymenu,
+			config_smime_add_new_key_keymenu,
+			config_smime_manage_view_cert_keymenu;
 
 extern struct key rev_msg_keys[];
 
