@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: ldapconf.c 1012 2008-03-26 00:44:22Z hubert@u.washin
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -130,7 +130,7 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 	    e != NULL;
 	    e = ldap_next_entry(res_list->ld, e)){
 	    char       *dn, *a;
-	    char      **cn, **org, **unit, **title, **mail, **sn;
+	    struct berval **cn, **org, **unit, **title, **mail, **sn;
 	    BerElement *ber;
 	    int         indent, have_mail;
 	    
@@ -143,28 +143,28 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		dprint((9, " %s", a ? a : "?"));
 		if(strcmp(a, res_list->info_used->cnattr) == 0){
 		    if(!cn)
-		      cn = ldap_get_values(res_list->ld, e, a);
+		      cn = ldap_get_values_len(res_list->ld, e, a);
 
-		    if(cn && !(cn[0] && cn[0][0])){
-			ldap_value_free(cn);
+		    if(cn && !ALPINE_LDAP_can_use(cn)){
+			ldap_value_free_len(cn);
 			cn = NULL;
 		    }
 		}
 		else if(strcmp(a, res_list->info_used->mailattr) == 0){
 		    if(!mail)
-		      mail = ldap_get_values(res_list->ld, e, a);
+		      mail = ldap_get_values_len(res_list->ld, e, a);
 		}
 		else if(strcmp(a, "o") == 0){
 		    if(!org)
-		      org = ldap_get_values(res_list->ld, e, a);
+		      org = ldap_get_values_len(res_list->ld, e, a);
 		}
 		else if(strcmp(a, "ou") == 0){
 		    if(!unit)
-		      unit = ldap_get_values(res_list->ld, e, a);
+		      unit = ldap_get_values_len(res_list->ld, e, a);
 		}
 		else if(strcmp(a, "title") == 0){
 		    if(!title)
-		      title = ldap_get_values(res_list->ld, e, a);
+		      title = ldap_get_values_len(res_list->ld, e, a);
 		}
 
 		our_ldap_memfree(a);
@@ -179,10 +179,10 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 
 		    if(strcmp(a,  res_list->info_used->snattr) == 0){
 			if(!sn)
-			  sn = ldap_get_values(res_list->ld, e, a);
+			  sn = ldap_get_values_len(res_list->ld, e, a);
 
-			if(sn && !(sn[0] && sn[0][0])){
-			    ldap_value_free(sn);
+			if(sn && !ALPINE_LDAP_can_use(sn)){
+			    ldap_value_free_len(sn);
 			    sn = NULL;
 			}
 		    }
@@ -191,7 +191,7 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		}
 	    }
 
-	    if(mail && mail[0] && mail[0][0])
+	    if(ALPINE_LDAP_can_use(mail))
 	      have_mail = 1;
 	    else
 	      have_mail = 0;
@@ -212,8 +212,8 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		if(need_mail && !have_mail)
 		  ctmpa->flags     |= CF_PRIVATE;
 
-		ctmpa->value      = cpystr(cn[0]);
-		ldap_value_free(cn);
+		ctmpa->value      = cpystr(cn[0]->bv_val);
+		ldap_value_free_len(cn);
 		ctmpa->valoffset  = indent;
 		ctmpa->keymenu    = km;
 		ctmpa->help       = help;
@@ -238,8 +238,8 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		if(need_mail && !have_mail)
 		  ctmpa->flags     |= CF_PRIVATE;
 
-		ctmpa->value      = cpystr(sn[0]);
-		ldap_value_free(sn);
+		ctmpa->value      = cpystr(sn[0]->bv_val);
+		ldap_value_free_len(sn);
 		ctmpa->valoffset  = indent;
 		ctmpa->keymenu    = km;
 		ctmpa->help       = help;
@@ -289,11 +289,11 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 	    }
 
 	    if(title){
-		for(i = 0; title[i] && title[i][0]; i++){
+		for(i = 0; ALPINE_LDAP_usable(title, i); i++){
 		    new_confline(&ctmpa);
 		    ctmpa->flags     |= CF_NOSELECT;
 		    ctmpa->valoffset  = indent + 2;
-		    ctmpa->value      = cpystr(title[i]);
+		    ctmpa->value      = cpystr(title[i]->bv_val);
 		    ctmpa->keymenu    = km;
 		    ctmpa->help       = help;
 		    ctmpa->help_title = _(srch_res_help_title);
@@ -305,15 +305,15 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		    ctmpa->d.a.ac     = ac;
 		}
 
-		ldap_value_free(title);
+		ldap_value_free_len(title);
 	    }
 
 	    if(unit){
-		for(i = 0; unit[i] && unit[i][0]; i++){
+		for(i = 0; ALPINE_LDAP_usable(unit, i); i++){
 		    new_confline(&ctmpa);
 		    ctmpa->flags     |= CF_NOSELECT;
 		    ctmpa->valoffset  = indent + 2;
-		    ctmpa->value      = cpystr(unit[i]);
+		    ctmpa->value      = cpystr(unit[i]->bv_val);
 		    ctmpa->keymenu    = km;
 		    ctmpa->help       = help;
 		    ctmpa->help_title = _(srch_res_help_title);
@@ -325,15 +325,15 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		    ctmpa->d.a.ac     = ac;
 		}
 
-		ldap_value_free(unit);
+		ldap_value_free_len(unit);
 	    }
 
 	    if(org){
-		for(i = 0; org[i] && org[i][0]; i++){
+		for(i = 0; ALPINE_LDAP_usable(org, i); i++){
 		    new_confline(&ctmpa);
 		    ctmpa->flags     |= CF_NOSELECT;
 		    ctmpa->valoffset  = indent + 2;
-		    ctmpa->value      = cpystr(org[i]);
+		    ctmpa->value      = cpystr(org[i]->bv_val);
 		    ctmpa->keymenu    = km;
 		    ctmpa->help       = help;
 		    ctmpa->help_title = _(srch_res_help_title);
@@ -345,20 +345,20 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		    ctmpa->d.a.ac     = ac;
 		}
 
-		ldap_value_free(org);
+		ldap_value_free_len(org);
 	    }
 
 	    if(have_mail){
 		/* Don't show long list of email addresses. */
-		if(!(mail[0] && mail[0][0]) ||
-		   !(mail[1] && mail[1][0]) ||
-		   !(mail[2] && mail[2][0]) ||
-		   !(mail[3] && mail[3][0])){
-		    for(i = 0; mail[i] && mail[i][0]; i++){
+		if(!ALPINE_LDAP_can_use_num(mail, 0) ||
+		   !ALPINE_LDAP_can_use_num(mail, 1) ||
+		   !ALPINE_LDAP_can_use_num(mail, 2) ||
+		   !ALPINE_LDAP_can_use_num(mail, 3)){
+		    for(i = 0; ALPINE_LDAP_usable(mail, i); i++){
 			new_confline(&ctmpa);
 			ctmpa->flags     |= CF_NOSELECT;
 			ctmpa->valoffset  = indent + 2;
-			ctmpa->value      = cpystr(mail[i]);
+			ctmpa->value      = cpystr(mail[i]->bv_val);
 			ctmpa->keymenu    = km;
 			ctmpa->help       = help;
 			ctmpa->help_title = _(srch_res_help_title);
@@ -373,7 +373,7 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 		else{
 		    char tmp[200];
 
-		    for(i = 4; mail[i] && mail[i][0]; i++)
+		    for(i = 4; ALPINE_LDAP_usable(mail, i); i++)
 		      ;
 		    
 		    new_confline(&ctmpa);
@@ -410,7 +410,7 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 	    }
 
 	    if(mail)
-	      ldap_value_free(mail);
+	      ldap_value_free_len(mail);
 
 	    new_confline(&ctmpa);		/* blank line */
 	    ctmpa->keymenu    = km;
@@ -471,7 +471,7 @@ ldap_addr_select(struct pine *ps, ADDR_CHOOSE_S *ac, LDAP_CHOOSE_S **result,
 	memset(&screen, 0, sizeof(screen));
 	/* TRANSLATORS: Print something1 using something2.
 	   "this" is something1 */
-	switch(conf_scroll_screen(ps,&screen,first_line,ac->title,_("this"),0)){
+	switch(conf_scroll_screen(ps,&screen,first_line,ac->title,_("this"),0, NULL)){
 	  case ADDR_SELECT_EXIT_VAL:
 	    retval = -1;
 	    break;
@@ -622,6 +622,7 @@ dir_init_display(struct pine *ps, CONF_S **ctmp, char **servers,
     int   i;
     char *serv;
     char *subtitle;
+    size_t sizeofsub;
     LDAP_SERV_S *info;
 
     if(first_line)
@@ -633,22 +634,21 @@ dir_init_display(struct pine *ps, CONF_S **ctmp, char **servers,
 	    serv = (info && info->nick && *info->nick) ? cpystr(info->nick) :
 		     (info && info->serv && *info->serv) ? cpystr(info->serv) :
 		       cpystr(_("Bad Server Config, Delete this"));
-	    subtitle = (char *)fs_get((((info && info->serv && *info->serv)
-					    ? strlen(info->serv)
-					    : 3) +
-					       strlen(_(dserv)) + 15) *
-					 sizeof(char));
+	    sizeofsub = (((info && info->serv && *info->serv)
+			   ? strlen(info->serv)
+			   : 3) + strlen(_(dserv)) + 15) * sizeof(char);
+	    subtitle = (char *)fs_get(sizeofsub);
 	    if(info && info->port >= 0)
-	      snprintf(subtitle, sizeof(subtitle), "%s%s:%d",
+	      snprintf(subtitle, sizeofsub, "%s%s:%d",
 		      _(dserv),
 		      (info && info->serv && *info->serv) ? info->serv : "<?>",
 		      info->port);
 	    else
-	      snprintf(subtitle, sizeof(subtitle), "%s%s",
+	      snprintf(subtitle, sizeofsub, "%s%s",
 		      _(dserv),
 		      (info && info->serv && *info->serv) ? info->serv : "<?>");
 
-	    subtitle[sizeof(subtitle)-1] = '\0';
+	    subtitle[sizeofsub-1] = '\0';
 
 	    add_ldap_server_to_display(ps, ctmp, serv, subtitle, var,
 				       i, &dir_conf_km, h_direct_config,
@@ -722,7 +722,7 @@ directory_config(struct pine *ps, int edit_exceptions)
     /* TRANSLATORS: Print something1 using something2.
        "servers" is something1 */
     (void)conf_scroll_screen(ps, &screen, first_line,
-			     _("SETUP DIRECTORY SERVERS"), _("servers"), 0);
+			     _("SETUP DIRECTORY SERVERS"), _("servers"), 0, NULL);
     ps->mangled_screen = 1;
 }
 
@@ -803,6 +803,7 @@ dir_config_add(struct pine *ps, CONF_S **cl)
 
 	if(info && info->serv && *info->serv){
 	    char  *subtitle;
+	    size_t sizeofsub;
 	    int    i, cnt = 0;
 	    char **new_list;
 	    CONF_S *cp;
@@ -837,22 +838,21 @@ dir_config_add(struct pine *ps, CONF_S **cl)
 	    set_variable_list(V_LDAP_SERVERS, new_list, FALSE, ew);
 	    free_list_array(&new_list);
 	    set_current_val((*cl)->var, TRUE, FALSE);
-	    subtitle = (char *)fs_get((((info && info->serv && *info->serv)
-					    ? strlen(info->serv)
-					    : 3) +
-					       strlen(_(dserv)) + 15) *
-					 sizeof(char));
+	    sizeofsub = (((info && info->serv && *info->serv)
+			    ? strlen(info->serv)
+			    : 3) + strlen(_(dserv)) + 15) * sizeof(char);
+	    subtitle = (char *)fs_get(sizeofsub);
 	    if(info && info->port >= 0)
-	      snprintf(subtitle, sizeof(subtitle), "%s%s:%d",
+	      snprintf(subtitle, sizeofsub, "%s%s:%d",
 		      _(dserv),
 		      (info && info->serv && *info->serv) ? info->serv : "<?>",
 		      info->port);
 	    else
-	      snprintf(subtitle, sizeof(subtitle), "%s%s",
+	      snprintf(subtitle, sizeofsub, "%s%s",
 		      _(dserv),
 		      (info && info->serv && *info->serv) ? info->serv : "<?>");
 
-	    subtitle[sizeof(subtitle)-1] = '\0';
+	    subtitle[sizeofsub-1] = '\0';
 
 	    if(cnt < 2){			/* first one */
 		struct variable *var;
@@ -1080,6 +1080,7 @@ dir_config_edit(struct pine *ps, CONF_S **cl)
 	    char  *subtitle;
 	    int    i, cnt;
 	    char **new_list;
+	    size_t sizeofsub;
 
 	    for(cnt = 0; lval && lval[cnt]; cnt++)
 	      ;
@@ -1109,22 +1110,22 @@ dir_config_edit(struct pine *ps, CONF_S **cl)
 	    if((*cl)->next->value)
 	      fs_give((void **)&(*cl)->next->value);
 
-	    subtitle = (char *)fs_get((((info && info->serv && *info->serv)
-					    ? strlen(info->serv)
-					    : 3) +
-					       strlen(_(dserv)) + 15) *
-					 sizeof(char));
+	    sizeofsub = (((info && info->serv && *info->serv)
+			    ? strlen(info->serv)
+			    : 3) + strlen(_(dserv)) + 15) * sizeof(char);
+
+	    subtitle = (char *)fs_get(sizeofsub);
 	    if(info && info->port >= 0)
-	      snprintf(subtitle, sizeof(subtitle), "%s%s:%d",
+	      snprintf(subtitle, sizeofsub, "%s%s:%d",
 		      _(dserv),
 		      (info && info->serv && *info->serv) ? info->serv : "<?>",
 		      info->port);
 	    else
-	      snprintf(subtitle, sizeof(subtitle), "%s%s",
+	      snprintf(subtitle, sizeofsub, "%s%s",
 		      _(dserv),
 		      (info && info->serv && *info->serv) ? info->serv : "<?>");
 
-	    subtitle[sizeof(subtitle)-1] = '\0';
+	    subtitle[sizeofsub] = '\0';
 
 	    (*cl)->next->value = subtitle;
 
@@ -1145,6 +1146,7 @@ dir_config_edit(struct pine *ps, CONF_S **cl)
 #define   LDAP_F_NOSUB 3
 #define   LDAP_F_TLS   4
 #define   LDAP_F_TLSMUST 5
+#define   LDAP_F_LDAPS 6
 bitmap_t  ldap_option_list;
 struct variable *ldap_srch_rule_ptr;
 
@@ -1351,6 +1353,8 @@ dir_edit_screen(struct pine *ps, LDAP_SERV_S *def, char *title, char **raw_serve
       setbitn(LDAP_F_TLS, ldap_option_list);
     if(def && def->tlsmust)
       setbitn(LDAP_F_TLSMUST, ldap_option_list);
+    if(def && def->ldaps)
+      setbitn(LDAP_F_LDAPS, ldap_option_list);
 
     /* save the old opt_screen before calling scroll screen again */
     saved_screen = opt_screen;
@@ -1474,6 +1478,38 @@ dir_edit_screen(struct pine *ps, LDAP_SERV_S *def, char *title, char **raw_serve
     
     lv = MIN(lv, 100);
 
+    /* enabling ldaps disables tls */
+    if((f = ldap_feature_list(LDAP_F_LDAPS)) != NULL
+	&& bitnset(f->value, ldap_option_list)){
+	int clear = 0;
+	if((f = ldap_feature_list(LDAP_F_TLS)) != NULL
+	   && bitnset(f->value, ldap_option_list)){
+	      clear++;
+	      clrbitn(f->value, ldap_option_list);
+	   }
+	if((f = ldap_feature_list(LDAP_F_TLSMUST)) != NULL
+	   && bitnset(f->value, ldap_option_list)){
+	      clear++;
+	      clrbitn(f->value, ldap_option_list);
+	}
+	if(clear > 0)
+	   q_status_message(SM_ORDER, 3, 3,
+			   _("Can not use TLS when connecting using LDAPS"));
+    }
+
+    /* enabling tls disables ldaps */
+    if(((f = ldap_feature_list(LDAP_F_TLS)) != NULL
+	&& bitnset(f->value, ldap_option_list))
+	|| ((f = ldap_feature_list(LDAP_F_TLSMUST)) != NULL
+	   && bitnset(f->value, ldap_option_list))){
+	if((f = ldap_feature_list(LDAP_F_LDAPS)) != NULL
+	   && bitnset(f->value, ldap_option_list)){
+	   clrbitn(f->value, ldap_option_list);
+	   q_status_message(SM_ORDER, 3, 3,
+			   _("Can not use LDAPS when connecting using TLS"));
+	}
+    }
+
     for(i = 0; (f = ldap_feature_list(i)); i++){
 	new_confline(&ctmp);
 	ctmp->var       = &opt_var;
@@ -1499,6 +1535,9 @@ dir_edit_screen(struct pine *ps, LDAP_SERV_S *def, char *title, char **raw_serve
 	  case LDAP_F_TLSMUST:
 	    ctmp->help      = h_config_ldap_opts_tlsmust;
 	    break;
+	  case LDAP_F_LDAPS:
+	    ctmp->help      = h_config_ldap_opts_ldaps;
+	    break;
 	}
 
 	ctmp->tool      = ldap_checkbox_tool;
@@ -1758,7 +1797,7 @@ dir_edit_screen(struct pine *ps, LDAP_SERV_S *def, char *title, char **raw_serve
     screen.ro_warning = saved_screen ? saved_screen->deferred_ro_warning : 0;
     /* TRANSLATORS: Print something1 using something2.
        servers is something1 */
-    rv = conf_scroll_screen(ps, &screen, first_line, tmp, _("servers"), 0);
+    rv = conf_scroll_screen(ps, &screen, first_line, tmp, _("servers"), 0, NULL);
 
     /*
      * Now look at the fake variables and extract the information we
@@ -1919,7 +1958,7 @@ dir_edit_screen(struct pine *ps, LDAP_SERV_S *def, char *title, char **raw_serve
 	    }
 	}
 
-	snprintf(dir_tmp, sizeof(dir_tmp), "%s%s%s \"/base=%s/binddn=%s/impl=%d/rhs=%d/ref=%d/nosub=%d/tls=%d/tlsm=%d/type=%s/srch=%s%s/time=%s/size=%s/cust=%s/nick=%s/matr=%s/catr=%s/satr=%s/gatr=%s\"",
+	snprintf(dir_tmp, sizeof(dir_tmp), "%s%s%s \"/base=%s/binddn=%s/impl=%d/rhs=%d/ref=%d/nosub=%d/tls=%d/tlsm=%d/ldaps=%d/type=%s/srch=%s%s/time=%s/size=%s/cust=%s/nick=%s/matr=%s/catr=%s/satr=%s/gatr=%s\"",
 		server ? server : "",
 		(portval >= 0 && port && *port) ? ":" : "",
 		(portval >= 0 && port && *port) ? port : "",
@@ -1931,6 +1970,7 @@ dir_edit_screen(struct pine *ps, LDAP_SERV_S *def, char *title, char **raw_serve
 		bitnset(LDAP_F_NOSUB, ldap_option_list) ? 1 : 0,
 		bitnset(LDAP_F_TLS, ldap_option_list) ? 1 : 0,
 		bitnset(LDAP_F_TLSMUST, ldap_option_list) ? 1 : 0,
+		bitnset(LDAP_F_LDAPS, ldap_option_list) ? 1 : 0,
 		srch_type ? srch_type : "",
 		srch_rule ? srch_rule : "",
 		custom_scope,
@@ -2341,6 +2381,21 @@ void
 toggle_ldap_option_bit(struct pine *ps, int index, struct variable *var, char *value)
 {
     NAMEVAL_S  *f;
+    int tls_is_on_now;
+    int tlsmust_is_on_now;
+    int ldaps_is_on_now;
+    int tls_is_on_after;
+    int tlsmust_is_on_after;
+    int ldaps_is_on_after;
+
+    tls_is_on_now = (f = ldap_feature_list(LDAP_F_TLS)) != NULL
+	   && bitnset(f->value, ldap_option_list) ? 1 : 0;
+
+    tlsmust_is_on_now = (f = ldap_feature_list(LDAP_F_TLSMUST)) != NULL
+	   && bitnset(f->value, ldap_option_list) ? 1 : 0;
+
+    ldaps_is_on_now = (f = ldap_feature_list(LDAP_F_LDAPS)) != NULL
+	&& bitnset(f->value, ldap_option_list) ? 1 : 0;
 
     f  = ldap_feature_list(index);
 
@@ -2350,6 +2405,46 @@ toggle_ldap_option_bit(struct pine *ps, int index, struct variable *var, char *v
     else
       setbitn(f->value, ldap_option_list);
 
+    tls_is_on_after = (f = ldap_feature_list(LDAP_F_TLS)) != NULL
+	   && bitnset(f->value, ldap_option_list) ? 1 : 0;
+
+    tlsmust_is_on_after = (f = ldap_feature_list(LDAP_F_TLSMUST)) != NULL
+	   && bitnset(f->value, ldap_option_list) ? 1 : 0;
+
+    ldaps_is_on_after = (f = ldap_feature_list(LDAP_F_LDAPS)) != NULL
+	&& bitnset(f->value, ldap_option_list) ? 1 : 0;
+
+    f  = ldap_feature_list(index);
+
+    if(!ldaps_is_on_now && ldaps_is_on_after){
+	if(tlsmust_is_on_after || tls_is_on_after){
+	   char *name;
+	   if(tlsmust_is_on_after)
+	      name = ldap_feature_list(LDAP_F_TLSMUST)->name;
+	   else
+	      name = ldap_feature_list(LDAP_F_TLS)->name;
+ 	   clrbitn(f->value, ldap_option_list);
+	   q_status_message1(SM_ORDER, 3, 3, 
+		_("Can not use LDAPS when using TLS. Disable \"%s\" first."), name);
+	}
+    }
+    else if(!tls_is_on_now && tls_is_on_after){
+	if(ldaps_is_on_after){
+	   char *name = ldap_feature_list(LDAP_F_LDAPS)->name;
+ 	   clrbitn(f->value, ldap_option_list);	   
+	   q_status_message1(SM_ORDER, 3, 3, 
+		_("Can not use TLS when using LDAPS. Disable \"%s\" first."), name);
+	}
+    }
+    else if(!tlsmust_is_on_now && tlsmust_is_on_after){
+	if(ldaps_is_on_after){
+	   char *name = ldap_feature_list(LDAP_F_LDAPS)->name;
+ 	   clrbitn(f->value, ldap_option_list);
+	   q_status_message1(SM_ORDER, 3, 3, 
+		_("Can not use TLS when using LDAPS. Disable \"%s\" first."), name);
+	}
+    }
+
     if(value)
       value[1] = bitnset(f->value, ldap_option_list) ? 'X' : ' ';
 }
@@ -2364,7 +2459,8 @@ ldap_feature_list(int index)
 	{"save-search-criteria-not-result",   NULL, LDAP_F_REF},
 	{"disable-ad-hoc-space-substitution", NULL, LDAP_F_NOSUB},
 	{"attempt-tls-on-connection",         NULL, LDAP_F_TLS},
-	{"require-tls-on-connection",         NULL, LDAP_F_TLSMUST}
+	{"require-tls-on-connection",         NULL, LDAP_F_TLSMUST},
+	{"require-ldaps-on-connection",     NULL, LDAP_F_LDAPS}
     };
 
     return((index >= 0 &&
