@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: mailindx.c 1142 2008-08-13 17:22:21Z hubert@u.washin
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -322,6 +322,7 @@ index_lister(struct pine *state, CONTEXT_S *cntxt, char *folder, MAILSTREAM *str
 		 cur_row, cur_col, km_popped, paint_status;
     static int   old_day = -1;
     long	 i, j, k, old_max_msgno;
+    long	 lflagged, old_lflagged = 0L;
     char        *utf8str;
     IndexType    style, old_style = MsgIndex;
     struct index_state id;
@@ -422,10 +423,12 @@ index_lister(struct pine *state, CONTEXT_S *cntxt, char *folder, MAILSTREAM *str
 		  : (any_lflagged(msgmap, MN_HIDE))
 		    ? ZoomIndex
 		    : (mn_total_cur(msgmap) > 1L) ? MultiMsgIndex : MsgIndex;
-	if(style != old_style){
+        lflagged = any_lflagged(msgmap, MN_HIDE);
+	if(style != old_style || lflagged < old_lflagged){
             state->mangled_header = 1;
             state->mangled_footer = 1;
 	    old_style = style;
+	    old_lflagged = lflagged;
 	    if(!(style == ThreadIndex || old_style == ThreadIndex))
 	      id.msg_at_top = 0L;
 	}
@@ -795,6 +798,21 @@ view_a_thread:
 
 
 	  case MC_THRDINDX :
+	    if(any_lflagged(msgmap, MN_SLCT)){
+		PINETHRD_S *thrd, *topthrd;
+		for(i = 1L; i > 0L && i <= mn_get_total(msgmap);){
+		    thrd = fetch_thread(stream, i);
+		    if(thrd && thrd->top)
+		      topthrd = fetch_thread(stream, thrd->top);
+		    if(topthrd){
+		       set_thread_lflags(stream, topthrd, msgmap, MN_CHID, 1);
+		       set_thread_lflags(stream, topthrd, msgmap, MN_CHID2, 0);
+		       set_lflag(stream, msgmap, mn_raw2m(msgmap, topthrd->rawno), MN_CHID, 0);
+		       set_lflag(stream, msgmap, mn_raw2m(msgmap, topthrd->rawno), MN_COLL, 1);
+		    }
+		    i = thrd->nextthd;
+		}
+	    }
 	    msgmap->top = msgmap->top_after_thrd;
 	    if(unview_thread(state, stream, msgmap)){
 		state->next_screen = mail_index_screen;
@@ -2141,6 +2159,7 @@ setup_index_state(int threaded)
 		    col++;
 
 		  if((cdesc->ctype == iSubject
+		      || cdesc->ctype == iShortSubject
 		      || cdesc->ctype == iSubjectText
 		      || cdesc->ctype == iSubjKey
 		      || cdesc->ctype == iSubjKeyText
