@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: debuging.c 900 2008-01-05 01:13:26Z hubert@u.washing
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -276,7 +276,7 @@ do_debug(FILE *debug_fp)
     }
 
     if(ok && ps_global->debug_timestamp)
-      fprintf(debug_fp, "\n%s\n", debug_time(0, ps_global->debug_timestamp));
+      fprintf(debug_fp, "\n%s\n", debug_time(0, ps_global->debug_timestamp, ps_global->signal_in_progress));
 
     return(ok);
 }
@@ -397,13 +397,13 @@ dump_config(struct pine *ps, gf_io_t pc, int brief)
 		       (i == 3) ? vars->post_user_val.l :
 			(i == 4) ? vars->global_val.l
 				 : vars->fixed_val.l;
-		if(t && *t){
+		if(t && *t){	/* MAILTMPLEN = sizeof(tmp) */
 		    snprintf(tmp, sizeof(tmp), " %20.20s : %.*s\n", vars->name,
-			    sizeof(tmp)-26, **t ? *t : quotes);
+			    MAILTMPLEN-26, **t ? *t : quotes);
 		    gf_puts(tmp, pc);
 		    while(++t && *t){
 			snprintf(tmp, sizeof(tmp)," %20.20s : %.*s\n","",
-				sizeof(tmp)-26, **t ? *t : quotes);
+				MAILTMPLEN-26, **t ? *t : quotes);
 			gf_puts(tmp, pc);
 		    }
 		}
@@ -416,9 +416,9 @@ dump_config(struct pine *ps, gf_io_t pc, int brief)
 		       (i == 3) ? vars->post_user_val.p :
 		        (i == 4) ? vars->global_val.p
 				 : vars->fixed_val.p;
-		if(t){
+		if(t){		/* MAILTMPLEN = sizeof(tmp) */
 		    snprintf(tmp, sizeof(tmp), " %20.20s : %.*s\n", vars->name,
-			    sizeof(tmp)-26, *t ? t : quotes);
+			    MAILTMPLEN-26, *t ? t : quotes);
 		    gf_puts(tmp, pc);
 		}
 	    }
