@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: execview.c 942 2008-03-04 18:21:33Z hubert@u.washing
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -227,7 +227,7 @@ exec_mailcap_cmd(MCAP_CMD_S *mc_cmd, char *image_file, int needsterminal)
 	    r_file_h    = &result_file;
 	}
 
-	if(syspipe = open_system_pipe(command, r_file_h, NULL, mode, 0, pipe_callback, NULL)){
+	if((syspipe = open_system_pipe(command, r_file_h, NULL, mode, 0, pipe_callback, NULL)) != NULL){
 	    close_system_pipe(&syspipe, NULL, pipe_callback);
 	    if(needsterminal == 1)
 	      q_status_message(SM_ORDER, 0, 4, "VIEWER command completed");
@@ -269,11 +269,11 @@ exec_mailcap_cmd(MCAP_CMD_S *mc_cmd, char *image_file, int needsterminal)
     q = cpystr("");
 #endif /* PSEFCMD */
 
-    l = 32 + strlen(cmd) + 2*strlen(image_file) + strlen(q);
+    l = 45 + strlen(cmd) + 2*strlen(image_file) + strlen(q);
     p = command = (char *)fs_get((l+1) * sizeof(char));
     if(!needsterminal)  /* put in background if it doesn't need terminal */
       *p++ = '(';
-    snprintf(p, l+1-(p-command), "%s ; %s rm -f %s", cmd, q, image_file);
+    snprintf(p, l+1-(p-command), "%s ; %s sleep 5 ; rm -f %s", cmd, q, image_file);
     fs_give((void **)&q);
     command[l] = '\0';
     p += strlen(p);
@@ -349,10 +349,9 @@ url_os_specified_browser(char *url)
     if(mime_os_specific_access()){
 	return(cpystr("open"));
     }
-#else
+#endif
     /* do nothing here */
     return(NULL);
-#endif
 }
 
 /*
