@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: termin.unx.c 1025 2008-04-08 22:59:38Z hubert@u.wash
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -732,8 +732,10 @@ pre_screen_config_opt_enter(char *string, int string_size, char *prompt,
 		    return_v = 0;
 		}
 		else{
-		    fputs("Password too long\n", stderr);
-		    return_v = -1;
+		    if(fputs("Password too long\n", stderr) != EOF)
+		       return_v = -1;
+		    else
+		       alpine_panic(_("error on fputs() call!"));
 		}
 	    }
 	    else
@@ -742,13 +744,15 @@ pre_screen_config_opt_enter(char *string, int string_size, char *prompt,
 	else{
 	    char *p;
 
-	    fputs(prompt, stdout);
-	    fgets(string, string_size, stdin);
-	    string[string_size-1] = '\0';
-	    if((p = strpbrk(string, "\r\n")) != NULL)
-	      *p = '\0';
-
-	    return_v = 0;
+	    if(fputs(prompt, stdout) != EOF
+		&& fgets(string, string_size, stdin) != NULL){
+		string[string_size-1] = '\0';
+		if((p = strpbrk(string, "\r\n")) != NULL)
+		  *p = '\0';
+		return_v = 0;
+	    }
+	    else
+		alpine_panic(_("error on fputs() or fgets() call!"));
 	}
     }
 
