@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: print.c 769 2007-10-24 00:15:40Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -497,7 +497,7 @@ select_printer(struct pine *ps, int edit_exceptions)
     switch(conf_scroll_screen(ps, &screen, start_line,
 			      edit_exceptions ? _("SETUP PRINTER EXCEPTIONS")
 					      : _("SETUP PRINTER"),
-			      _("printer config"), 0)){
+			      _("printer config"), 0, NULL)){
       case 0:
 	break;
     
