@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: radio.c 769 2007-10-24 00:15:40Z hubert@u.washington
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -73,8 +73,9 @@ pre_screen_config_want_to(char *question, int dflt, int on_ctrl_C)
     }
 #endif
     while(!ret){
-      fprintf(stdout, "%s? [%c]:", question, dflt);
-      fgets(rep, WANT_TO_BUF, stdin);
+      if(fprintf(stdout, "%s? [%c]:", question, dflt) < 0
+	  || fgets(rep, WANT_TO_BUF, stdin) == NULL)
+	alpine_panic(_("error on fprintf() or fgets()"));
       if((p = strpbrk(rep, "\r\n")) != NULL)
 	*p = '\0';
       switch(*rep){
