@@ -2,6 +2,7 @@
  * $Id: reply.h 761 2007-10-23 22:35:18Z hubert@u.washington.edu $
  *
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -28,7 +29,7 @@ int	    reply(struct pine *, ACTION_S *);
 int	    confirm_role(long, ACTION_S **);
 int	    reply_to_all_query(int *);
 int	    reply_using_replyto_query(void);
-int	    reply_text_query(struct pine *, long, char **);
+int	    reply_text_query(struct pine *, long, ENVELOPE *, char **);
 int	    reply_news_test(ENVELOPE *, ENVELOPE *);
 char       *get_signature_file(char *, int, int, int);
 int	    forward(struct pine *, ACTION_S *);
