@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: signal.c 1025 2008-04-08 22:59:38Z hubert@u.washingt
 
 /* ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -238,6 +238,8 @@ Not much to do. Rely on periodic mail file check pointing.
 RETSIGTYPE
 hup_signal(void)
 {
+    if(ps_global)
+      ps_global->signal_in_progress = 1;
     end_signals(1);			/* don't catch any more signals */
     dprint((1, "\n\n** Received SIGHUP **\n\n\n\n"));
     cleanup_called_from_sig_handler = 1;
