@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: smime.c 1074 2008-06-04 00:08:43Z hubert@u.washingto
 
 /*
  * ========================================================================
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -56,12 +56,16 @@ void     revert_to_saved_smime_config(struct pine *ps, SAVED_CONFIG_S *vsave);
 SAVED_CONFIG_S *save_smime_config_vars(struct pine *ps);
 void     free_saved_smime_config(struct pine *ps, SAVED_CONFIG_S **vsavep);
 int      smime_helper_tool(struct pine *, int, CONF_S **, unsigned);
-//int      smime_public_certs_tool(struct pine *, int, CONF_S **, unsigned);
 void 	 manage_certificates(struct pine *, WhichCerts);
-void     smime_manage_certs_init (struct pine *, CONF_S **, CONF_S **, WhichCerts, int);
-void	 display_certificate_information(struct pine *, X509 *, char *, WhichCerts, int num);
-int	 manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags);
-int	 manage_certificate_info_tool(int, MSGNO_S *, SCROLL_S *);
+#ifdef PASSFILE
+void 	 manage_password_file_certificates(struct pine *);
+#endif /* PASSFILE */
+void smime_manage_certs_init (struct pine *, CONF_S **, CONF_S **, WhichCerts, int);
+void smime_manage_password_file_certs_init(struct pine *, CONF_S **, CONF_S **, int, int *);
+void display_certificate_information(struct pine *, X509 *, char *, WhichCerts, int num);
+int  manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags);
+int  manage_certificate_info_tool(int, MSGNO_S *, SCROLL_S *);
+void smime_setup_size(char **, size_t, size_t);
 
 
 /*
@@ -97,40 +101,40 @@ smime_get_passphrase(void)
     return rc;	/* better return rc and make the caller check its return value */
 }
 
+int
+smime_check(BODY *body)
+{
+  int rv = 0;
+  PKCS7 *p7 = NULL;
+
+  if(body->type == TYPEMULTIPART){
+	PART *p;
+        
+	for(p=body->nested.part; p && rv == 0; p=p->next)
+	  rv += smime_check(&p->body);
+  }
+  if(rv > 0) return rv;
+  if(body->sparep)
+	p7 = get_smime_sparep_type(body->sparep) == P7Type
+		? (PKCS7 *)get_smime_sparep_data(body->sparep)
+		: NULL;
+  if(p7 && (PKCS7_type_is_signed(p7) || PKCS7_type_is_enveloped(p7)))
+    rv += 1;
+  return rv;
+}
+
 
 void
-smime_info_screen(struct pine *ps)
-{
-    long      msgno;
-    OtherMenu what;
-    int       offset = 0;
-    BODY     *body;
-    ENVELOPE *env;
+display_smime_info(struct pine *ps, ENVELOPE *env, BODY *body)
+{    
+    OtherMenu what = FirstMenu;
     HANDLE_S *handles = NULL;
     SCROLL_S  scrollargs;
     STORE_S  *store = NULL;
-    
-    ps->prev_screen = smime_info_screen;
-    ps->next_screen = SCREEN_FUN_NULL;
-
-    if(mn_total_cur(ps->msgmap) > 1L){
-	q_status_message(SM_ORDER | SM_DING, 0, 3,
-			 _("Can only view one message's information at a time."));
-	return;
-    }
-    /* else check for existence of smime bits */
+    long  msgno;
+    int	  offset = 0;
 
     msgno = mn_m2raw(ps->msgmap, mn_get_cur(ps->msgmap));
-    
-    env = mail_fetch_structure(ps->mail_stream, msgno, &body, 0);
-    if(!env || !body){
-	q_status_message(SM_ORDER, 0, 3,
-			 _("Can't fetch body of message."));
-	return;
-    }
-    
-    what = FirstMenu;
-
     store = so_get(CharStar, NULL, EDIT_ACCESS);
 
     while(ps->next_screen == SCREEN_FUN_NULL){
@@ -183,6 +187,41 @@ smime_info_screen(struct pine *ps)
     so_give(&store);
 }
 
+void
+smime_info_screen(struct pine *ps)
+{
+    long      msgno;
+    BODY     *body;
+    ENVELOPE *env;
+    
+/*    ps->prev_screen = smime_info_screen;
+    ps->next_screen = SCREEN_FUN_NULL; */
+
+    msgno = mn_m2raw(ps->msgmap, mn_get_cur(ps->msgmap));
+    
+    env = mail_fetch_structure(ps->mail_stream, msgno, &body, 0);
+
+    if(!env || !body){
+	q_status_message(SM_ORDER, 0, 3,
+			 _("Can't fetch body of message."));
+	return;
+    }
+
+    if(smime_check(body) == 0){
+      q_status_message(SM_ORDER | SM_DING, 0, 3,
+			 _("Not a signed or encrypted message"));
+      return;
+    }
+
+    if(mn_total_cur(ps->msgmap) > 1L){
+	q_status_message(SM_ORDER | SM_DING, 0, 3,
+			 _("Can only view one message's information at a time."));
+	return;
+    }
+
+    display_smime_info(ps, env, body);
+}
+
 
 void
 format_smime_info(int pass, BODY *body, long msgno, gf_io_t pc)
@@ -342,93 +381,79 @@ output_cert_info(X509 *cert, gf_io_t pc)
 
     gf_set_so_writec(&spc, left);
 
-    if(!cert->cert_info){
-    	gf_puts("Couldn't find certificate info.", spc);
-	gf_puts(NEWLINE, spc);
+    gf_puts_uline("Certificate Owner", spc);
+    gf_puts(NEWLINE, spc);
+
+    output_X509_NAME(X509_get_subject_name(cert), spc);
+    gf_puts(NEWLINE, spc);
+
+    gf_puts_uline("Serial Number", spc);
+    gf_puts(NEWLINE, spc);
+
+    {   ASN1_INTEGER *bs;
+	long l;
+	const char *neg;
+	int i;
+
+	bs = X509_get_serialNumber(cert);
+	if (bs->length <= (int)sizeof(long)){
+	   l = ASN1_INTEGER_get(bs);
+           if (bs->type == V_ASN1_NEG_INTEGER){
+	      l = -l;
+	      neg="-";
+	   }
+           else
+              neg="";
+	   snprintf(buf, sizeof(buf), " %s%lu (%s0x%lx)", neg, l, neg, l);
+	} else {
+	    snprintf(buf, sizeof(buf), "%s", bs->type == V_ASN1_NEG_INTEGER ? "(Negative)" : "");
+	    for (i = 0; i < bs->length; i++)
+		 snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%02x%s", bs->data[i],
+			i+1 == bs->length ? "" : ":");
+	}
     }
-    else{
-	gf_puts_uline("Subject (whose certificate it is)", spc);
-	gf_puts(NEWLINE, spc);
+    gf_puts(buf, spc);
+    gf_puts(NEWLINE, spc);
+    gf_puts(NEWLINE, spc);
 
-	output_X509_NAME(cert->cert_info->subject, spc);
-	gf_puts(NEWLINE, spc);
-
-	gf_puts_uline("Serial Number", spc);
-	gf_puts(NEWLINE, spc);
-
-	{ 
-	  ASN1_INTEGER *bs;
-	  long l;
-	  const char *neg;
-	  int i;
-
-	  bs = X509_get_serialNumber(cert);
-	  if (bs->length <= (int)sizeof(long)){
-	     l = ASN1_INTEGER_get(bs);
-             if (bs->type == V_ASN1_NEG_INTEGER){
-	        l = -l;
-	        neg="-";
-	     }
-             else
-                neg="";
-	     snprintf(buf, sizeof(buf), " %s%lu (%s0x%lx)", neg, l, neg, l);
-	  } else {
-	      snprintf(buf, sizeof(buf), "%s", bs->type == V_ASN1_NEG_INTEGER ? "(Negative)" : "");
-	      for (i = 0; i < bs->length; i++)
-		  snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%02x%s", bs->data[i],
-				i+1 == bs->length ? "" : ":");
-	  }
-	}
-	gf_puts(buf, spc);
-	gf_puts(NEWLINE, spc);
-	gf_puts(NEWLINE, spc);
-
-	gf_puts_uline("Validity", spc);
-	gf_puts(NEWLINE, spc);
-    	{
-    	    BIO *mb = BIO_new(BIO_s_mem());
-	    char iobuf[4096];
+    gf_puts_uline("Validity", spc);
+    gf_puts(NEWLINE, spc);
+    { BIO *mb = BIO_new(BIO_s_mem());
+      char iobuf[4096];
 	    
-	    gf_puts("Not Before: ", spc);
+      gf_puts("Not Before: ", spc);
 
-	    (void) BIO_reset(mb);
-	    ASN1_UTCTIME_print(mb, cert->cert_info->validity->notBefore);
-	    (void) BIO_flush(mb);
-	    while((len = BIO_read(mb, iobuf, sizeof(iobuf))) > 0)
-	      gf_nputs(iobuf, len, spc);
+      (void) BIO_reset(mb);
+      ASN1_UTCTIME_print(mb, X509_get0_notBefore(cert));
+      (void) BIO_flush(mb);
+      while((len = BIO_read(mb, iobuf, sizeof(iobuf))) > 0)
+       gf_nputs(iobuf, len, spc);
 
-	    gf_puts(NEWLINE, spc);
+      gf_puts(NEWLINE, spc);
 
-	    gf_puts("Not After:  ", spc);
+      gf_puts("Not After:  ", spc);
 
-	    (void) BIO_reset(mb);
-	    ASN1_UTCTIME_print(mb, cert->cert_info->validity->notAfter);
-	    (void) BIO_flush(mb);
-	    while((len = BIO_read(mb, iobuf, sizeof(iobuf))) > 0)
-	      gf_nputs(iobuf, len, spc);
+      (void) BIO_reset(mb);
+      ASN1_UTCTIME_print(mb, X509_get0_notAfter(cert));
+      (void) BIO_flush(mb);
+      while((len = BIO_read(mb, iobuf, sizeof(iobuf))) > 0)
+        gf_nputs(iobuf, len, spc);
     	    
-	    gf_puts(NEWLINE, spc);
-	    gf_puts(NEWLINE, spc);
+      gf_puts(NEWLINE, spc);
+      gf_puts(NEWLINE, spc);
 	    	    
-	    BIO_free(mb);
-	}
+      BIO_free(mb);
     }
 
     gf_clear_so_writec(left);
 
     gf_set_so_writec(&spc, right);
 
-    if(!cert->cert_info){
-    	gf_puts(_("Couldn't find certificate info."), spc);
-	gf_puts(NEWLINE, spc);
-    }
-    else{
-	gf_puts_uline("Issuer", spc);
-	gf_puts(NEWLINE, spc);
+    gf_puts_uline("Issuer", spc);
+    gf_puts(NEWLINE, spc);
 
-	output_X509_NAME(cert->cert_info->issuer, spc);
-	gf_puts(NEWLINE, spc);
-    }
+    output_X509_NAME(X509_get_issuer_name(cert), spc);
+    gf_puts(NEWLINE, spc);
     
     gf_clear_so_writec(right);
     
@@ -455,6 +480,7 @@ output_cert_info(X509 *cert, gf_io_t pc)
        X509_NAME_ENTRY *e;
        int i, offset = 2;
        char space[256];
+       X509_NAME *subject;
 
        for(i = 0; i < offset; i++) space[i] = ' ';
 
@@ -463,7 +489,7 @@ output_cert_info(X509 *cert, gf_io_t pc)
 
 	  x = i == -1 ? cert : sk_X509_value(chain, i);
 
-	  if(x && x->cert_info){
+	  if(x){
 	    if(i>=0){ 
 	      space[offset + i + 0] = ' ';
 	      space[offset + i + 1] = '\\';
@@ -481,11 +507,10 @@ output_cert_info(X509 *cert, gf_io_t pc)
 	    else
 	      gf_puts_uline("Issued to: ", pc);
 
-	    e = X509_NAME_get_entry(x->cert_info->subject,
-			X509_NAME_entry_count(x->cert_info->subject)-1);
+	    subject = X509_get_subject_name(x);
 
-	    if(e){
-	      X509_NAME_get_text_by_OBJ(x->cert_info->subject, e->object, buf, sizeof(buf));
+	    if((e = X509_NAME_get_entry(subject, X509_NAME_entry_count(subject)-1)) != NULL){
+	      X509_NAME_get_text_by_OBJ(subject, X509_NAME_ENTRY_get_object(e), buf, sizeof(buf));
 	      gf_puts(buf, pc);
 	      gf_puts(NEWLINE, pc);    
 	    }
@@ -496,10 +521,10 @@ output_cert_info(X509 *cert, gf_io_t pc)
 	    break;
 	  }
        }
-       e = X509_NAME_get_entry(x->cert_info->issuer,
-			X509_NAME_entry_count(x->cert_info->issuer)-1);
+       e = X509_NAME_get_entry(X509_get_issuer_name(x),
+			X509_NAME_entry_count(X509_get_issuer_name(x))-1);
        if(e){
-	  X509_NAME_get_text_by_OBJ(x->cert_info->issuer, e->object, buf, sizeof(buf));
+	  X509_NAME_get_text_by_OBJ(X509_get_issuer_name(x), X509_NAME_ENTRY_get_object(e), buf, sizeof(buf));
 	  space[offset + i + 0] = ' ';
 	  space[offset + i + 1] = '\\';
 	  space[offset + i + 2] = '-';
@@ -534,7 +559,7 @@ output_X509_NAME(X509_NAME *name, gf_io_t pc)
 	if(!e)
 	  continue;
 	
-    	X509_NAME_get_text_by_OBJ(name, e->object, buf, sizeof(buf));
+    	X509_NAME_get_text_by_OBJ(name, X509_NAME_ENTRY_get_object(e), buf, sizeof(buf));
 	
     	gf_puts(buf, pc);
 	gf_puts(NEWLINE, pc);    
@@ -691,7 +716,7 @@ smime_config_screen(struct pine *ps, int edit_exceptions)
 					      : _("SETUP S/MIME"),
 			      /* TRANSLATORS: Print something1 using something2.
 				 configuration is something1 */
-			      _("configuration"), 0)){
+			      _("configuration"), 0, NULL)){
       case 0:
 	break;
 
@@ -992,6 +1017,7 @@ smime_config_init_display(struct pine *ps, CONF_S **ctmp, CONF_S **first_line)
     (*ctmp)->help           = h_config_smime_public_certificates;
     (*ctmp)->value          = cpystr(_("Manage Public Certificates"));
     (*ctmp)->varmem         = 9;
+    (*ctmp)->d.s.ctype	    = Public;
 
     /* manage private keys */
     new_confline(ctmp);
@@ -1000,6 +1026,7 @@ smime_config_init_display(struct pine *ps, CONF_S **ctmp, CONF_S **first_line)
     (*ctmp)->help           = h_config_smime_private_keys;
     (*ctmp)->value          = cpystr(_("Manage Private Keys"));
     (*ctmp)->varmem         = 10;
+    (*ctmp)->d.s.ctype	    = Private;
 
     /* manage Certificate Authorities */
     new_confline(ctmp);
@@ -1008,6 +1035,36 @@ smime_config_init_display(struct pine *ps, CONF_S **ctmp, CONF_S **first_line)
     (*ctmp)->help           = h_config_smime_certificate_authorities;
     (*ctmp)->value          = cpystr(_("Manage Certificate Authorities"));
     (*ctmp)->varmem         = 11;
+    (*ctmp)->d.s.ctype	    = CACert;
+
+#ifdef PASSFILE
+    new_confline(ctmp)->var = vtmp;
+    (*ctmp)->flags |= CF_NOSELECT | CF_B_LINE;
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT;
+    (*ctmp)->value = cpystr(tmp);
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT;
+    (*ctmp)->value = cpystr(_("Manage Key and Certificate for Password File"));
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT;
+    (*ctmp)->value = cpystr(tmp);
+
+    new_confline(ctmp)->var = vtmp;
+    (*ctmp)->flags |= CF_NOSELECT | CF_B_LINE;
+
+    /* manage password file certificates */
+    new_confline(ctmp);
+    (*ctmp)->tool           = smime_helper_tool;
+    (*ctmp)->keymenu        = &config_smime_manage_password_file_menu_keymenu;
+    (*ctmp)->help           = h_config_smime_password_file_certificates;
+    (*ctmp)->value          = cpystr(_("Manage Password File Key and Certificate"));
+    (*ctmp)->varmem         = 12;
+    (*ctmp)->d.s.ctype	    = Password;
+#endif /* PASSFILE */
 
     (*ctmp)->next	    = NULL;
 }
@@ -1156,11 +1213,16 @@ void display_certificate_information(struct pine *ps, X509 *cert, char *email, W
       scrollargs.keys.menu   = &smime_certificate_info_keymenu;
       setbitmap(scrollargs.keys.bitmap);
       if(ctype != Public || error != X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
+/*error != X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY)*/
 	  clrbitn(TRUST_KEY, scrollargs.keys.bitmap);
       if(ctype != Private){
 	  clrbitn(PUBLIC_KEY,  scrollargs.keys.bitmap);
 	  clrbitn(PRIVATE_KEY, scrollargs.keys.bitmap);
       }
+      if(ctype == Password){
+	  clrbitn(DELETE_CERT_KEY,  scrollargs.keys.bitmap);
+	  clrbitn(UNDELETE_CERT_KEY,  scrollargs.keys.bitmap);
+      }
 
       cmd = scrolltool(&scrollargs);
 
@@ -1207,11 +1269,46 @@ manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
     WhichCerts ctype = (*cl)->d.s.ctype;
 
     switch(cmd){
+      case MC_ADD:	/* create a self signed certificate and import it */
+	   if(ctype == Password){
+	      PERSONAL_CERT *pc;
+	      char pathdir[MAXPATH+1], filename[MAXPATH+1];
+	      struct stat sbuf;
+	      int st;
+	      smime_path(DF_SMIMETMPDIR, pathdir, sizeof(pathdir));   
+	      if(((st = our_stat(pathdir, &sbuf)) == 0
+		   && (sbuf.st_mode & S_IFMT) == S_IFDIR)
+		 || (st != 0
+		     && can_access(pathdir, ACCESS_EXISTS) != 0
+	             && our_mkpath(pathdir, 0700) == 0)){
+	        pc = ALPINE_self_signed_certificate(NULL, 0, pathdir, MASTERNAME);
+		snprintf(filename, sizeof(filename), "%s/%s.key", 
+					pathdir, MASTERNAME);
+		filename[sizeof(filename)-1] = '\0';
+		rv = import_certificate(ctype, pc, filename);
+		if(our_stat(pathdir, &sbuf) == 0){
+		  if(unlink(filename) < 0)
+		   q_status_message1(SM_ORDER, 0, 2, 
+			_("Could not remove private key %s.key"), MASTERNAME);
+		  filename[strlen(filename)-4] = '\0';
+		  strcat(filename, ".crt");
+		  if(unlink(filename) < 0)
+		   q_status_message1(SM_ORDER, 0, 2, 
+			_("Could not remove public certifica %s.crt"), MASTERNAME);
+		  if(rmdir(pathdir) < 0)
+		    q_status_message1(SM_ORDER, 0, 2, 
+		      _("Could not remove temporary directory %s"), pathdir);
+		}
+	      }
+	      rv = 10;		/* forces redraw */
+	   }
+	   break;
+
       case MC_CHOICE:
 	   if(PATHCERTDIR(ctype) == NULL)
 	     return 0;
 	   
-	   if((cert = get_cert_for((*cl)->d.s.address, ctype)) == NULL){
+	   if((cert = get_cert_for((*cl)->d.s.address, ctype, 0)) == NULL){
 	      q_status_message(SM_ORDER, 1, 3, _("Problem Reading Certificate"));
 	      rv = 0;
 	   }
@@ -1258,7 +1355,7 @@ manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
 	  break;
 	}
       case MC_IMPORT: 
-	rv = import_certificate(ctype);
+	rv = import_certificate(ctype, NULL, NULL);
 	if(rv < 0){
 	  switch(rv){
 	    default:
@@ -1289,19 +1386,167 @@ manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
 }
 
 void
-smime_setup_size(char **s, size_t n)
+smime_setup_size(char **s, size_t buflen, size_t n)
 {
    char *t = *s;
   *t++ = ' ';
   *t++ = '%';
   *t++ = '-';
-   sprintf(t+strlen(t), "%d.%d", n, n);
+   snprintf(t, buflen-3, "%zu.%zu", n, n);
    t += strlen(t);
    *t++ = 's';
-   *t++ = ' ';
    *s = t;
 }
 
+#ifdef PASSFILE
+void manage_password_file_certificates(struct pine *ps)
+{
+    OPT_SCREEN_S    screen;
+    int             readonly_warning = 0, rv = 10, fline, state = 0;
+
+    dprint((9, "manage_password_file_certificates"));
+    ps->next_screen = SCREEN_FUN_NULL;
+
+    do {
+      CONF_S *ctmp = NULL, *first_line = NULL;
+
+      fline = rv >= 10 ? rv - 10 : 0;
+
+      smime_manage_password_file_certs_init(ps, &ctmp, &first_line, fline, &state);
+
+      if(ctmp == NULL){
+	ps->mangled_screen = 1;
+	q_status_message(SM_ORDER, 1, 3, _("Failed to initialize password management screen (no key)"));
+        return;
+      }
+
+      memset(&screen, 0, sizeof(screen));
+      screen.deferred_ro_warning = readonly_warning;
+      rv = conf_scroll_screen(ps, &screen, first_line,
+			       _("MANAGE PASSWORD FILE CERTS"),
+			      /* TRANSLATORS: Print something1 using something2.
+				 configuration is something1 */
+			      _("configuration"), 0, NULL);
+    } while (rv != 0);
+
+    ps->mangled_screen = 1;
+    smime_reinit();
+}
+
+ /* state: 0 = first time, 
+  *        1 = second or another time
+  */
+void 
+smime_manage_password_file_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line, int fline, int *state)
+{
+    char     tmp[200];
+    char    *ext;
+    CertList *cl;
+    int	  i;
+    void  *pwdcert = NULL;	/* this is our current password file */
+    PERSONAL_CERT *pc;
+    X509_LOOKUP *lookup = NULL;
+    X509_STORE  *store  = NULL;
+
+    if(*state == 0){		/* first time around? */
+      setup_pwdcert(&pwdcert);
+      if(pwdcert == NULL) return;
+      if(ps->pwdcert == NULL)
+         ps->pwdcert = pwdcert;
+      else
+         free_personal_certs((PERSONAL_CERT **) &pwdcert);
+      (*state)++;
+    }
+
+    pc = (PERSONAL_CERT *) ps_global->pwdcert;
+    ps->pwdcertlist = cl = smime_X509_to_cert_info(X509_dup(pc->cert), pc->name);
+
+    for(i = 0; i < sizeof(tmp) && i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp)); i++)
+	tmp[i] = '-';
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT;
+    (*ctmp)->value = cpystr(tmp);
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT;
+    (*ctmp)->value = cpystr(_("Manage Certificates and Keys Used to Encrypt your Password File"));
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT;
+    (*ctmp)->value = cpystr(tmp);
+
+    new_confline(ctmp);
+    (*ctmp)->flags |= CF_NOSELECT | CF_B_LINE;
+
+    if(cl){
+      int s, e, df, dt, md5;	/* sizes of certain fields */
+      int nf;			/* number of fields */
+      char u[MAILTMPLEN], *t;
+
+      e = MIN(strlen(cl->name), ps->ttyo->screen_cols/3);	/* do not use too much screen */
+      nf = 5;		/* there are 5 fields */
+      s = 3;		/* status has fixed size */
+      df = dt = 10;	/* date from and date to have fixed size */
+      md5 = ps->ttyo->screen_cols - s - df - dt - e - (nf - 1);
+
+      t = u;
+      smime_setup_size(&t, sizeof(u), s);
+      smime_setup_size(&t, sizeof(u) - strlen(t), e);
+      smime_setup_size(&t, sizeof(u) - strlen(t), df);
+      *t++ = ' ';	/* leave an extra space between dates */
+      smime_setup_size(&t, sizeof(u) - strlen(t), dt);
+      *t++ = ' ';	/* and another space between date and md5 sum */
+      smime_setup_size(&t, sizeof(u) - strlen(t), md5);
+      *t = '\0';	/* tie off */
+
+      new_confline(ctmp);
+      (*ctmp)->flags |= CF_NOSELECT;
+      (*ctmp)->value = cpystr(_("New Public Certificate and Key:"));
+
+      new_confline(ctmp);
+      (*ctmp)->d.s.ctype  = Password;
+      (*ctmp)->help	  = h_config_smime_password_file_certificates;
+      (*ctmp)->tool       = manage_certs_tool;
+      (*ctmp)->keymenu    = &config_smime_add_new_key_keymenu;
+      s += 2;
+      for(i = 0; i < s; i++) tmp[i] = ' ';
+      tmp[i] = '\0';
+      strncpy(tmp+s, _("Press \"RETURN\" to add new personal key"), sizeof(tmp)-s-1);
+      for(i = strlen(tmp); i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp) - 1); i++)
+         tmp[i] = ' ';
+      tmp[i] = '\0';
+      (*ctmp)->value      = cpystr(tmp);
+      *first_line = *ctmp;
+
+      new_confline(ctmp);
+      (*ctmp)->flags |= CF_NOSELECT | CF_B_LINE;
+
+      new_confline(ctmp);
+      (*ctmp)->flags |= CF_NOSELECT;
+      (*ctmp)->value = cpystr(_("Current Public Certificate and Key:"));
+
+      new_confline(ctmp);
+      (*ctmp)->d.s.ctype   = Password;
+      (*ctmp)->d.s.deleted = 0;
+      (*ctmp)->help	   = h_config_smime_password_file_certificates;
+      (*ctmp)->tool        = manage_certs_tool;
+      (*ctmp)->keymenu     = &config_smime_manage_view_cert_keymenu;
+      (*ctmp)->varmem      = 0;
+      (*ctmp)->help	   = h_config_smime_manage_public_menu;
+      strncpy((*ctmp)->d.s.address, cl->name, sizeof((*ctmp)->d.s.address));
+      (*ctmp)->d.s.address[sizeof((*ctmp)->d.s.address) - 1] = '\0';
+      snprintf(tmp, sizeof(tmp), u,
+			(*ctmp)->d.s.deleted ? "D" : " ", 
+			cl->name, 
+			DATEFROMCERT(cl), DATETOCERT(cl), MD5CERT(cl));
+      (*ctmp)->value      = cpystr(tmp);
+
+   }
+}
+#endif /* PASSFILE */
+
+
 void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line, WhichCerts ctype, int fline)
 {
     char            tmp[200];
@@ -1327,9 +1572,9 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
 
     new_confline(ctmp);
     (*ctmp)->flags |= CF_NOSELECT;
-    sprintf(tmp, _("List of %scertificates"), ctype == Public ? _("public ")
-		: (ctype == Private ? _("private ") 
-		: (ctype == CACert ? _("certificate authority ") : "unknown (?) ")));
+    sprintf(tmp, _("List of %s certificates"), ctype == Public ? _("public")
+		: (ctype == Private ? _("private") 
+		: (ctype == CACert ? _("certificate authority") : "unknown (?)")));
     (*ctmp)->value = cpystr(tmp);
 
     for(i = 0; i < sizeof(tmp) && i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp)); i++)
@@ -1351,20 +1596,23 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
 	 if(cl->name && strlen(cl->name) > e)
 	   e = strlen(cl->name);
 
-      if(ctype != Private)
-	e -= 4;		/* remove extension length FIX FIX FIX */
+      if(ctype != Private && SMHOLDERTYPE(ctype) == Directory)
+	e -= 4;		/* remove extension length */
+      e = MIN(e, ps->ttyo->screen_cols/3);	/* do not use too much screen */
       nf = 5;		/* there are 5 fields */
       s = 3;		/* status has fixed size */
-      df = dt = 8;	/* date from and date to have fixed size */
-      md5 = ps->ttyo->screen_cols - s - df - dt - e - 2*nf;
+      df = dt = 10;	/* date from and date to have fixed size */
+      md5 = ps->ttyo->screen_cols - s - df - dt - e - (nf - 1);
 
-      memset(u, '\0', sizeof(u));
       t = u;
-      smime_setup_size(&t, s);
-      smime_setup_size(&t, e);
-      smime_setup_size(&t, df);
-      smime_setup_size(&t, dt);
-      smime_setup_size(&t, md5);
+      smime_setup_size(&t, sizeof(u), s);
+      smime_setup_size(&t, sizeof(u) - strlen(t), e);
+      smime_setup_size(&t, sizeof(u) - strlen(t), df);
+      *t++ = ' ';	/* leave an extra space between dates */
+      smime_setup_size(&t, sizeof(u) - strlen(t), dt);
+      *t++ = ' ';	/* and another space between date and md5 sum */
+      smime_setup_size(&t, sizeof(u) - strlen(t), md5);
+      *t = '\0';	/* tie off */
 
       for(cl = data, i = 0; cl; cl = cl->next)
 	 if(cl->name){
@@ -1385,7 +1633,8 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
 	    (*ctmp)->d.s.address[sizeof((*ctmp)->d.s.address) - 1] = '\0';
 	    snprintf(tmp, sizeof(tmp), u,
 			(*ctmp)->d.s.deleted ? "D" : " ", 
-			cl->name, DATEFROMCERT(cl), DATETOCERT(cl), MD5CERT(cl));
+			ctype == CACert ? cl->cn : cl->name, 
+			DATEFROMCERT(cl), DATETOCERT(cl), MD5CERT(cl));
 	    if(ctype != Private && SMHOLDERTYPE(ctype) == Directory)
 	       cl->name[strlen(cl->name)] = '.';
 	    (*ctmp)->value      = cpystr(tmp);
@@ -1433,7 +1682,7 @@ void manage_certificates(struct pine *ps, WhichCerts ctype)
 			       _("MANAGE CERTIFICATES"),
 			      /* TRANSLATORS: Print something1 using something2.
 				 configuration is something1 */
-			      _("configuration"), 0);
+			      _("configuration"), 0, NULL);
     } while (rv != 0);
 
     ps->mangled_screen = 1;
@@ -1542,6 +1791,10 @@ smime_helper_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
 	  case 10: manage_certificates(ps, Private); break;
 	  case 11: manage_certificates(ps, CACert) ; break;
 
+#ifdef PASSFILE
+	  case 12: manage_password_file_certificates(ps); break;
+#endif /* PASSFILE */
+
 	  default:
 	    rv = -1;
 	    break;
@@ -1554,12 +1807,7 @@ smime_helper_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
 	break;
 
       case MC_IMPORT:
-	{ WhichCerts ctype;
-	  /* keep this selection consistent with the codes above */
-	  ctype = (*cl)->varmem == 9 ? Public 
-			: ((*cl)->varmem == 10 ? Private : CACert); 
-	  rv = import_certificate(ctype);
-	}
+	rv = import_certificate((*cl)->d.s.ctype, NULL, NULL);
 	break;
 
       default:
