@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: takeaddr.c 1012 2008-03-26 00:44:22Z hubert@u.washin
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -1569,7 +1569,7 @@ whereis_taline(TA_S *current)
     buf[0] = '\0';
     snprintf(tmp, sizeof(tmp), _("Word to find %s%.*s%s: "),
 	     (last[0]) ? "[" : "",
-	     sizeof(tmp)-20, (last[0]) ? last : "",
+	     MAX_SEARCH, (last[0]) ? last : "",	/* MAX_SEARCH == sizeof(tmp) - 20 */
 	     (last[0]) ? "]" : "");
     tmp[sizeof(tmp)-1] = '\0';
     help = NO_HELP;
@@ -1892,11 +1892,11 @@ update_takeaddr_screen(struct pine *ps, TA_S *current, TA_SCREEN_S *screen, Pos
 		}
 	    }
 	    else{
-		if(screen->mode == ListMode)
+		if(screen->mode == ListMode) /* 6*MAX_SCREEN_COLS + 24 = sizeof(buf2)-6 */
 	          snprintf(buf2, sizeof(buf2), "[%c]  %.*s", ctmp->checked ? 'X' : SPACE,
-			  sizeof(buf2)-6, buf1);
+			  6*MAX_SCREEN_COLS + 24, buf1); 
 		else
-	          snprintf(buf2, sizeof(buf2), "     %.*s", sizeof(buf2)-6, buf1);
+	          snprintf(buf2, sizeof(buf2), "     %.*s", 6*MAX_SCREEN_COLS+24, buf1);
 
 		buf2[sizeof(buf2)-1] = '\0';
 	    }
@@ -2066,6 +2066,7 @@ attached_addr_handler(TA_S *current, int added)
 int
 take_without_edit(TA_S *ta_list, int num_in_list, int command_line, TA_STATE_S **tas, char *cmd)
 {
+#define OURTMPBUFLEN 200
     PerAddrBook   *pab_dst;
     SAVE_STATE_S   state;  /* For saving state of addrbooks temporarily */
     int            rc, total_to_copy;
@@ -2074,7 +2075,7 @@ take_without_edit(TA_S *ta_list, int num_in_list, int command_line, TA_STATE_S *
     int		   err = 0, need_write = 0, we_cancel = 0;
     adrbk_cntr_t   new_entry_num;
     char           warn[2][MAX_NICKNAME+1];
-    char           tmp[200];
+    char           tmp[OURTMPBUFLEN];
     TA_S          *current;
     SWOOP_S       *swoop_list = NULL, *sw;
 
@@ -2150,13 +2151,13 @@ take_without_edit(TA_S *ta_list, int num_in_list, int command_line, TA_STATE_S *
 	    else{
 		snprintf(tmp, sizeof(tmp),
 		        "Entry with nickname \"%.*s\" already exists, replace ",
-		        sizeof(tmp)-50, warn[0]);
+		        OURTMPBUFLEN-50, warn[0]);
 	    }
 	}
 	else if(how_many_dups == 2)
 	  snprintf(tmp, sizeof(tmp),
 		  "Nicknames \"%.*s\" and \"%.*s\" already exist, replace ",
-		  (sizeof(tmp)-50)/2, warn[0], (sizeof(tmp)-50)/2, warn[1]);
+		  (OURTMPBUFLEN-50)/2, warn[0], (OURTMPBUFLEN-50)/2, warn[1]);
 	else
 	  snprintf(tmp, sizeof(tmp), "%d of the nicknames already exist, replace ",
 		  how_many_dups);
@@ -2413,14 +2414,15 @@ get_out:
     ps_global->mangled_footer = 1;
 
     if(err){
-	char capcmd[50];
+#define CAPCMDLEN 50
+	char capcmd[CAPCMDLEN];
 
 	ret = -1;
 	snprintf(capcmd, sizeof(capcmd),
 		"%c%.*s",
 		islower((unsigned char)(*cmd)) ? toupper((unsigned char)*cmd)
 					       : *cmd,
-		sizeof(capcmd)-2, cmd+1);
+		CAPCMDLEN-2, cmd+1);
 	if(need_write)
 	  q_status_message1(SM_ORDER | SM_DING, 3, 4,
 			   "%.200s only partially completed", capcmd);
@@ -2433,7 +2435,7 @@ get_out:
 	snprintf(tmp, sizeof(tmp), "Saved %d %s to \"%.*s\"",
 		total_to_copy,
 		(total_to_copy > 1) ? "entries" : "entry",
-		sizeof(tmp)-30, pab_dst->abnick);
+		OURTMPBUFLEN-30, pab_dst->abnick);
 	q_status_message(SM_ORDER, 4, 4, tmp);
     }
 
@@ -2805,7 +2807,7 @@ take_to_export(struct pine *ps, LINES_TO_TAKE *lines_to_take)
       q_status_message(SM_ORDER, 3, 3, _("No lines to export"));
     else{
 	memset(&screen, 0, sizeof(screen));
-	conf_scroll_screen(ps, &screen, first_line, _("Take Export"), NULL, 0);
+	conf_scroll_screen(ps, &screen, first_line, _("Take Export"), NULL, 0, NULL);
     }
 }
 
@@ -2980,16 +2982,15 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
 	  *first = NULL,
 	  *last = NULL,
 	  *comment = NULL;
-    char **cn = NULL,
+    struct berval **cn = NULL,
 	 **mail = NULL,
 	 **sn = NULL,
 	 **givenname = NULL,
 	 **title = NULL,
 	 **telephone = NULL,
 	 **elecmail = NULL,
-	 **note = NULL,
-	 **ll;
-    int    j,
+	 **note = NULL;
+    int    j, num,
 	   export = 0;
 
 
@@ -3006,35 +3007,35 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
 	    dprint((9, " %s", a ? a : "?"));
 	    if(strcmp(a, e->info_used->cnattr) == 0){
 		if(!cn)
-		  cn = ldap_get_values(e->ld, e->selected_entry, a);
+		  cn = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, e->info_used->mailattr) == 0){
 		if(!mail)
-		  mail = ldap_get_values(e->ld, e->selected_entry, a);
+		  mail = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, "electronicmail") == 0){
 		if(!elecmail)
-		  elecmail = ldap_get_values(e->ld, e->selected_entry, a);
+		  elecmail = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, "comment") == 0){
 		if(!note)
-		  note = ldap_get_values(e->ld, e->selected_entry, a);
+		  note = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, e->info_used->snattr) == 0){
 		if(!sn)
-		  sn = ldap_get_values(e->ld, e->selected_entry, a);
+		  sn = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, e->info_used->gnattr) == 0){
 		if(!givenname)
-		  givenname = ldap_get_values(e->ld, e->selected_entry, a);
+		  givenname = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, "telephonenumber") == 0){
 		if(!telephone)
-		  telephone = ldap_get_values(e->ld, e->selected_entry, a);
+		  telephone = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	    else if(strcmp(a, "title") == 0){
 		if(!title)
-		  title = ldap_get_values(e->ld, e->selected_entry, a);
+		  title = ldap_get_values_len(e->ld, e->selected_entry, a);
 	    }
 	}
 
@@ -3065,10 +3066,10 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
     }
 
     if(elecmail){
-	if(elecmail[0] && elecmail[0][0] && !mail)
+	if(ALPINE_LDAP_can_use(elecmail) && !mail)
 	  mail = elecmail;
 	else
-	  ldap_value_free(elecmail);
+	  ldap_value_free_len(elecmail);
 
 	elecmail = NULL;
     }
@@ -3078,51 +3079,51 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
 	char    *d,
 		*fakedomain = "@";
 	size_t   len;
-	char   **cm;
+	struct berval  **cm;
 	int      how_many_selected = 0;
 
 	
-	if(cn && cn[0] && cn[0][0])
-	  fullname = cpystr(cn[0]);
-	if(sn && sn[0] && sn[0][0])
-	  last = cpystr(sn[0]);
-	if(givenname && givenname[0] && givenname[0][0])
-	  first = cpystr(givenname[0]);
+	if(ALPINE_LDAP_can_use(cn))
+	  fullname = cpystr(cn[0]->bv_val);
+	if(ALPINE_LDAP_can_use(sn))
+	  last = cpystr(sn[0]->bv_val);
+	if(ALPINE_LDAP_can_use(givenname))
+	  first = cpystr(givenname[0]->bv_val);
 
-	if(note && note[0] && note[0][0])
+	if(ALPINE_LDAP_can_use(note))
 	  cm = note;
 	else
 	  cm = title;
 
-	for(ll = cm, len = 0; ll && *ll; ll++)
-	  len += strlen(*ll) + 2;
+	for(len = 0, num = 0; ALPINE_LDAP_usable(cm, num); num++)
+	  len += strlen(cm[num]->bv_val) + 2;
 
 	if(len){
 	    comment = (char *)fs_get(len * sizeof(char));
 	    d = comment;
-	    ll = cm;
-	    while(*ll){
-		sstrncpy(&d, *ll, len-(d-comment));
-		ll++;
-		if(*ll)
+	    num = 0;
+	    while(ALPINE_LDAP_usable(cm, num)){
+		sstrncpy(&d, cm[num]->bv_val, len-(d-comment));
+		num++;
+		if(ALPINE_LDAP_usable(cm, num))
 		  sstrncpy(&d, "; ", len-(d-comment));
 	    }
 
 	    comment[len-1] = '\0';
 	}
 
-	for(ll = mail, len = 0; ll && *ll; ll++)
-	  len += strlen(*ll) + 1;
+	for(len = 0, num = 0; ALPINE_LDAP_usable(mail, num); num++)
+	  len += strlen(mail[num]->bv_val) + 2;
 	
 	/* paste the email addresses together */
 	if(len){
 	    address = (char *)fs_get(len * sizeof(char));
 	    d = address;
-	    ll = mail;
-	    while(*ll){
-		sstrncpy(&d, *ll, len-(d-address));
-		ll++;
-		if(*ll)
+	    num = 0;
+	    while(ALPINE_LDAP_usable(mail, num)){
+		sstrncpy(&d, mail[num]->bv_val, len-(d-address));
+		num++;
+		if(ALPINE_LDAP_usable(mail, num))
 		  sstrncpy(&d, ", ", len-(d-address));
 	    }
 
@@ -3295,14 +3296,14 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
 	    break;
 	
 	  case 'a':
-	    if(mail && mail[0] && mail[0][0]){
+	    if(ALPINE_LDAP_can_use(mail)){
 
 		srctype = CharStar;
 		if(!(srcstore = so_get(srctype, NULL, EDIT_ACCESS)))
 		  q_status_message(SM_ORDER,0,2, _("Error allocating space"));
 		else{
-		    for(ll = mail; ll && *ll; ll++){
-			so_puts(srcstore, *ll);
+		    for(num = 0; ALPINE_LDAP_usable(mail, num); num++){
+			so_puts(srcstore, mail[num]->bv_val);
 			so_puts(srcstore, "\n");
 		    }
 
@@ -3325,26 +3326,26 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
 		vinfo = (VCARD_INFO_S *)fs_get(sizeof(VCARD_INFO_S));
 		memset((void *)vinfo, 0, sizeof(VCARD_INFO_S));
 
-		if(cn && cn[0] && cn[0][0])
-		  vinfo->fullname = copy_list_array(cn);
+		if(ALPINE_LDAP_can_use(cn))
+		  vinfo->fullname = berval_to_array(cn);
 
-		if(note && note[0] && note[0][0])
-		  vinfo->note = copy_list_array(note);
+		if(ALPINE_LDAP_can_use(note))
+		  vinfo->note = berval_to_array(note);
 
-		if(title && title[0] && title[0][0])
-		  vinfo->title = copy_list_array(title);
+		if(ALPINE_LDAP_can_use(title))
+		  vinfo->title = berval_to_array(title);
 
-		if(telephone && telephone[0] && telephone[0][0])
-		  vinfo->tel = copy_list_array(telephone);
+		if(ALPINE_LDAP_can_use(telephone))
+		  vinfo->tel = berval_to_array(telephone);
 
-		if(mail && mail[0] && mail[0][0])
-		  vinfo->email = copy_list_array(mail);
+		if(ALPINE_LDAP_can_use(mail))
+		  vinfo->email = berval_to_array(mail);
 
-		if(sn && sn[0] && sn[0][0])
-		  vinfo->last = cpystr(sn[0]);
+		if(ALPINE_LDAP_can_use(sn))
+		  vinfo->last = cpystr(sn[0]->bv_val);
 
-		if(givenname && givenname[0] && givenname[0][0])
-		  vinfo->first = cpystr(givenname[0]);
+		if(ALPINE_LDAP_can_use(givenname))
+		  vinfo->first = cpystr(givenname[0]->bv_val);
 
 		gf_set_so_writec(&pc, srcstore);
 
@@ -3366,21 +3367,21 @@ save_ldap_entry(struct pine *ps, LDAP_CHOOSE_S *e, int save)
     }
 
     if(cn)
-      ldap_value_free(cn);
+      ldap_value_free_len(cn);
     if(mail)
-      ldap_value_free(mail);
+      ldap_value_free_len(mail);
     if(elecmail)
-      ldap_value_free(elecmail);
+      ldap_value_free_len(elecmail);
     if(note)
-      ldap_value_free(note);
+      ldap_value_free_len(note);
     if(sn)
-      ldap_value_free(sn);
+      ldap_value_free_len(sn);
     if(givenname)
-      ldap_value_free(givenname);
+      ldap_value_free_len(givenname);
     if(telephone)
-      ldap_value_free(telephone);
+      ldap_value_free_len(telephone);
     if(title)
-      ldap_value_free(title);
+      ldap_value_free_len(title);
     if(fullname)
       fs_give((void **)&fullname);
     if(address)
