@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: titlebar.c 1075 2008-06-04 00:19:39Z hubert@u.washin
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -83,7 +83,8 @@ static struct titlebar_state {
 		 del_column, 
 		 percent_column,
 		 page_column,
-		 screen_cols;
+		 screen_cols,
+		 pushed;
     enum	 {Normal, OnlyRead, Closed} stream_status;
     TitleBarType style;
     TITLE_S      titlecontainer;
@@ -92,13 +93,20 @@ static struct titlebar_state {
 
 static int titlebar_is_dirty = 1;
 
+char *as_fname;		/* folder name */
+char *as_cname;		/* context name */
 
 void
 push_titlebar_state(void)
 {
+    as.pushed = 1;
     titlebar_stack     = as;
-    as.folder_name     = NULL;	/* erase knowledge of malloc'd data */
-    as.context_name    = NULL;
+    if(as_fname) fs_give((void **)&as_fname);
+    as_fname = cpystr(as.folder_name);
+    if(as_cname) fs_give((void **)&as_cname);
+    as_cname = cpystr(as.context_name);
+    if(as.folder_name) fs_give((void **)&as.folder_name);
+    if(as.context_name) fs_give((void **)&as.context_name);
 }
 
 
@@ -110,6 +118,11 @@ pop_titlebar_state(void)
 	fs_give((void **)&(as.folder_name)); /* free malloc'd values */
 	fs_give((void **)&(as.context_name));
 	as = titlebar_stack;
+	if(as.pushed){
+	  as.folder_name = as_fname ? cpystr(as_fname) : NULL;
+	  as.context_name = as_cname ? cpystr(as_cname): NULL;
+	}
+	as.pushed = 0;
     }
 }
 
@@ -274,7 +287,12 @@ set_titlebar(char *title, MAILSTREAM *stream, CONTEXT_S *cntxt, char *folder,
 	   && rawno <= as.stream->nmsgs
 	   && !((mc = mail_elt(as.stream, rawno)) && mc->valid)){
 	    pine_mail_fetch_flags(as.stream, long2string(rawno), NIL);
-	    mc = mail_elt(as.stream, rawno);
+	    if(rawno <= as.stream->nmsgs && as.stream && rawno > 0L)
+	      mc = mail_elt(as.stream, rawno);
+	    else
+	      mc = NULL;
+	    if(mc && !mc->valid)
+	      mc = NULL;
 	}
     }
     
@@ -366,12 +384,10 @@ titlebar_stream_closing(MAILSTREAM *stream)
 COLOR_PAIR *
 current_titlebar_color(void)
 {
-    TITLE_S       *tc;
     COLOR_PAIR    *col;
     COLOR_PAIR    *the_color = NULL;
 
-    tc = format_titlebar();
-    col = tc ? &tc->color : NULL;
+    col = as.title ? &as.titlecontainer.color : NULL;
 
     if(col && col->fg && col->fg[0] && col->bg && col->bg[0])
       the_color = new_color_pair(col->fg, col->bg);
@@ -396,6 +412,8 @@ current_titlebar_color(void)
         "Title".  The rule for existence is that in the
         space between 1) and 2) there must be one space between
         3) and 4) AND at least 50% of 4) must be displayed.
+	However, if the folder name can be displayed, then do
+	so, and display as much as possible of the collection name.
 
    Returns - Formatted title bar 
  ----*/
@@ -758,6 +776,15 @@ try_smaller_loc:
 			else if(ss_len > 0 && ss_len <= avail){
 			    q = fold_tmp;
 			    strncpy(q, ss_string, sizeof(fold_tmp)-(q-fold_tmp));
+			} else if(fold_len < avail){
+			    q = fold_tmp;
+			    if(fold_len + 7 < avail){
+			       *q++ = '<';
+			       q += utf8_pad_to_width(q, as.context_name, sizeof(fold_tmp)-(q-fold_tmp), avail - fold_len - 3, 1);
+			       *q++ = '>';
+			       *q++ = ' ';
+			    }
+			    strncpy(q, as.folder_name, sizeof(fold_tmp)-(q-fold_tmp));
 			}
 			/* else leave it out */
 		    }
