@@ -1,4 +1,5 @@
 /* ========================================================================
+ * Copyright 2015 Eduardo Chappa
  * Copyright 2008 Mark Crispin
  * ========================================================================
  */
@@ -9,7 +10,8 @@
  * Author:	Mark Crispin
  *
  * Date:	27 July 1988
- * Last Edited:	19 November 2008
+ * Last Edited:	19 November 2008 (Crispin)
+ * Last Edited: 16 January 2015 (Chappa)
  *
  * Previous versions of this file were
  *
@@ -165,10 +167,10 @@ SENDSTREAM *smtp_open_full (NETDRIVER *dv,char **hostlist,char *service,
       }
 				/* else port argument overrides SMTP port */
       else if (!port) port = smtp_port ? smtp_port : SMTPTCPPORT;
-      if (netstream =		/* try to open ordinary connection */
+      if ((netstream =		/* try to open ordinary connection */
 	  net_open (&mb,dv,port,
 		    (NETDRIVER *) mail_parameters (NIL,GET_SSLDRIVER,NIL),
-		    "*smtps",smtp_sslport ? smtp_sslport : SMTPSSLPORT)) {
+		    "*smtps",smtp_sslport ? smtp_sslport : SMTPSSLPORT)) != NULL) {
 	stream = (SENDSTREAM *) memset (fs_get (sizeof (SENDSTREAM)),0,
 					sizeof (SENDSTREAM));
 	stream->netstream = netstream;
@@ -345,8 +347,9 @@ void *smtp_challenge (void *s,unsigned long *len)
   if ((stream->replycode == SMTPAUTHREADY) &&
       !(ret = rfc822_base64 ((unsigned char *) stream->reply + 4,
 			     strlen (stream->reply + 4),len))) {
-    sprintf (tmp,"SMTP SERVER BUG (invalid challenge): %.80s",stream->reply+4);
+    sprintf (tmp,"SMTP SERVER BUG (invalid challenge, continuing): %.80s",stream->reply+4);
     mm_log (tmp,ERROR);
+    ret = cpystr("");	/* This is silly: fake a reply, it will be ignored */
   }
   return ret;
 }
@@ -785,7 +788,7 @@ long smtp_soutr (void *stream,char *s)
 				/* "." on first line */
   if (s[0] == '.') net_sout (stream,".",1);
 				/* find lines beginning with a "." */
-  while (t = strstr (s,"\015\012.")) {
+  while ((t = strstr (s,"\015\012.")) != NULL) {
     c = *(t += 3);		/* remember next character after "." */
     *t = '\0';			/* tie off string */
 				/* output prefix */
