@@ -31,7 +31,10 @@
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 1
 #endif /* _XOPEN_SOURCE */
-#ifndef _BSD_SOURCE
+#ifndef _DEFAULT_SOURCE
+#define _DEFAULT_SOURCE 1
+#endif /* _DEFAULT_SOURCE */
+#ifndef _BSD_SOURCE	/* gcc 6 deprecates this, so will we */
 #define _BSD_SOURCE 1
 #endif /* _BSD_SOURCE */
 
