@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: attach.c 1082 2008-06-12 18:39:50Z hubert@u.washingt
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -62,9 +62,10 @@ AskAttach(char *cmnt, size_t cmntlen, LMLIST **lm)
 
     while(i){
 	if(i == 2){
-	    EXTRAKEYS menu_attach[4];
+	    EXTRAKEYS menu_attach[10];
 	    int	      n;
 
+	    memset(&menu_attach, 0, 10*sizeof(EXTRAKEYS));
 	    menu_attach[n = 0].name  = "^T";
 	    menu_attach[n].label     = N_("To Files");
 	    menu_attach[n].key	     = (CTRL|'T');
@@ -169,7 +170,7 @@ AskAttach(char *cmnt, size_t cmntlen, LMLIST **lm)
 				? "."
 				: ((gmode & MDTREE) || opertree[0])
 				    ? opertree : gethomedir(NULL),
-			      C_FILESEP, p - fn, fn);
+			      C_FILESEP, (int) (p - fn), fn);
 		}
 		else{
 		    fname = fn;
@@ -1068,8 +1069,8 @@ process_tag:					/* enclosed in []         */
 		 * whole attachment, comment or done! 
 		 */
 		if(c == ',' || c == '\0' || c == '\"'){
-		    strncpy(sz, (lblsz) ? lblsz : prettysz(attsz), sizeof(sz));
-		    sz[sizeof(sz)-1] = '\0';
+		    strncpy(sz, (lblsz) ? lblsz : prettysz(attsz), szlen);
+		    sz[szlen-1] = '\0';
 
 		    snprintf(ctmp, sizeof(ctmp), " (%s) %s", sz, (c == '\"') ? "" : "\"\"");
 		    u = utf8_to_ucs4_cpystr(ctmp);
