@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: browse.c 942 2008-03-04 18:21:33Z hubert@u.washingto
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -317,7 +317,8 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
     UCS c, new_c;
     int status, i, j;
     int row, col, crow, ccol;
-    int bsearch;	/* search forward by default */
+    int flags;
+    int add_file;
     char *p, *envp, child[NLINE], tmp[NLINE];
     struct bmaster *mp;
     struct fcell *tp;
@@ -803,6 +804,14 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		static EXTRAKEYS opts[] = {
 		    {"^X", N_("Add Name"), CTRL|'X', KS_NONE},
 		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE},
+		    {NULL, NULL, 0, KS_NONE}
 		};
 
 		status = mlreply_utf8(_("Command to execute: "),
@@ -1044,6 +1053,7 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		break;
 	    }
 
+	    add_file = 1;
 	    i = 0;
 	    child[0] = '\0';
 	    /* pass in default filename */
@@ -1053,9 +1063,16 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 	    }
 
 	    while(!i){
+		int repaint = 0;
+		EXTRAKEYS opts[10];
 
-		switch(status=mlreply_utf8(_("Name of file to add: "), child, NLINE,
-				      QFFILE, NULL)){
+		memset((void *) &opts, 0, 10*sizeof(EXTRAKEYS));
+		opts[0].name  = "^X";
+		opts[0].label = add_file ? N_("Add Dir") : N_("Add File");
+		opts[0].key   = (CTRL|'X');
+
+		switch(status=mlreply_utf8(add_file ? _("Name of file to add: ") : _("Name of directory to add: "), child, NLINE,
+				      QFFILE, opts)){
 		  case HELPCH:
 		    emlwrite(_("\007No help yet!"), NULL);
 /* remove break and sleep after help text is installed */
@@ -1064,8 +1081,11 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		  case (CTRL|'L'):
 		    PaintBrowser(gmp, 0, &crow, &ccol);
 		    break;
+		  case (CTRL|'X'):
+		    if(add_file > 0) add_file = 0; else add_file = 1;
+		    break;
 		  case ABORT:
-		    emlwrite(_("Add File Cancelled"), NULL);
+		    emlwrite(add_file > 0 ? _("Add File Cancelled") : _("Add Directory Cancelled"), NULL);
 		    i++;
 		    break;
 		  case FALSE:
@@ -1080,7 +1100,7 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		    i++;
 
 		    if(child[0] == '\0'){
-			emlwrite(_("No file named.  Add Cancelled."), NULL);
+			emlwrite(add_file > 0 ? _("No file named.  Add Cancelled.") : _("No directory named. Add Cancelled"), NULL);
 			break;
 		    }
 
@@ -1097,8 +1117,8 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		    }
 
 		    if((status = fexist(child, "w", (off_t *)NULL)) == FIOSUC){
-			snprintf(tmp, sizeof(tmp), _("File \"%.*s\" already exists!"),
-				NLINE - 20, child);
+			snprintf(tmp, sizeof(tmp), _("%s \"%.*s\" already exists!"),
+				NLINE - 20, add_file > 0 ? "File" : "Directory", child);
 			emlwrite(tmp, NULL);
 			break;
 		    }
@@ -1107,7 +1127,15 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 			break;
 		    }
 
-		    if(ffwopen(child, FALSE) != FIOSUC){
+		    if(add_file == 0){
+			if(our_mkdir(child, (0700)) < 0){
+			   eml.s = child;
+			   emlwrite(_("Error adding Directory \"%s\""), &eml);
+			}
+			else	/* success! Directory added! */
+			  repaint = 1;
+		    }
+		    else if(ffwopen(child, FALSE) != FIOSUC){
 			/* ffwopen should've complained */
 			break;
 		    }
@@ -1115,7 +1143,10 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 			ffclose();
 		        eml.s = child;
 			emlwrite(_("Added File \"%s\""), &eml);
+			repaint = 1;
+		    }
 
+		    if(repaint > 0){
 			if((p = strrchr(child, C_FILESEP)) == NULL){
 			    emlwrite(_("Problems refiguring browser"), NULL);
 			    break;
@@ -1474,7 +1505,7 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		      tmp[sizeof(tmp)-1] = '\0';
 		    }
 
-		    strncat(tmp, gmp->current->fname, sizeof(tmp)-strlen(tmp));
+		    strncat(tmp, gmp->current->fname, sizeof(tmp)-strlen(tmp)-1);
 		    tmp[sizeof(tmp)-1] = '\0';
 		}
 
@@ -1597,11 +1628,11 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 	  case 'w':				/* Where is */
 	  case 'W':
 	  case (CTRL|'W'):
-	    bsearch = i = 0;
+	    i = 0;
+	    flags = SR_ORIGMEN | SR_FORWARD | SR_NOEXACT;
 
 	    while(!i){
-
-		switch(readpattern(_("File name to find"), FALSE, bsearch)){
+		switch(readpattern(_("File name to find"), FALSE, flags)){
 		  case HELPCH:
 		    emlwrite(_("\007No help yet!"), NULL);
 /* remove break and sleep after help text is installed */
@@ -1611,7 +1642,13 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		    PaintBrowser(gmp, 0, &crow, &ccol);
 		    break;
 		  case (CTRL|'P'):
-		    bsearch = bsearch == 0 ? 1 : 0;
+		    if(flags & SR_FORWARD){
+			flags &= ~SR_FORWARD;
+			flags |=  SR_BACKWRD;
+		    } else {
+			flags &= ~SR_BACKWRD;
+			flags |=  SR_FORWARD;
+		    }
 		    break;
 		  case (CTRL|'Y'):		/* first first cell */
 		    for(tp = gmp->top; tp->prev; tp = tp->prev)
@@ -1673,7 +1710,7 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 		    if(pat && pat[0])
 		      utf8 = ucs4_to_utf8_cpystr(pat);
 
-		    if(utf8 && (tp = FindCell(gmp, utf8, bsearch)) != NULL){
+		    if(utf8 && (tp = FindCell(gmp, utf8, flags & SR_BACKWRD)) != NULL){
 			PlaceCell(gmp, gmp->current, &row, &col);
 			PaintCell(row, col, gmp->cpf, gmp->current, 0);
 			gmp->current = tp;
