@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: display.c 1025 2008-04-08 22:59:38Z hubert@u.washing
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -43,7 +43,8 @@ static char rcsid[] = "$Id: display.c 1025 2008-04-08 22:59:38Z hubert@u.washing
 void     vtmove(int, int);
 void     vtputc(CELL);
 void     vteeol(void);
-void     updateline(int, CELL *, CELL *, short *);
+void     updateline(int, CELL *, CELL *, short *, int);
+void     updatelinecolor(int, CELL *, CELL *, short *, int);
 void     updext(void);
 void     mlputi(int, int);
 void     pprints(int, int);
@@ -54,6 +55,7 @@ int      dumbroot(int, int);
 int      dumblroot(long, int);
 unsigned cellwidth_ptr_to_ptr(CELL *pstart, CELL *pend);
 unsigned vcellwidth_a_to_b(int row, int a, int b);
+int	window_signature_block(WINDOW *wp);
 #ifdef _WINDOWS
 void    pico_config_menu_items (KEYMENU *);
 int     update_scroll (void);
@@ -109,8 +111,7 @@ static KEYMENU menu_compose[] = {
 
 #define VFCHG   0x0001                  /* Changed flag			*/
 #define	VFEXT	0x0002			/* extended (beyond column 80)	*/
-#define	VFREV	0x0004			/* reverse video status		*/
-#define	VFREQ	0x0008			/* reverse video request	*/
+#define VFSIG	0x0004			/* in signature block		*/
 
 int     vtrow   = 0;                    /* Row location of SW cursor */
 int     vtcol   = 0;                    /* Column location of SW cursor */
@@ -178,6 +179,7 @@ vtinit(void)
 	    vp->v_text[j] = ac;
 
 	vp->v_flag = 0;
+	vp->v_length = 0;
         vscreen[i] = vp;
 
         vp = (VIDEO *) malloc(sizeof(VIDEO)+(term.t_ncol*sizeof(CELL)));
@@ -200,6 +202,7 @@ vtinit(void)
 	    vp->v_text[j] = ac;
 
 	vp->v_flag = 0;
+	vp->v_length = 0;
         pscreen[i] = vp;
     }
 
@@ -271,6 +274,7 @@ vtputc(CELL c)
     vp = vscreen[vtrow];
     ac.c = ' ';
     ac.a = c.a;
+    ac.d = c.d;
 
     if (vtcol >= term.t_ncol) {
 	/*
@@ -367,6 +371,41 @@ vteeol(void)
     vtcol = term.t_ncol;
 }
 
+int
+window_signature_block(WINDOW *wp)
+{
+  LINE *lp, *llp;
+  int in_sig, is_sig_start;
+  int change = 0;
+
+  llp = wp->w_linep;
+  lp = lforw(wp->w_bufp->b_linep);
+
+  do {
+    in_sig = lback(lp) == wp->w_bufp->b_linep ? 0 : lback(lp)->l_sig;
+    if(in_sig == 0){
+       if(llength(lp) == 3){
+	 if(lgetc(lp, 0).c == '-' 
+	     && lgetc(lp, 1).c == '-' 
+	     && lgetc(lp, 2).c == ' '){
+	   in_sig = 1; 
+	   is_sig_start = 1;
+	 }
+       }
+    } else {
+	if(lisblank(lp))
+	   if(is_sig_start == 0) in_sig = 0;
+	is_sig_start = 0;
+    }
+    if(lp->l_sig != in_sig)
+      change++;
+    lp->l_sig = in_sig;
+    lp = lforw(lp);
+  } while(lp != wp->w_bufp->b_linep);
+  wp->w_linep = llp;
+  return change;
+}
+
 
 /*
  * Make sure that the display is right. This is a three part process. First,
@@ -385,7 +424,10 @@ update(void)
     int     i;
     int     j;
     int     scroll = 0;
+    int     repaint= 0;
+    int	    quoted;
     CELL	     c;
+    PCOLORS *pcolors = Pmaster && Pmaster->colors ? Pmaster->colors : Pcolors;
 
 #if	TYPEAH
     if (typahead())
@@ -412,6 +454,10 @@ update(void)
     while (wp != NULL){
         /* Look at any window with update flags set on. */
 
+	if(pcolors && (repaint = window_signature_block(wp))){
+	   sgarbf = TRUE;
+	   wp->w_flag |= WFEDIT | WFHARD;
+	}
         if (wp->w_flag != 0){
             /* If not force reframe, check the framing. */
 
@@ -568,18 +614,54 @@ out:
                     ++i;
                     lp = lforw(lp);
 		}
+		vscreen[i]->v_flag |= (lp->l_sig ? VFSIG : 0)| VFCHG;
+		/* compute physical length of line in screen */
+		vscreen[i]->v_length = 0;
+		for (j = 0; vscreen[i]->v_length < term.t_ncol
+			    && j < llength(lp); ++j){
+		    c = lgetc(lp, j);
+		    if(c.c == '\t'){
+		       vscreen[i]->v_length |= 0x07;
+		       vscreen[i]->v_length++;
+		    }
+		    else if(ISCONTROL(c.c)){
+		       vscreen[i]->v_length += 2;
+		    }
+		    else{
+		       int w;
 
-                vscreen[i]->v_flag |= VFCHG;
+		       w = wcellwidth((UCS) c.c);
+		       vscreen[i]->v_length += (w >= 0 ? w : 1);
+		    }
+		}
                 vtmove(i, 0);
 
                 for (j = 0; j < llength(lp); ++j)
                     vtputc(lgetc(lp, j));
-
                 vteeol();
 	    }
 	    else if ((wp->w_flag & (WFEDIT | WFHARD)) != 0){
                 while (i < wp->w_toprow+wp->w_ntrows){
-                    vscreen[i]->v_flag |= VFCHG;
+                    vscreen[i]->v_flag |= (lp->l_sig ? VFSIG : 0 )| VFCHG;
+		    /* compute physical length of line in screen */
+		    vscreen[i]->v_length = 0;
+		    for (j = 0; vscreen[i]->v_length < term.t_ncol
+			    && j < llength(lp); ++j){
+		        c = lgetc(lp, j);
+		        if(c.c == '\t'){
+		           vscreen[i]->v_length |= 0x07;
+		           vscreen[i]->v_length++;
+		        }
+		        else if(ISCONTROL(c.c)){
+		           vscreen[i]->v_length += 2;
+		        }
+		        else{
+		           int w;
+
+			   w = wcellwidth((UCS) c.c);
+		           vscreen[i]->v_length += (w >= 0 ? w : 1);
+		       }
+		    }
                     vtmove(i, 0);
 
 		    /* if line has been changed */
@@ -717,18 +799,22 @@ out:
 	    movecursor(wheadp->w_toprow, 0);
 	}
 	else{
-	    for (i = 0; i < term.t_nrow-term.t_mrow; ++i){
+	    c.c = ' ';
+	    c.a = 0;
+	    for (i = 0; i < term.t_nrow-term.t_mrow; i++){
 		vscreen[i]->v_flag |= VFCHG;
 		vp1 = pscreen[i];
-		c.c = ' ';
-		c.a = 0;
-		for (j = 0; j < term.t_ncol; ++j)
+		for (j = 0; j < term.t_ncol; j++)
 		  vp1->v_text[j] = c;
+		if(sgarbf == FALSE){
+		   movecursor(i, 0);
+		   term.t_eeol();
+		}
+	    }
+	    if(sgarbf != FALSE){
+	      movecursor(0, 0);	               /* Erase the screen. */
+	      (*term.t_eeop)();
 	    }
-
-	    movecursor(0, 0);	               /* Erase the screen. */
-	    (*term.t_eeop)();
-
 	}
 
         sgarbf = FALSE;				/* Erase-page clears */
@@ -773,7 +859,7 @@ out:
 #endif
             vp2 = pscreen[i];
 
-            updateline(i, &vp1->v_text[0], &vp2->v_text[0], &vp1->v_flag);
+            updateline(i, &vp1->v_text[0], &vp2->v_text[0], &vp1->v_flag, vp1->v_length);
 	}
     }
 
@@ -899,6 +985,164 @@ updext(void)
     }
 }
 
+/* update line color, to be executed to update lines when color is on */
+void
+updatelinecolor (int row, CELL vline[], CELL pline[], short *flags, int len)
+{
+    CELL *cp1, *cp2, *cp3, *cp4, *cp5, *cp6, *cp7;
+    int   nbflag;		/* non-blanks to the right flag? */
+    int   cleartoeol = 0;
+    int   in_quote, quote_found = 0, level;
+    PCOLORS *pcolors = Pmaster && Pmaster->colors ? Pmaster->colors : Pcolors;
+    COLOR_PAIR *qcolor = NULL, *lastc = NULL, *pcolor = NULL;
+    int first = 1, lastattr = -1, change = 0;
+
+    if(pcolors == NULL){
+      updateline(row, vline, pline, flags, len);
+      return;
+    }
+
+    nbflag = FALSE;
+    lastc = pico_get_cur_color();
+
+    /* set up pointers to virtual and physical lines */
+    cp1 = &vline[0];
+    cp2 = &pline[0];
+    cp3 = &vline[term.t_ncol];
+    cp4 = &pline[term.t_ncol];
+
+    if(cellwidth_ptr_to_ptr(cp1, cp3) == cellwidth_ptr_to_ptr(cp2, cp4))
+      while (cp3 != cp1 && cp3[-1].c == cp4[-1].c && cp3[-1].a == cp4[-1].a) {
+	--cp3;
+	--cp4;
+	if (cp3[0].c != ' ' || cp3[0].a != 0)	/* Note if any nonblank */
+	  nbflag = TRUE;			/* in right match. */
+      }
+
+    cp5 = cp3;
+
+    if (nbflag == FALSE && TERM_EOLEXIST) {	/* Erase to EOL ? */
+	while (cp5 != cp1 && cp5[-1].c == ' ' && cp5[-1].a == 0)
+	  --cp5;
+
+	if (cp3-cp5 <= 3)		/* Use only if erase is */
+	  cp5 = cp3;			/* fewer characters. */
+    }
+
+    /* go to start of line */
+    movecursor(row, 0);
+
+    if(cp1 != cp5){
+	int w1, w2;
+
+	w1 = cellwidth_ptr_to_ptr(cp1, cp3);
+	w2 = cellwidth_ptr_to_ptr(cp2, cp2 + (cp3-cp1));
+
+	if(w1 < w2 || (nbflag && w1 != w2)){
+	    if(TERM_EOLEXIST){
+		if(nbflag){
+		    /*
+		     * Draw all of the characters starting with cp1
+		     * until we get to all spaces, then clear to the end of
+		     * line from there. Watch out we don't run over the
+		     * right hand edge, which shouldn't happen.
+		     *
+		     * Set cp5 to the first of the repeating spaces.
+		     */
+		    cp5 = &vline[term.t_ncol];
+		    while (cp5 != cp1 && cp5[-1].c == ' ' && cp5[-1].a == 0)
+		      --cp5;
+		}
+
+		/*
+		 * In the !nbflag case we want spaces from cp5 on.
+		 * Setting cp3 to something different from cp5 triggers
+		 * the clear to end of line below.
+		 */
+		if(cellwidth_ptr_to_ptr(&vline[0], cp5) < term.t_ncol)
+		  cleartoeol++;
+	    }
+	    else{
+		int w;
+
+		/*
+		 * No peeol so draw all the way to the edge whether they
+		 * are spaces or not.
+		 */
+		cp3 = &vline[0];
+		for(w = 0; w < term.t_ncol; cp3++){
+		    int ww;
+
+		    ww = wcellwidth((UCS) cp3->c);
+		    w += (ww >= 0 ? ww : 1);
+		}
+
+		cp5 = cp3;
+	    }
+	}
+    }
+
+    if(row != 0 && len < term.t_ncol)
+      cp5 = cp1 + len;
+
+    in_quote = 1;
+    level = -1;
+    while (cp1 != cp5){		/* Ordinary. */
+	int ww;
+
+	if(lastattr < 0){
+	    lastattr = cp1->a;
+	    change = 0;
+	}
+	else 
+	    change = lastattr != cp1->a;
+	if(first != 0){
+	  first = 0;
+	  if(row == 0)
+	     pico_set_colorp(pcolors->tbcp, PSC_NONE);
+	  else if(row < term.t_nrow - 2)
+	     pcolor = (*flags & VFSIG) ? pcolors->sbcp : pcolors->ntcp;
+	}
+	if(cp1->c != '>' && cp1->c != ' ')
+	   in_quote = 0;
+	else if (in_quote && cp1->c == '>' && pcolors != NULL)
+	   level = (level + 1) % 3;
+	if(level >= 0){
+	   if(level == 0) pcolor = pcolors->qlcp;
+	   else if(level == 1) pcolor = pcolors->qllcp;
+	   else if(level == 2) pcolor = pcolors->qlllcp;
+	}
+	if(cp1->a == 1)
+	   pcolor = pcolors->rtcp;	/* pcolor = proposed color */
+	if(change == 0)
+	  pico_set_colorp(pcolor, PSC_NONE);
+	else
+	  (*term.t_rev)(cp1->a);	/* set inverse for this char */
+
+	if(change == 0)
+	   (*term.t_rev)(cp1->a);	/* set inverse for this char */
+	(*term.t_putchar)(cp1->c);
+
+	ww = wcellwidth((UCS) cp1->c);
+	ttcol += (ww >= 0 ? ww : 1);
+
+	*cp2++ = *cp1++;
+    }
+
+    if (lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+    }
+
+    (*term.t_rev)(0);			/* turn off inverse anyway! */
+
+    if (cp5 != cp3 || cleartoeol)	/* Erase. */
+	peeol();
+
+    *flags &= ~(VFCHG|VFSIG);			/* flag this line is changed */
+}
+
+
 
 /*
  * Update a single line. This does not know how to use insert or delete
@@ -909,7 +1153,8 @@ void
 updateline(int row,			/* row on screen */
 	   CELL vline[],		/* what we want it to end up as */
 	   CELL pline[],		/* what it looks like now       */
-	   short *flags)
+	   short *flags,
+	   int len)
 {
     CELL *cp1, *cp2, *cp3, *cp4, *cp5, *cp6, *cp7;
     int   display = TRUE;
@@ -919,6 +1164,11 @@ updateline(int row,			/* row on screen */
     if(row < 0 || row > term.t_nrow)
       return;
 
+    if((Pmaster && Pmaster->colors) || Pcolors){
+      updatelinecolor(row, vline, pline, flags, len);
+      return;
+    }
+
     /* set up pointers to virtual and physical lines */
     cp1 = &vline[0];
     cp2 = &pline[0];
@@ -1307,6 +1557,201 @@ mlerase(void)
     mpresf = FALSE;
 }
 
+/* returns the chosen dictionary. If one was already chosen
+ * return that one 
+ */
+char *
+speller_choice(char **sp_list, int *choice)
+{
+    int ch_dict = -1;
+    int cnt;
+
+    if(sp_list == NULL || sp_list[0] == NULL || sp_list[0][0] == '\0')
+      return NULL;
+
+    if(choice && *choice >= 0)
+      return sp_list[*choice];
+
+    for(cnt = 0; sp_list[cnt] != NULL && sp_list[cnt][0] != '\0'; cnt++)
+	;
+
+    if(cnt > 10)		/* only the first 10 dictionaries */
+	cnt = 10;
+
+    if(cnt == 1)		/* only one dictionary? choose it! */
+        ch_dict = 0;
+
+    if(ch_dict > cnt - 1)	/* choose again in case something changed */
+	ch_dict = -1;
+
+    if(ch_dict < 0){		/* not a choice yet? do one now! */
+	UCS buf[128];
+	int i;
+	char *utf8_prompt;
+	UCS  *ucs4_prompt;
+	EXTRAKEYS    menu_dictionary[] = {
+		{"0", NULL, '0'},
+		{"1", NULL, '1'},
+		{"2", NULL, '2'},
+		{"3", NULL, '3'},
+		{"4", NULL, '4'},
+		{"5", NULL, '5'},
+		{"6", NULL, '6'},
+		{"7", NULL, '7'},
+		{"8", NULL, '8'},
+		{"9", NULL, '9'}
+	};
+
+	for(i = 0; i < cnt; i++)
+	   menu_dictionary[i].label = sp_list[i];
+
+	if(cnt < 10)
+	  menu_dictionary[cnt].name = NULL;
+
+	buf[0] = '\0';
+	/* write the prompt in utf8, and let internal functions translate it to ucs4 */
+	ucs4_prompt = utf8_to_ucs4_cpystr(_("Choose Dictionary: "));
+
+	i = mlchoose(ucs4_prompt, menu_dictionary);
+
+	if(i >= '0' && i <= '9')
+	  ch_dict = i - '0';
+	
+	if (i == -2) /* user cancelled */
+		ch_dict = -2;
+
+	if(ucs4_prompt)
+	   fs_give((void **)&ucs4_prompt);
+  }
+  else ch_dict = -1;
+
+  if(choice)
+     *choice = ch_dict;
+
+  return  ch_dict >= 0 ? sp_list[ch_dict] : NULL;
+}   
+
+/* just like mlreplyd, but user cannot fill a prompt */
+int
+mlchoose(UCS *prompt, EXTRAKEYS *extras)
+{
+    UCS      c;
+    UCS      buf[NLINE];
+    int      i;
+    int      changed = FALSE;
+    int      return_val = 0;
+    KEYMENU  menu_choose[12];
+    COLOR_PAIR *lastc = NULL;
+
+    for(i = 0; i < 12; i++){
+	menu_choose[i].name = NULL;
+	KS_OSDATASET(&menu_choose[i], KS_NONE);
+    }
+
+    menu_choose[0].name = "^G";
+    menu_choose[0].label = N_("Get Help");
+    KS_OSDATASET(&menu_choose[0], KS_SCREENHELP);
+
+    menu_choose[6].name = "^C";
+    menu_choose[6].label = N_("Cancel");
+    KS_OSDATASET(&menu_choose[6], KS_NONE);
+
+    for(i = 0; i < 10; i++){
+	if((i % 2) == 0){
+	  menu_choose[i / 2 + 1].name = extras[i].name;
+	  menu_choose[i / 2 + 1].label = extras[i].label;
+	}
+	else{
+	  menu_choose[(i + 13) / 2].name = extras[i].name;
+	  menu_choose[(i + 13) / 2].label = extras[i].label;
+	}
+    }
+    wkeyhelp(menu_choose);		/* paint generic menu */
+    sgarbk = TRUE;			/* mark menu dirty */
+    if(Pmaster && curwp)
+      curwp->w_flag |= WFMODE;
+
+    ucs4_strncpy(buf, prompt, NLINE);
+    buf[NLINE-1] = '\0';
+    mlwrite(buf, NULL);
+    if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
+       && pico_is_good_colorpair(Pmaster->colors->prcp)){
+	lastc = pico_get_cur_color();
+	(void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
+    }
+    else
+      (*term.t_rev)(1);
+
+    return_val = -1;
+    while(1){
+	c = GetKey();
+	for(i = 0; i < 10 && extras[i].name != NULL && extras[i].key != c; i++)
+	   ;
+	if(i < 10 && extras[i].name)
+	   return_val = c;
+	else switch(c){
+	  case (CTRL|'C') :		/* Bail out! */
+	  case F2         :
+	    pputs_utf8(_("Cancel"), 1);
+	    return_val = -2;
+	  break;
+
+	  case (CTRL|'G') :
+	    if(term.t_mrow == 0 && km_popped == 0){
+		movecursor(term.t_nrow-2, 0);
+		peeol();
+		term.t_mrow = 2;
+		if(lastc){
+		    (void) pico_set_colorp(lastc, PSC_NONE);
+		    free_color_pair(&lastc);
+		}
+		else
+		  (*term.t_rev)(0);
+
+		wkeyhelp(menu_choose);		/* paint generic menu */
+		mlwrite(buf, NULL);
+		if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
+		   && pico_is_good_colorpair(Pmaster->colors->prcp)){
+		    lastc = pico_get_cur_color();
+		    (void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
+		}
+		else
+		  (*term.t_rev)(1);
+
+		sgarbk = TRUE;			/* mark menu dirty */
+		km_popped++;
+		break;
+	    }
+	    /* else fall through */
+
+	  default:
+	    (*term.t_beep)();
+	  case NODATA :
+	    break;
+	}
+
+	(*term.t_flush)();
+	if (return_val != -1){ /* abort sets rv = -2, other return values are positive */
+	    if(lastc){
+		(void) pico_set_colorp(lastc, PSC_NONE);
+		free_color_pair(&lastc);
+	    }
+	    else
+	      (*term.t_rev)(0);
+
+	    if(km_popped){
+		term.t_mrow = 0;
+		movecursor(term.t_nrow, 0);
+		peeol();
+		sgarbf = 1;
+		km_popped = 0;
+	    }
+
+	    return(return_val);
+	}
+    }
+}
+
 
 int
 mlyesno_utf8(char *utf8prompt, int dflt)
@@ -1338,6 +1783,7 @@ mlyesno(UCS *prompt, int dflt)
     UCS     buf[NLINE], lbuf[10];
     KEYMENU menu_yesno[12];
     COLOR_PAIR *lastc = NULL;
+    PCOLORS *pcolors = Pmaster && Pmaster->colors ? Pmaster->colors : Pcolors;
 
 #ifdef _WINDOWS
     if (mswin_usedialog ()) 
@@ -1371,12 +1817,11 @@ mlyesno(UCS *prompt, int dflt)
     ucs4_strncat(buf, lbuf, NLINE - ucs4_strlen(buf) - 1);
     buf[NLINE-1] = '\0';
     mlwrite(buf, NULL);
-    if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
-       && pico_is_good_colorpair(Pmaster->colors->prcp)){
+    if(pcolors && pcolors->prcp
+       && pico_is_good_colorpair(pcolors->prcp)){
 	lastc = pico_get_cur_color();
-	(void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
-    }
-    else
+	(void) pico_set_colorp(pcolors->prcp, PSC_NONE);
+    } else 
       (*term.t_rev)(1);
 
     rv = -1;
@@ -1426,10 +1871,10 @@ mlyesno(UCS *prompt, int dflt)
 
 		wkeyhelp(menu_yesno);		/* paint generic menu */
 		mlwrite(buf, NULL);
-		if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
-		   && pico_is_good_colorpair(Pmaster->colors->prcp)){
-		    lastc = pico_get_cur_color();
-		    (void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
+		if(pcolors && pcolors->prcp
+		       && pico_is_good_colorpair(pcolors->prcp)){
+		   lastc = pico_get_cur_color();
+		   (void) pico_set_colorp(pcolors->prcp, PSC_NONE);
 		}
 		else
 		  (*term.t_rev)(1);
@@ -1558,6 +2003,8 @@ writeachar(UCS ucs)
  *	      answer already typed in.  Carriage return accepts the
  *	      default.  answer returned in buf which also holds the initial
  *            default, nbuf is its length, def set means use default value.
+ *	      In order to be able to eliminate keys from a menu, EXTRAKEYS
+ *	      always has size 10.
  */
 int
 mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
@@ -1572,6 +2019,7 @@ mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
     UCS	     extra_v[12];
     struct   display_line dline;
     COLOR_PAIR *lastc = NULL;
+    PCOLORS *pcolors = Pmaster && Pmaster->colors ? Pmaster->colors : Pcolors;
 
 #ifdef _WINDOWS
     if(mswin_usedialog()){
@@ -1619,6 +2067,7 @@ mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
     }
 #endif
 
+    memset(&menu_mlreply, 0, 12*sizeof(KEYMENU));
     menu_mlreply[0].name = "^G";
     menu_mlreply[0].label = N_("Get Help");
     KS_OSDATASET(&menu_mlreply[0], KS_SCREENHELP);
@@ -1627,9 +2076,7 @@ mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
 	KS_OSDATASET(&menu_mlreply[i], KS_NONE);
 	rfkm[2*i][1] = 0;
 	if(extras){
-	    for(; extras[j].name && j != 2*(i-1); j++)
-	      ;
-
+	    j = 2*(i-1);
 	    if(extras[j].name){
 		rfkm[2*i][1]	      = extras[j].key;
 		menu_mlreply[i].name  = extras[j].name;
@@ -1646,9 +2093,7 @@ mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
 	menu_mlreply[i].name = NULL;
 	rfkm[2*(i-6)+1][1] = 0;
 	if(extras){
-	    for(; extras[j].name && j != (2*(i-6)) - 1; j++)
-	      ;
-
+	    j = 2*(i-6) - 1;
 	    if(extras[j].name){
 		rfkm[2*(i-6)+1][1]    = extras[j].key;
 		menu_mlreply[i].name  = extras[j].name;
@@ -1693,10 +2138,10 @@ mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
 
     sgarbk = 1;				/* mark menu dirty */
 
-    if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
-       && pico_is_good_colorpair(Pmaster->colors->prcp)){
-	lastc = pico_get_cur_color();
-	(void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
+    if(pcolors && pcolors->prcp
+       && pico_is_good_colorpair(pcolors->prcp)){
+       lastc = pico_get_cur_color();
+       (void) pico_set_colorp(pcolors->prcp, PSC_NONE);
     }
     else
       (*term.t_rev)(1);
@@ -1776,10 +2221,10 @@ mlreplyd(UCS *prompt, UCS *buf, int nbuf, int flg, EXTRAKEYS *extras)
 
 		wkeyhelp(menu_mlreply);		/* paint generic menu */
 		plen = mlwrite(prompt, NULL);		/* paint prompt */
-		if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
-		   && pico_is_good_colorpair(Pmaster->colors->prcp)){
-		    lastc = pico_get_cur_color();
-		    (void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
+		if(pcolors && pcolors->prcp
+		   && pico_is_good_colorpair(pcolors->prcp)){
+		   lastc = pico_get_cur_color();
+		   (void) pico_set_colorp(pcolors->prcp, PSC_NONE);
 		}
 		else
 		  (*term.t_rev)(1);
@@ -1960,6 +2405,7 @@ emlwrite_ucs4(UCS *message, EML *eml)
     UCS  *bufp, *ap;
     int   width;
     COLOR_PAIR *lastc = NULL;
+    PCOLORS *pcolors = Pmaster && Pmaster->colors ? Pmaster->colors : Pcolors;
 
     mlerase();
 
@@ -2004,10 +2450,10 @@ emlwrite_ucs4(UCS *message, EML *eml)
     else
       movecursor(term.t_nrow-term.t_mrow, 0);
 
-    if(Pmaster && Pmaster->colors && Pmaster->colors->stcp
-       && pico_is_good_colorpair(Pmaster->colors->stcp)){
-	lastc = pico_get_cur_color();
-	(void) pico_set_colorp(Pmaster->colors->stcp, PSC_NONE);
+    if(pcolors && pcolors->stcp
+       && pico_is_good_colorpair(pcolors->stcp)){
+	   lastc = pico_get_cur_color();
+	   (void) pico_set_colorp(pcolors->stcp, PSC_NONE);
     }
     else
       (*term.t_rev)(1);
@@ -2094,6 +2540,7 @@ mlwrite(UCS *fmt, void *arg)
     UCS   c;
     char *ap;
     COLOR_PAIR *lastc = NULL;
+    PCOLORS *pcolors = Pmaster && Pmaster->colors ? Pmaster->colors : Pcolors;
 
     /*
      * the idea is to only highlight if there is something to show
@@ -2101,10 +2548,10 @@ mlwrite(UCS *fmt, void *arg)
     mlerase();
     movecursor(ttrow, 0);
 
-    if(Pmaster && Pmaster->colors && Pmaster->colors->prcp
-       && pico_is_good_colorpair(Pmaster->colors->prcp)){
-	lastc = pico_get_cur_color();
-	(void) pico_set_colorp(Pmaster->colors->prcp, PSC_NONE);
+    if(pcolors && pcolors->prcp
+       && pico_is_good_colorpair(pcolors->prcp)){
+	   lastc = pico_get_cur_color();
+	   (void) pico_set_colorp(pcolors->prcp, PSC_NONE);
     }
     else
       (*term.t_rev)(1);
@@ -2913,10 +3360,13 @@ wstripe(int line, int column, char *utf8pmt, int key)
 
       if(klcp && pico_is_good_colorpair(Pmaster->colors->kncp))
         kncp = Pmaster->colors->kncp;
-
-      lastc = pico_get_cur_color();
+    }
+    else if(Pcolors){
+       klcp = Pcolors->klcp;
+       kncp = Pcolors->kncp;
     }
 
+    lastc = pico_get_cur_color();
     ucs4pmt = utf8_to_ucs4_cpystr(utf8pmt);
     l = ucs4_strlen(ucs4pmt);
     while(1){
