@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: file.c 1082 2008-06-12 18:39:50Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -115,12 +115,13 @@ insfile(int f, int n)
     char	 fname[NLINE], dir[NLINE];
     int		 retval, bye = 0, msg = 0;
     char	 prompt[64], *infile;
-    EXTRAKEYS    menu_ins[5];
+    EXTRAKEYS    menu_ins[10];
     EML          eml;
     
     if (curbp->b_mode&MDVIEW) /* don't allow this command if */
       return(rdonly()); /* we are in read only mode */
 
+    memset(&menu_ins, 0, 10*sizeof(EXTRAKEYS));
     fname[0] = dir[0] = '\0';
     while(!bye){
 	/* set up keymenu stuff */
@@ -407,7 +408,7 @@ file_split(char *dirbuf, size_t dirbuflen, char *orig_fname, int is_for_browse)
 		  ? opertree
 		  : ((is_for_browse && browse_dir[0])
 		     ? browse_dir : gethomedir(NULL)),
-		  C_FILESEP, p - orig_fname, orig_fname);
+		  C_FILESEP, (int) (p - orig_fname), orig_fname);
     }
     else{
 	fn = orig_fname;
@@ -566,9 +567,10 @@ filewrite(int f, int n)
         register int    s;
         char            fname[NFILEN];
 	char		shows[NLINE], origshows[NLINE], *bufp;
-	EXTRAKEYS	menu_write[3];
+	EXTRAKEYS	menu_write[10];
 	EML             eml;
 
+	memset(&menu_write, 0, 10*sizeof(EXTRAKEYS));
 	if(curbp->b_fname[0] != 0){
 	  strncpy(fname, curbp->b_fname, sizeof(curbp->b_fname));
 	  curbp->b_fname[sizeof(curbp->b_fname)-1] = '\0';
@@ -643,7 +645,7 @@ filewrite(int f, int n)
 				     ? "."
 				     : ((gmode & MDTREE) || opertree[0])
 					  ? opertree : gethomedir(NULL),
-				  C_FILESEP, p - fname, fname);
+				  C_FILESEP, (int) (p - fname), fname);
 		    }
 		    else{
 			fn = fname;
@@ -1014,7 +1016,7 @@ int
 pico_fncomplete(char *dirarg, char *fn, size_t fnlen)
 {
     char *p, *dlist, tmp[NLINE], dir[NLINE];
-    int   n, i, match = -1;
+    int   n, i, match = -1, orign;
 #ifdef	DOS
 #define	FILECMP(x, y)	(toupper((unsigned char)(x))\
 				== toupper((unsigned char)(y)))
@@ -1025,9 +1027,16 @@ pico_fncomplete(char *dirarg, char *fn, size_t fnlen)
     strncpy(dir, dirarg, sizeof(dir));
     dir[sizeof(dir)-1] = '\0';
     pfnexpand(dir, sizeof(dir));
-    if(*fn && (dlist = p = getfnames(dir, fn, &n, NULL, 0))){
+    if((dlist = p = getfnames(dir, fn, &n, NULL, 0))){
+	orign = n;
 	memset(tmp, 0, sizeof(tmp));
 	while(n--){			/* any names in it */
+	    if(strcmp(p, ".") == 0 || strcmp(p, "..") == 0){
+	      p += strlen(p) + 1;
+	      orign--;
+	      continue;
+	    }
+
 	    for(i = 0; i < sizeof(tmp)-1 && fn[i] && FILECMP(p[i], fn[i]); i++)
 	      ;
 
@@ -1052,6 +1061,9 @@ pico_fncomplete(char *dirarg, char *fn, size_t fnlen)
 	free(dlist);
     }
 
+    if(fn[0] == '\0' && orign != 1)
+	match = -1;
+
     if(match >= 0){
 	strncpy(fn, tmp, fnlen);
 	fn[fnlen-1] = '\0';
