@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: line.c 769 2007-10-24 00:15:40Z hubert@u.washington.
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -90,6 +90,7 @@ lalloc(int used)
 
     lp->l_size = size;
     lp->l_used = used;
+    lp->l_sig  = 0;	/* assume it is not a signature line */
     return (lp);
 }
 
@@ -617,7 +618,9 @@ lisblank(LINE *line)
 	  ? ucs4_strlen(qstr) : 0;
 
     for(; n < llength(line); n++)
-      if(!ucs4_isspace(lgetc(line, n).c))
+      if(!ucs4_isspace(lgetc(line, n).c) 
+	  || lgetc(line, n).c >= 0xff 
+	  || (unsigned char) lgetc(line,n).c != NBSPC)
 	return(FALSE);
 
     return(TRUE);
