@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: spell.c 854 2007-12-07 17:44:43Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -285,7 +285,7 @@ movetoword(UCS *w)
     olddotp = curwp->w_dotp;
 
     curwp->w_bufp->b_mode |= MDEXACT;		/* case sensitive */
-    while(forscan(&i, w, 0, NULL, 0, 1) == TRUE){
+    while(forscan(&i, w, SR_FORWARD, NULL, 0, 1) == TRUE){
 	if(i)
 	  break;				/* wrap NOT allowed! */
 
