@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: pico.c 921 2008-01-31 02:09:25Z hubert@u.washington.
 /*
  * ========================================================================
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -107,17 +107,19 @@ void
 remove_directions_mark(void)
 {
    LINE *lp;
-   int i, ll;
+   int i;
    UCS c;
 
    for(lp = lforw(curbp->b_linep); lp != curbp->b_linep; lp = lforw(lp)){
-      ll = llength(lp);
-      for(i = 0; i < ll;){
+      for(i = 0; i < llength(lp);){
 	 c = lgetc(lp, i).c;
 	 if(c == 0x200E || c == 0x200F){
 	    curwp->w_dotp = lp;
 	    curwp->w_doto = i;
-	    forwdel(FALSE, 1);
+	    if(i < llength(lp))
+	       forwdel(FALSE, 1);
+	    else
+	      llength(lp)--;
 	    direction = c == 0x200E ? 0 : 1;
 	 }
 	 else
@@ -163,6 +165,10 @@ pico(PICO *pm)
     gmode        |= pm->pine_flags;	/* high 4 bits rsv'd for pine */
 
     alt_speller   = pm->alt_spell;
+#ifdef _WINDOWS
+    dictionary	  = pm->dict;
+    chosen_dict   = pm->chosen_dict;
+#endif /* _WINDOWS */
     pico_all_done = 0;
     km_popped     = 0;
 
@@ -952,12 +958,13 @@ stripwhitespace(void)
       if(!(cur_line->l_used == 3 && 
 	   lgetc(cur_line, 0).c == '-' &&
 	   lgetc(cur_line, 1).c == '-' &&
-	   lgetc(cur_line, 2).c == ' '))
+	   lgetc(cur_line, 2).c == ' ')){
 	for(i = cur_line->l_used - 1; i >= 0; i--)
 	  if(ucs4_isspace(lgetc(cur_line, i).c))
 	    cur_line->l_used--;
 	  else
 	    break;
+      }
     }while((cur_line = lforw(cur_line)) != curbp->b_linep);
     return 0;
 }
