@@ -3,7 +3,7 @@
  *
  * ========================================================================
  * Copyright 2006-2009 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -159,11 +159,17 @@ typedef struct pico_atmt {
  * Structure to contain color options
  */
 typedef struct pico_colors {
+    COLOR_PAIR *ntcp;                   /* normal text color pair */
+    COLOR_PAIR *rtcp;                   /* reverse text color pair */
     COLOR_PAIR *tbcp;                   /* title bar color pair */
     COLOR_PAIR *klcp;                   /* key label color pair */
     COLOR_PAIR *kncp;                   /* key name color pair  */
     COLOR_PAIR *stcp;                   /* status color pair    */
     COLOR_PAIR *prcp;                   /* prompt color pair    */
+    COLOR_PAIR *qlcp;			/* quote level 1 pair	*/
+    COLOR_PAIR *qllcp;			/* quote level 2 pair	*/
+    COLOR_PAIR *qlllcp;			/* quote level 3 pair	*/
+    COLOR_PAIR *sbcp;                   /* signature block color pair */
 } PCOLORS;
 
 /*
@@ -190,6 +196,10 @@ typedef struct pico_struct {
     char  *ctrlr_label;			/* Label for ^R in keymenu */
     char  *alt_spell;			/* Checker to use other than "spell" */
     char **alt_ed;			/* name of alternate editor or NULL */
+#ifdef _WINDOWS
+    char  **dict;			/* list of dictionaries to choose  */
+    int   chosen_dict;			/* chosen default dictionary       */
+#endif /* WINDOWS */
     UCS   *wordseps;			/* word separator characters other than space */
     int    fillcolumn;			/* where to wrap */
     int    menu_rows;			/* number of rows in menu (0 or 2) */
@@ -379,7 +389,7 @@ typedef struct  KBSTREE {
  */
 #ifdef	maindef
 PICO	*Pmaster = NULL;		/* composer specific stuff */
-char	*version = "5.07";		/* PICO version number */
+char	*version = "5.09";		/* PICO version number */
 
 #else
 extern	PICO *Pmaster;			/* composer specific stuff */
