@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: search.c 1266 2009-07-14 18:39:12Z hubert@u.washingt
 
 /*
  * ========================================================================
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -50,6 +50,47 @@ char   *sucs4_to_utf8_cpystr(UCS *, int);
 			    return(RV);			\
 			}
 
+/* The search menu leaves a number of keys free, some are taken
+ * as subcommands of the search command, and some are taken are
+ * editing commands. This leaves the following keys open:
+ * ^J, ^N, ^O, ^P, ^R, ^T, ^U, ^V, ^W, ^X and ^Y.
+ * Out of these keys, ^J, ^N, ^P and ^X are not defined as commands, however,
+ * in some patches ^N, ^P and ^X are defined. ^N is defined as part of
+ * an editing command, ^P and ^X are defined to delete paragraphs and to
+ * remove text to the end of file, so only ^J is undefined.
+ */
+
+#define REPLACE_KEY	2	/* the location of the replace key in the array below */
+
+EXTRAKEYS    menu_srchpat[] = {
+	{"^Y", N_("FirstLine"),	    (CTRL|'Y')},
+	{"^V", N_("LastLine"), 	    (CTRL|'V')},
+	{"^R", N_("No Replace"),    (CTRL|'R')},
+	{"^^", N_("Optns Menu"),    (CTRL|'^')},  /* toggle this menu or options menu */
+	{"^T", N_("LineNumber"),    (CTRL|'T')},
+	{"^W", N_("Start of Para"), (CTRL|'W')},
+	{"^O", N_("End of Para"),   (CTRL|'O')},
+	{"^U", N_("FullJustify"),   (CTRL|'U')},
+	{NULL, NULL, 0},
+	{NULL, NULL, 0}
+};
+
+#define EXACTSR_KEY     1	/* toggle an exact or approximate search */
+#define BGNLINE_KEY	3	/* the location of Bgn Line command  in the array below */
+#define ENDLINE_KEY	4	/* the location of End Line command  in the array below */
+#define BSEARCH_KEY	5	/* the location of the bsearch key in the array below */
+EXTRAKEYS    menu_srchopt[] = {
+	{"^^", N_("Orig Menu"),  (CTRL|'^')},	/* toggle original or options menu */
+	{"^X", N_("Exact"),      (CTRL|'X')},	/* toggle exact vs non exact */
+	{"^R", N_("No Replace"), (CTRL|'R')},	/* toggle replace or not replace */
+	{"^V", N_("Bgn Line"),   (CTRL|'V')},	/* toggle Bgn Line or anywhere */
+	{"^N", N_("End Line"),   (CTRL|'N')},	/* toggle End Line or anywhere */
+	{"^P", N_("BackSearch"), (CTRL|'P')},	/* toggle Backward or forward */
+	{NULL, NULL, 0},
+	{NULL, NULL, 0},
+	{NULL, NULL, 0},
+	{NULL, NULL, 0}
+};
 
 /*
  * Search forward. Get a search string from the user, and search, beginning at
@@ -114,7 +155,7 @@ eq(UCS bc, UCS pc)
 int
 forwsearch(int f, int n)
 {
-  int              status, bsearch;
+  int              status, flags;
   int              wrapt = FALSE, wrapt2 = FALSE;
   int              repl_mode = FALSE;
   UCS              defpat[NPAT];
@@ -130,15 +171,26 @@ forwsearch(int f, int n)
       FWS_RETURN(0);
 
     defpat[0] = '\0';
-    bsearch = 0;		/* search forward by default */
+	/* defaults: usual menu, search forward, not case sensitive */
+
+    flags = SR_ORIGMEN | SR_FORWARD;
+
+    /* exact search is sticky -- that is, once one is done, so will be
+     * the next ones. This is consistent with all all searches being
+     * case insensitive by default.
+     */
+    if((curwp->w_bufp->b_mode & MDEXACT) == 0)
+	flags |= SR_NOEXACT;
+    else
+	flags |= SR_EXACTSR;
 
     /* ask the user for the text of a pattern */
     while(1){
 
 	if (gmode & MDREPLACE)
-	  status = srpat("Search", defpat, NPAT, repl_mode, bsearch);
+	  status = srpat("Search", defpat, NPAT, repl_mode, flags);
 	else
-	  status = readpattern("Search", TRUE, bsearch);
+	  status = readpattern("Search", TRUE, flags);
 
 	switch(status){
 	  case TRUE:                         /* user typed something */
@@ -164,48 +216,112 @@ forwsearch(int f, int n)
 	    pico_refresh(FALSE, 1);
 	    update();
 	    break;
+	    
 
 	  case (CTRL|'P'):
-	    bsearch = bsearch == 0 ? 1 : 0;
+	    if(flags & SR_ORIGMEN){
+		/* Undefined still */
+	    }
+	    if(flags & SR_OPTNMEN){
+	      if(flags & SR_FORWARD){
+		flags &= ~SR_FORWARD;
+		flags |=  SR_BACKWRD;
+	      } else {
+		flags &= ~SR_BACKWRD;
+		flags |=  SR_FORWARD;
+	      }
+	    }
 	    break;
 
 	  case  (CTRL|'V'):
-	    gotoeob(0, 1);
-	    mlerase();
-	    FWS_RETURN(TRUE);
+	    if(flags & SR_ORIGMEN){
+	       gotoeob(0, 1);
+	       mlerase();
+	       FWS_RETURN(TRUE);
+	    } else if (flags & SR_OPTNMEN){
+		if(flags & SR_ENDLINE)
+		  flags &= ~SR_ENDLINE;
+		if(flags & SR_BEGLINE)
+		  flags &= ~SR_BEGLINE;
+		else
+		  flags |= SR_BEGLINE;
+	    }
+	    break;
+
+	  case  (CTRL|'N'):
+	    if(flags & SR_ORIGMEN){
+		/* undefined still */
+	    } else if (flags & SR_OPTNMEN){
+		if(flags & SR_BEGLINE)
+		  flags &= ~SR_BEGLINE;
+		if(flags & SR_ENDLINE)
+		  flags &= ~SR_ENDLINE;
+		else
+		  flags |= SR_ENDLINE;
+	    }
+	    break;
 
 	  case (CTRL|'Y'):
-	    gotobob(0, 1);
-	    mlerase();
-	    FWS_RETURN(TRUE); 
-
-	  case (CTRL|'T') :
-	    switch(status = readnumpat(_("Search to Line Number : "))){
-	      case -1 :
-		emlwrite(_("Search to Line Number Cancelled"), NULL);
-		FWS_RETURN(FALSE);
-
-	      case  0 :
-		emlwrite(_("Line number must be greater than zero"), NULL);
-		FWS_RETURN(FALSE);
-
-	      case -2 :
-		emlwrite(_("Line number must contain only digits"), NULL);
-		FWS_RETURN(FALSE);
-		
-	      case -3 :
-		continue;
-
-	      default :
-		gotoline(0, status);
-		mlerase();
-		FWS_RETURN(TRUE);
+	    if(flags & SR_ORIGMEN){
+	      gotobob(0, 1);
+	      mlerase();
+	      FWS_RETURN(TRUE);
 	    }
 
+	  case (CTRL|'^'):
+	    if (flags & SR_ORIGMEN){
+		flags &= ~SR_ORIGMEN;
+		flags |=  SR_OPTNMEN;
+	    } else {
+		flags &= ~SR_OPTNMEN;
+		flags |=  SR_ORIGMEN;
+	    }
+	    break;
+
+	  case (CTRL|'X'):
+	    if(flags & SR_OPTNMEN){
+	      if (flags & SR_NOEXACT){
+		flags &= ~SR_NOEXACT;
+		flags |=  SR_EXACTSR;
+	      } else {
+		flags &= ~SR_EXACTSR;
+		flags |=  SR_NOEXACT;
+	      }
+	      if((curwp->w_bufp->b_mode & MDEXACT) == 0)
+		curwp->w_bufp->b_mode |= MDEXACT;
+	      else
+		curwp->w_bufp->b_mode &= ~MDEXACT;
+	    }
+	    break;
+
+	  case (CTRL|'T') :
+	    if(flags & SR_ORIGMEN){
+	      switch(status = readnumpat(_("Search to Line Number : "))){
+	        case -1 :
+		  emlwrite(_("Search to Line Number Cancelled"), NULL);
+		  FWS_RETURN(FALSE);
+
+	        case  0 :
+		  emlwrite(_("Line number must be greater than zero"), NULL);
+		  FWS_RETURN(FALSE);
+
+	        case -2 :
+		  emlwrite(_("Line number must contain only digits"), NULL);
+		  FWS_RETURN(FALSE);
+		
+	        case -3 :
+		  continue;
+
+	        default :
+		  gotoline(0, status);
+		  mlerase();
+		  FWS_RETURN(TRUE);
+	      }
+	    }
 	    break;
 
 	  case  (CTRL|'W'):
-	    {
+	    if(flags & SR_ORIGMEN){
 		LINE *linep = curwp->w_dotp;
 		int   offset = curwp->w_doto;
 
@@ -224,24 +340,29 @@ forwsearch(int f, int n)
 		    gotobop(0, 1);
 		    gotobol(0, 1);
 		}
+		mlerase();
+		FWS_RETURN(TRUE);
 	    }
-
-	    mlerase();
-	    FWS_RETURN(TRUE);
+	    break;
 
 	  case  (CTRL|'O'):
-	    if(curwp->w_dotp != curbp->b_linep){
+	    if(flags & SR_ORIGMEN){
+	      if(curwp->w_dotp != curbp->b_linep){
 		gotoeop(0, 1);
 		forwchar(0, 1);
+	      }
+	      mlerase();
+	      FWS_RETURN(TRUE);
 	    }
-
-	    mlerase();
-	    FWS_RETURN(TRUE);
+	    break;
 
 	  case (CTRL|'U'):
-	    fillbuf(0, 1);
-	    mlerase();
-	    FWS_RETURN(TRUE);
+	    if(flags & SR_ORIGMEN){
+	      fillbuf(0, 1);
+	      mlerase();
+	      FWS_RETURN(TRUE);
+	    }
+	    break;
 
 	  case  (CTRL|'R'):        /* toggle replacement option */
 	    repl_mode = !repl_mode;
@@ -269,7 +390,7 @@ forwsearch(int f, int n)
 	}
     }
 
-    reverse_all(defpat, bsearch);
+    reverse_all(defpat, flags & SR_BACKWRD);
 
     /*
      * This code is kind of dumb.  What I want is successive C-W 's to 
@@ -295,7 +416,7 @@ forwsearch(int f, int n)
     /* search for the pattern */
     
     while (n-- > 0) {
-	if((status = forscan(&wrapt,defpat, bsearch, NULL,0,PTBEG)) == FALSE)
+	if((status = forscan(&wrapt,defpat, flags, NULL,0,PTBEG)) == FALSE)
 	  break;
     }
 
@@ -306,7 +427,7 @@ forwsearch(int f, int n)
 
       x[0] = '\0';
 
-      utf8 = sucs4_to_utf8_cpystr(defpat ? defpat : x, bsearch); 
+      utf8 = sucs4_to_utf8_cpystr(defpat ? defpat : x, flags & SR_BACKWRD); 
       /* TRANSLATORS: reporting the result of a failed search */
       eml.s = utf8;
       emlwrite(_("\"%s\" not found"), &eml);
@@ -314,7 +435,7 @@ forwsearch(int f, int n)
 	fs_give((void **) &utf8);
     }
     else if((gmode & MDREPLACE) && repl_mode == TRUE){
-        status = replace_pat(defpat, &wrapt2, bsearch);    /* replace pattern */
+        status = replace_pat(defpat, &wrapt2, flags & SR_BACKWRD);    /* replace pattern */
 	if (wrapt == TRUE || wrapt2 == TRUE){
 	    eml.s = (status == ABORT) ? "cancelled but wrapped" : "Wrapped";
 	    emlwrite("Replacement %s", &eml);
@@ -327,7 +448,7 @@ forwsearch(int f, int n)
 	emlwrite("", NULL);
     }
 
-    reverse_all(defpat, bsearch);
+    reverse_all(defpat, flags & SR_BACKWRD);
     if(curwp->w_doto == -1){
       curwp->w_doto  = 0;
       curwp->w_flag |= WFMOVE;
@@ -342,25 +463,29 @@ replace_pat(UCS *defpat, int *wrapt, int bsearch)
 {
   register         int status;
   UCS              lpat[NPAT], origpat[NPAT];	/* case sensitive pattern */
-  EXTRAKEYS        menu_pat[2];
+  EXTRAKEYS        menu_pat[12];
   int              repl_all = FALSE;
   UCS             *b;
   char             utf8tmp[NPMT];
   UCS              prompt[NPMT];
   UCS             *promptp;
+  int		   i, flags;
 
-    if(bsearch)
+    if(bsearch){
+	flags = SR_BACKWRD;
 	curwp->w_doto -= ucs4_strlen(defpat) - 1;
-    forscan(wrapt, defpat, bsearch, NULL, 0, PTBEG);    /* go to word to be replaced */
+    }
+    else flags = SR_FORWARD;
+
+    forscan(wrapt, defpat, flags, NULL, 0, PTBEG);    /* go to word to be replaced */
 
     lpat[0] = '\0';
-
+    memset((void *)&menu_pat, 0, sizeof(menu_pat));
     /* additional 'replace all' menu option */
     menu_pat[0].name  = "^X";
     menu_pat[0].key   = (CTRL|'X');
     menu_pat[0].label = N_("Repl All");
     KS_OSDATASET(&menu_pat[0], KS_NONE);
-    menu_pat[1].name  = NULL;
 
     while(1) {
 
@@ -558,18 +683,23 @@ replace_all(UCS *orig, UCS *repl, int bsearch)
   LINE		  *stop_line   = curwp->w_dotp;
   int		   stop_offset = curwp->w_doto;
   EML              eml;
+  int		   flags;
 
   /* similar to replace_pat. When we come here, if bsearch is set,
    * the cursor is at the end of the match, so we must bring it back
    * to the beginning.
    */
   if(bsearch){
+    flags = SR_BACKWRD;
     curwp->w_doto -= ucs4_strlen(orig) - 1;
     curwp->w_flag |= WFMOVE;
   }
+  else
+    flags = SR_FORWARD;
+
   stop_offset = curwp->w_doto;
   while (1)
-    if (forscan(&wrapt, orig, bsearch, stop_line, stop_offset, PTBEG)){
+    if (forscan(&wrapt, orig, flags, stop_line, stop_offset, PTBEG)){
         curwp->w_flag |= WFMOVE;            /* put cursor back */
 
 	reverse_all(orig, bsearch); /* undo reverse buffer and pattern */
@@ -653,63 +783,65 @@ replace_all(UCS *orig, UCS *repl, int bsearch)
 
 /* Read a replacement pattern.  Modeled after readpattern(). */
 int
-srpat(char *utf8prompt, UCS *defpat, size_t defpatlen, int repl_mode, int bsearch)
+srpat(char *utf8prompt, UCS *defpat, size_t defpatlen, int repl_mode, int flags)
 {
 	register int s;
 	int	     i = 0;
+	int	     toggle, bsearch, bol, eol, exact;
 	UCS         *b;
 	UCS	     prompt[NPMT];
 	UCS         *promptp;
-	EXTRAKEYS    menu_pat[9];
+	EXTRAKEYS    menu_pat[12];
 
-	menu_pat[i = 0].name = "^Y";
-	menu_pat[i].label    = N_("FirstLine");
-	menu_pat[i].key	     = (CTRL|'Y');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
+	bsearch = flags & SR_BACKWRD;
+	bol	= flags & SR_BEGLINE;
+	eol	= flags & SR_ENDLINE;
+	exact   = flags & SR_EXACTSR;
+	toggle  = 0;	/* reserved for future use */
 
-	menu_pat[++i].name = "^V";
-	menu_pat[i].label  = N_("LastLine");
-	menu_pat[i].key	   = (CTRL|'V');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	menu_pat[++i].name = "^R";
-	menu_pat[i].label  = repl_mode ? N_("No Replace") : N_("Replace");
-	menu_pat[i].key	   = (CTRL|'R');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	if(!repl_mode){
-	    menu_pat[++i].name = "^T";
-	    menu_pat[i].label  = N_("LineNumber");
-	    menu_pat[i].key    = (CTRL|'T');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	    menu_pat[++i].name = "^W";
-	    /* TRANSLATORS: Start of paragraph */
-	    menu_pat[i].label  = N_("Start of Para");
-	    menu_pat[i].key    = (CTRL|'W');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	    menu_pat[++i].name = "^O";
-	    menu_pat[i].label  = N_("End of Para");
-	    menu_pat[i].key    = (CTRL|'O');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	    menu_pat[++i].name = "^U";
-	    /* TRANSLATORS: Instead of justifying (formatting) just a
-	       single paragraph, Full Justify justifies the entire
-	       message. */
-	    menu_pat[i].label  = N_("FullJustify");
-	    menu_pat[i].key    = (CTRL|'U');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+	memset(&menu_pat, 0, sizeof(menu_pat));
+	/* add exceptions here based on the location of the items in the menu */
+	for(i = 0; i < 10; i++){
+	    if(flags & SR_ORIGMEN){
+	      menu_pat[i].name  = menu_srchpat[10*toggle + i].name;
+	      menu_pat[i].label = menu_srchpat[10*toggle + i].label;
+	      menu_pat[i].key   = menu_srchpat[10*toggle + i].key;
+	      if(toggle == 0){
+		if (i == REPLACE_KEY)
+		   menu_pat[i].label  = repl_mode ? N_("No Replace")
+						  : N_("Replace");
+	      }
+	    } else if(flags & SR_OPTNMEN){
+	      menu_pat[i].name  = menu_srchopt[i].name;
+	      menu_pat[i].label = menu_srchopt[i].label;
+	      menu_pat[i].key   = menu_srchopt[i].key;
+	      switch(i){
+		case EXACTSR_KEY:
+			menu_pat[i].label = exact ? N_("No Exact")
+						  : N_("Exact");
+			break;
+		case REPLACE_KEY:
+			menu_pat[i].label  = repl_mode ? N_("No Replace")
+						       : N_("Replace");
+			break;
+		case BSEARCH_KEY: 
+			menu_pat[i].label  = bsearch ? N_("Srch Fwd")
+						     : N_("Srch Back");
+			break;
+		case BGNLINE_KEY:
+			menu_pat[i].label  = bol ? N_("Anywhere")
+						 : N_("Bgn Line"); 
+			break;
+		case ENDLINE_KEY:
+			menu_pat[i].label  = eol ? N_("Anywhere")
+						 : N_("End Line"); 
+			break;
+		default : break;
+	      }
+	      if(menu_pat[i].name)
+	        KS_OSDATASET(&menu_pat[i], KS_NONE);
+	   }
 	}
-
-	menu_pat[++i].name = "^P";
-	menu_pat[i].label  = bsearch ? N_("Forward") : N_("Backward") ;
-	menu_pat[i].key    = (CTRL|'P');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	menu_pat[++i].name = NULL;
-
 	b = utf8_to_ucs4_cpystr(utf8prompt);
 	if(b){
 	    ucs4_strncpy(prompt, b, NPMT);
@@ -720,6 +852,23 @@ srpat(char *utf8prompt, UCS *defpat, size_t defpatlen, int repl_mode, int bsearc
 	      if(b) ucs4_strncat(prompt, b, ucs4_strlen(b));
 	      prompt[NPMT-1] = '\0';
 	    }
+	    if(bol){
+	      fs_give((void **) &b);
+	      b = utf8_to_ucs4_cpystr(N_(" at start of line"));
+	      if(b) ucs4_strncat(prompt, b, ucs4_strlen(b));
+	      prompt[NPMT-1] = '\0';
+	    } else if(eol){
+	      fs_give((void **) &b);
+	      b = utf8_to_ucs4_cpystr(N_(" at end of line"));
+	      if(b) ucs4_strncat(prompt, b, ucs4_strlen(b));
+	      prompt[NPMT-1] = '\0';
+	    }
+	    if(exact){
+	      fs_give((void **) &b);
+	      b = utf8_to_ucs4_cpystr(N_(" exactly for"));
+	      if(b) ucs4_strncat(prompt, b, ucs4_strlen(b));
+	      prompt[NPMT-1] = '\0';
+	    }
 	    if(b) fs_give((void **) &b);
 	}
 
@@ -794,8 +943,9 @@ readnumpat(char *utf8prompt)
 {
     int		 i, n;
     char	 numpat[NPMT];
-    EXTRAKEYS    menu_pat[2];
+    EXTRAKEYS    menu_pat[12];
 
+    memset(&menu_pat, 0, 10*sizeof(EXTRAKEYS));
     menu_pat[i = 0].name  = "^T";
     menu_pat[i].label	  = N_("No Line Number");
     menu_pat[i].key	  = (CTRL|'T');
@@ -833,54 +983,63 @@ readnumpat(char *utf8prompt)
 
 
 int
-readpattern(char *utf8prompt, int text_mode, int bsearch)
+readpattern(char *utf8prompt, int text_mode, int flags)
 {
 	register int s;
 	int	     i;
+	int	     toggle, bsearch, bol, eol, exact;
 	UCS         *b;
 	UCS	     tpat[NPAT+20];
 	UCS         *tpatp;
-	EXTRAKEYS    menu_pat[8];
+	EXTRAKEYS    menu_pat[12];
 
-	menu_pat[i = 0].name = "^Y";
-	menu_pat[i].label    = N_("FirstLine");
-	menu_pat[i].key	     = (CTRL|'Y');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
+	bsearch = flags & SR_BACKWRD;
+	bol	= flags & SR_BEGLINE;
+	eol	= flags & SR_ENDLINE;
+	exact   = flags & SR_EXACTSR;
+	toggle  = 0;	/* reserved for future use */
 
-	menu_pat[++i].name = "^V";
-	menu_pat[i].label  = N_("LastLine");
-	menu_pat[i].key	   = (CTRL|'V');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	if(text_mode){
-	    menu_pat[++i].name = "^T";
-	    menu_pat[i].label  = N_("LineNumber");
-	    menu_pat[i].key    = (CTRL|'T');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	    menu_pat[++i].name = "^W";
-	    menu_pat[i].label  = N_("Start of Para");
-	    menu_pat[i].key    = (CTRL|'W');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	    menu_pat[++i].name = "^O";
-	    menu_pat[i].label  = N_("End of Para");
-	    menu_pat[i].key    = (CTRL|'O');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	    menu_pat[++i].name = "^U";
-	    menu_pat[i].label  = N_("FullJustify");
-	    menu_pat[i].key    = (CTRL|'U');
-	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+	memset(&menu_pat, 0, sizeof(menu_pat));
+	/* add exceptions here based on the location of the items in the menu */
+	for(i = 0; i < 10; i++){
+	    if(flags & SR_ORIGMEN){
+	      menu_pat[i].name  = menu_srchpat[10*toggle + i].name;
+	      menu_pat[i].label = menu_srchpat[10*toggle + i].label;
+	      menu_pat[i].key   = menu_srchpat[10*toggle + i].key;
+	      if(toggle == 0){
+		if (i == REPLACE_KEY)
+		   memset(&menu_pat[i], 0, sizeof(EXTRAKEYS));
+		if (i > REPLACE_KEY && !text_mode)
+		   memset(&menu_pat[i], 0, sizeof(EXTRAKEYS));
+	      }
+	    } else if(flags & SR_OPTNMEN){
+	      menu_pat[i].name  = menu_srchopt[i].name;
+	      menu_pat[i].label = menu_srchopt[i].label;
+	      menu_pat[i].key   = menu_srchopt[i].key;
+	      switch(i){
+		case EXACTSR_KEY:
+			menu_pat[i].label  = exact ? N_("No Exact")
+						   : N_("Exact");
+			break;
+		case BSEARCH_KEY: 
+			menu_pat[i].label  = bsearch ? N_("Srch Fwd")
+						     : N_("Srch Back");
+			break;
+		case BGNLINE_KEY:
+			menu_pat[i].label  = bol ? N_("Anywhere")
+						 : N_("Bgn Line"); 
+			break;
+		case ENDLINE_KEY:
+			menu_pat[i].label  = eol ? N_("Anywhere")
+						 : N_("End Line"); 
+			break;
+		default : break;
+	     }
+	     if(menu_pat[i].name)
+	       KS_OSDATASET(&menu_pat[i], KS_NONE);
+	   }
 	}
 
-	menu_pat[++i].name = "^P";
-	menu_pat[i].label  = bsearch ? N_("Forward") : N_("Backward") ;
-	menu_pat[i].key    = (CTRL|'P');
-	KS_OSDATASET(&menu_pat[i], KS_NONE);
-
-	menu_pat[++i].name = NULL;
-
 	b = utf8_to_ucs4_cpystr(utf8prompt);
 	if(b){
 	    ucs4_strncpy(tpat, b, NPAT+20);
@@ -891,6 +1050,23 @@ readpattern(char *utf8prompt, int text_mode, int bsearch)
 	      if(b) ucs4_strncat(tpat, b, ucs4_strlen(b));
 	      tpat[NPAT+20-1] = '\0';
 	    }
+	    if(bol){
+	      fs_give((void **) &b);
+	      b = utf8_to_ucs4_cpystr(N_(" at start of line"));
+	      if(b) ucs4_strncat(tpat, b, ucs4_strlen(b));
+	      tpat[NPAT+20-1] = '\0';
+	    } else if (eol){
+	      fs_give((void **) &b);
+	      b = utf8_to_ucs4_cpystr(N_(" at end of line"));
+	      if(b) ucs4_strncat(tpat, b, ucs4_strlen(b));
+	      tpat[NPAT+20-1] = '\0';
+	    }
+	    if (exact){
+	      fs_give((void **) &b);
+	      b = utf8_to_ucs4_cpystr(N_(" exactly for"));
+	      if(b) ucs4_strncat(tpat, b, ucs4_strlen(b));
+	      tpat[NPAT+20-1] = '\0';
+	    } 
 	    if(b) fs_give((void **) &b);
 	}
 
@@ -1010,7 +1186,7 @@ void reverse_ucs4(UCS *s)
 int
 forscan(int *wrapt,	/* boolean indicating search wrapped */
 	UCS *patrn,	/* string to scan for */
-	int bsearch,	/* search backwards */
+	int flags,      /* direction and position */
 	LINE *limitp,	/* stop searching if reached */
 	int limito,	/* stop searching if reached */
 	int leavep)	/* place to leave point
@@ -1028,7 +1204,14 @@ forscan(int *wrapt,	/* boolean indicating search wrapped */
     UCS *patptr;	/* pointer into pattern */
     int stopoff;	/* offset to stop search */
     LINE *stopline;	/* line to stop search */
+    int ftest;		/* position of first character of test */
+    int bsearch;
+    int bol;
+    int eol;
 
+    bsearch = flags & SR_BACKWRD;
+    bol	= flags & SR_BEGLINE;
+    eol	= flags & SR_ENDLINE;
     *wrapt = FALSE;
 
     /* if bsearch is set we return the match at the beginning of the
@@ -1091,8 +1274,20 @@ forscan(int *wrapt,	/* boolean indicating search wrapped */
 	else
 	    c = lgetc(curline, curoff++).c;	/* get the char */
 
+	if(bol)
+	   ftest = bsearch == 0 ? 1 : llength(curline) - ucs4_strlen(patrn) + 1;
+	else if (eol)
+	   ftest = bsearch == 0 ? llength(curline) - ucs4_strlen(patrn) + 1 : 1;
 	/* test it against first char in pattern */
-	if (eq(c, patrn[0]) != FALSE) {	/* if we find it..*/
+	if (eq(c, patrn[0]) != FALSE 		/* if we find it..*/
+	       && ((bol == 0 && eol == 0)	/* ...and if it is anywhere */
+	            || (bol != 0		/* ...or is at the begin or line */
+		   	&& ((bsearch == 0 && curoff == ftest)	/* and search forward and found at beginning of line */
+			    || (bsearch != 0 && curoff == ftest))) /* or search backward and found at end of line */
+	            || (eol != 0		/* ...or is at the end or line */
+		   	&& ((bsearch == 0 && curoff == ftest)	/* and search forward and found at beginning of line */
+			    || (bsearch != 0 && curoff == ftest))) /* or search backward and found at end of line */
+		  )){
 	    /* setup match pointers */
 	    matchline = curline;
 	    matchoff = curoff;
