@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: utf8.c 1184 2008-12-16 23:52:15Z hubert@u.washington
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,8 @@ static char rcsid[] = "$Id: utf8.c 1184 2008-12-16 23:52:15Z hubert@u.washington
 #ifdef _WINDOWS
 /* wingdi.h uses ERROR (!) and we aren't using the c-client ERROR so... */
 #undef ERROR
+#else
+#define _XOPEN_SOURCE
 #endif
 
 #include <system.h>
@@ -98,6 +100,16 @@ wcellwidth(UCS ucs)
 #endif
 }
 
+/* ambiguous width zone character function */
+int
+pith_ucs4width(UCS ucs)
+{
+#ifndef _WINDOWS
+  return wcwidth((wchar_t) ucs);
+#else
+  return (ucs >= 0x2100) ? 2 : 1; 
+#endif /* _WINDOWS */
+}
 
 /*
  * Argument is a UCS-4 wide character.
@@ -2362,6 +2374,7 @@ line_paint(int offset,			/* current dot offset into vl */
      * is double wide. We don't want the offset to be under that > character.)
      */
     for(w = (*width_a_to_b)(displ->vl, displ->vbase, offset);
+	displ->dwid > 1 && 
 	w + 2 + (displ->vbase ? 1 : 0) > displ->dwid;
         w = (*width_a_to_b)(displ->vl, displ->vbase, offset)){
 	/*
@@ -2419,7 +2432,7 @@ line_paint(int offset,			/* current dot offset into vl */
     vlast = displ->vused-1;			/* end */
     w = (*width_a_to_b)(displ->vl, vfirst, vlast);
 
-    if(w + dfirst > displ->dwid){			/* off window right */
+    if(displ->dwid > 0 && w + dfirst > displ->dwid){			/* off window right */
 
 	/* find last ucs character to be printed */
 	while(w + dfirst > displ->dwid - 1)	/* -1 for > */
