@@ -2,7 +2,7 @@
  * $Id: conftype.h 1155 2008-08-21 18:33:21Z hubert@u.washington.edu $
  *
  * ========================================================================
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -79,6 +79,9 @@ typedef	enum {    V_PERSONAL_NAME = 0
 		, V_UNK_CHAR_SET
 		, V_EDITOR
 		, V_SPELLER
+#ifdef _WINDOWS
+		, V_DICTIONARY
+#endif /* _WINDOWS */
 		, V_FILLCOL
 		, V_REPLY_STRING
 		, V_REPLY_INTRO
@@ -123,6 +126,7 @@ typedef	enum {    V_PERSONAL_NAME = 0
 		, V_MAILCAP_PATH
 		, V_MIMETYPE_PATH
 		, V_BROWSER
+		, V_HISTORY
 		, V_MAXREMSTREAM
 		, V_PERMLOCKED
 		, V_INCCHECKTIMEO
@@ -343,6 +347,7 @@ typedef enum {
 	F_FORCE_ARROW,
 	F_PRUNE_USES_ISO,
 	F_ALT_ED_NOW,
+	F_IGNORE_SIZE,
 	F_SHOW_DELAY_CUE,
 	F_CANCEL_CONFIRM,
 	F_AUTO_OPEN_NEXT_UNREAD,
@@ -503,6 +508,7 @@ typedef enum {
 	F_MAILDROPS_PRESERVE_STATE,
 	F_EXPOSE_HIDDEN_CONFIG,
 	F_ALT_COMPOSE_MENU,
+	F_ALT_REPLY_MENU,
 	F_ALT_ROLE_MENU,
 	F_ALWAYS_SPELL_CHECK,
 	F_QUELL_TIMEZONE,
@@ -672,7 +678,7 @@ typedef enum {Main, Post, None} EditWhich;
 
 typedef enum {Directory, Container, Keychain, Nada} SmimeHolderType;
 
-typedef enum {Public, Private, CACert} WhichCerts;
+typedef enum {Public, Private, CACert, Password} WhichCerts;
 
 typedef struct certdata {
   unsigned   deleted:1;		/* certificate is marked deleted	     */
@@ -683,7 +689,8 @@ typedef struct certdata {
 } CertData;
 
 typedef struct certlist {
-    char            *name;
+    char            *name;		/* file name		*/
+    char	    *cn;		/* CN field from certificate */
     void            *x509_cert;		/* this is type (X509 *) */
     CertData	    data;
     struct certlist *next;
@@ -717,7 +724,6 @@ typedef struct smime_stuff {
     char           *privatecontent;
     CertList	   *privatecertlist;
     CertList	   *backupprivatecertlist;
-    void	   *backuppersonal_certs;	/* this is type (PERSONAL_CERT *) */
     void           *personal_certs;		/* this is type (PERSONAL_CERT *) */
 
     SmimeHolderType catype;
@@ -728,25 +734,35 @@ typedef struct smime_stuff {
 
 } SMIME_STUFF_S;
 
-#define BACKUPDATACERT(X) (((X) == Public ? ps_global->smime->backuppubliccertlist	\
-			 : ((X) == Private ? ps_global->smime->backupprivatecertlist	\
-			   : ps_global->smime->backupcacertlist)))
+#define BACKUPDATACERT(X) ((X) == Public  ? ps_global->smime->backuppubliccertlist :	\
+			   (X) == Private ?  ps_global->smime->backupprivatecertlist :	\
+			   (X) == CACert  ?  ps_global->smime->backupcacertlist :	\
+			   (X) == Password ?  (CertList *) ps_global->backuppassword :	\
+			    NULL)
 
-#define DATACERT(X) (((X) == Public ? ps_global->smime->publiccertlist		\
-			 : ((X) == Private ? ps_global->smime->privatecertlist	\
-			   : ps_global->smime->cacertlist)))
+#define DATACERT(X)    ((X) == Public   ? ps_global->smime->publiccertlist  :		\
+			(X) == Private  ? ps_global->smime->privatecertlist :		\
+			(X) == CACert   ? ps_global->smime->cacertlist	    :		\
+			(X) == Password ? (CertList *) ps_global->pwdcertlist :		\
+			NULL)
 
-#define PATHCERTDIR(X) (((X) == Public ? ps_global->smime->publicpath	\
-			  : ((X) == Private ? ps_global->smime->privatepath	\
-			    : ((X) == CACert ? ps_global->smime->capath : NULL))))
+#define PATHCERTDIR(X) ((X) == Public   ? ps_global->smime->publicpath	:  \
+			(X) == Private  ? ps_global->smime->privatepath :  \
+			(X) == CACert   ? ps_global->smime->capath	:  \
+			(X) == Password ? ps_global->pwdcertdir 	:  \
+			NULL)
 
-#define CONTENTCERTLIST(X) 	(((X) == Public ? ps_global->smime->publiccontent	\
-			  : ((X) == Private ? ps_global->smime->privatecontent	\
-			    : ((X) == CACert ? ps_global->smime->cacontent : NULL))))
+#define CONTENTCERTLIST(X) ((X) == Public   ? ps_global->smime->publiccontent :		\
+			(X) == Private  ? ps_global->smime->privatecontent :		\
+			(X) == CACert   ? ps_global->smime->cacontent      :		\
+			(X) == Password ? ps_global->pwdcertcontent 	   :		\
+			NULL)
 
-#define SMHOLDERTYPE(X) (((X) == Public ? ps_global->smime->publictype	\
-			  : ((X) == Private ? ps_global->smime->privatetype	\
-			    : ((X) == CACert ? ps_global->smime->catype : Nada))))
+#define SMHOLDERTYPE(X) ((X) == Public  ? ps_global->smime->publictype	:		\
+			(X) == Private  ? ps_global->smime->privatetype :		\
+			(X) == CACert   ? ps_global->smime->catype      :		\
+			(X) == Password ? Directory 	   		:		\
+			Nada)
 
 #define EXTCERT(X)  (((X) == Public ? ".crt"		\
 			  : ((X) == Private ? ".key"	\
