@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: detach.c 1074 2008-06-04 00:08:43Z hubert@u.washingt
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -202,14 +202,16 @@ detach(MAILSTREAM *stream,		/* c-client stream to use         */
     }
 
     /* convert all text to UTF-8 */
-    if(is_text){
+    if(is_text & !(flags & DT_BINARY)){
 	charset = parameter_val(body->parameter, "charset");
 
 	/*
 	 * If the charset is unlabeled or unknown replace it
 	 * with the user's configured unknown charset.
 	 */
-	if(!charset || !strucmp(charset, UNKNOWN_CHARSET) || !strucmp(charset, "us-ascii")){
+	if(!charset || !strucmp(charset, UNKNOWN_CHARSET) 
+		|| !strucmp(charset, "MISSING_PARAMETER_VALUE")
+		|| !strucmp(charset, "us-ascii")){
 	    if(charset)
 	      fs_give((void **) &charset);
 
@@ -259,7 +261,9 @@ detach(MAILSTREAM *stream,		/* c-client stream to use         */
      * a multipart segment since an external handler's going to have to
      * make sense of it...
      */
-    if(is_text || body->type == TYPEMESSAGE || body->type == TYPEMULTIPART)
+    if((is_text & !(flags & DT_BINARY)) 
+	|| body->type == TYPEMESSAGE 
+	|| body->type == TYPEMULTIPART)
       gf_link_filter(gf_nvtnl_local, NULL);
 
     /*
@@ -294,7 +298,7 @@ detach(MAILSTREAM *stream,		/* c-client stream to use         */
 	    FILTLIST_S *p, *aux = NULL;
 	    size_t	count;
 
-	    if(aux_filters){
+	    if(aux_filters && !(flags & DT_BINARY)){
 		/* insert NL conversion filters around remaining aux_filters
 		 * so they're not tripped up by local NL convention
 		 */
@@ -334,7 +338,8 @@ detach(MAILSTREAM *stream,		/* c-client stream to use         */
 		for( ; aux_filters->filter ; aux_filters++)
 		  gf_link_filter(aux_filters->filter, aux_filters->data);
 
-		gf_link_filter(gf_nvtnl_local, NULL);
+		if(!(flags & DT_BINARY))
+		  gf_link_filter(gf_nvtnl_local, NULL);
 	    }
 
 	    if((status = gf_pipe(gc, pc)) != NULL){	/* Second pass, sheesh */
