@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: folder.c 1142 2008-08-13 17:22:21Z hubert@u.washingt
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -871,7 +871,7 @@ update_bboard_spec(char *bboard, char *buf, size_t buflen)
 	if(*bboard == '*')
 	  bboard++;
 
-	if(!nntp)
+	if(!nntp){
 	  /*
 	   * See if path portion looks newsgroup-ish while being aware
 	   * of the "view" portion of the spec...
@@ -891,6 +891,7 @@ update_bboard_spec(char *bboard, char *buf, size_t buflen)
 	    }
 	    else if(!(isalnum((unsigned char) *p) || strindex(".-", *p)))
 	      break;
+	}
 
 	snprintf(buf, buflen-(buf-origbuf), "%s%s%s",
 		(!nntp && *p) ? "#public" : "#news.",
@@ -1406,7 +1407,7 @@ unsigned char *folder_name_decoded(unsigned char *mailbox)
 {
   unsigned char *s;
   s = (unsigned char *) utf8_from_mutf7((unsigned char *) mailbox);
-  if (s == NULL) s = (unsigned char *) cpystr(mailbox);
+  if (s == NULL) s = (unsigned char *) cpystr((char *)mailbox);
   return s;
 }
 
@@ -1416,8 +1417,8 @@ unsigned char *folder_name_decoded(unsigned char *mailbox)
 unsigned char *folder_name_encoded(unsigned char *mailbox)
 {
   unsigned char *s;
-  s = (char *) utf8_to_mutf7(mailbox);
-  if (s == NULL) s = cpystr(mailbox);
+  if ((s = utf8_to_mutf7(mailbox)) == NULL)
+     s = (unsigned char *) cpystr((char *) mailbox);
   return s;
 }
 
