@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: hist.c 807 2007-11-09 01:21:33Z hubert@u.washington.
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -49,7 +49,6 @@ free_hist(HISTORY_S **history)
     int i;
 
     if(history && *history){
-
 	for(i = 0; i < (*history)->histsize; i++)
 	  if((*history)->hist[i] && (*history)->hist[i]->str)
 	    fs_give((void **) &(*history)->hist[i]->str);
@@ -58,6 +57,19 @@ free_hist(HISTORY_S **history)
     }
 }
 
+char *
+hist_in_pos(int pos, char **list, int llen, HISTORY_S *hist, int n)
+{
+  if(pos < 0 || pos > llen + n)
+    return NULL;
+
+  if(pos < llen)
+     return list[pos];
+
+  hist->curindex = (hist->origindex + n - pos + llen) % hist->histsize;
+  return((hist->hist[hist->curindex] && hist->hist[hist->curindex]->str)
+           ? hist->hist[hist->curindex]->str : NULL);
+}
 
 char *
 get_prev_hist(HISTORY_S *history, char *savethis, unsigned saveflags, void *cntxt)
