@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: icache.c 874 2007-12-15 02:51:06Z hubert@u.washingto
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,7 +45,7 @@ clear_index_cache_ent(MAILSTREAM *stream, long int msgno, unsigned int flags)
     MESSAGECACHE *mc;
 
     if(stream){
-	if(flags && IC_USE_RAW_MSGNO)
+	if(flags & IC_USE_RAW_MSGNO)
 	  rawno = msgno;
 	else
 	  rawno = mn_m2raw(sp_msgmap(stream), msgno);
@@ -261,7 +261,7 @@ fetch_ice(MAILSTREAM *stream, long unsigned int rawno)
     /*
      * any private elt data yet?
      */
-    if((*(peltp = (PINELT_S **) &mc->sparep) == NULL)){
+    if(*(peltp = (PINELT_S **) &mc->sparep) == NULL){
 	*peltp = (PINELT_S *) fs_get(sizeof(PINELT_S));
 	memset(*peltp, 0, sizeof(PINELT_S));
     }
@@ -291,7 +291,7 @@ fetch_ice_ptr(MAILSTREAM *stream, long unsigned int rawno)
     /*
      * any private elt data yet?
      */
-    if((*(peltp = (PINELT_S **) &mc->sparep) == NULL)){
+    if(*(peltp = (PINELT_S **) &mc->sparep) == NULL){
 	*peltp = (PINELT_S *) fs_get(sizeof(PINELT_S));
 	memset(*peltp, 0, sizeof(PINELT_S));
     }
