@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: ldap.c 1204 2009-02-02 19:54:23Z hubert@u.washington
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ static char rcsid[] = "$Id: ldap.c 1204 2009-02-02 19:54:23Z hubert@u.washington
 #include "../pith/busy.h"
 #include "../pith/signal.h"
 #include "../pith/ablookup.h"
-
+#include "../pith/readfile.h"
 
 /*
  * Until we can think of a better way to do this. If user exits from an
@@ -402,7 +402,7 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
     char    *mailattr, *snattr, *gnattr, *cnattr;
     int      we_cancel = 0, we_turned_on = 0;
     LDAP_SERV_RES_S *serv_res = NULL;
-    LDAP *ld;
+    LDAP *ld = NULL;
     long  pwdtrial = 0L;
     int   ld_errnum;
     char *ld_errstr;
@@ -472,7 +472,66 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
     if((ld = ldap_init(serv, info->port)) == NULL)
 #else
 #if (LDAPAPI >= 11)
+#ifdef _WINDOWS
     if((ld = ldap_init(serv, info->port)) == NULL)
+#else
+#ifdef SMIME_SSLCERTS
+    /* If we are attempting a ldaps secure connection, we need to tell 
+     * ldap that we have certificates. There are many ways to do so. 
+     * OpenLDAP has many ways to configure this through configuration
+     * files. For example the global (to the system) ldap.conf file, or the
+     * personal ldaprc or .ldaprc files. Setting the location of the 
+     * certificates must happen at the time we call ldap_initialize, we 
+     * cannot set it up before that call, nor after, so what we are going 
+     * to do is to test for a .ldaprc file in the home directory. If such 
+     * file exists we read it, if not we create it and if it does not have
+     * a line for the location of the certificates in the system, we add one.
+     * (so we ignore all other configuration files)
+     */
+    if(info->ldaps && ps_global->home_dir){
+	int done = 0;
+	char *text, *tls_conf;
+	char filename[MAXPATH+1];
+
+        build_path(filename, ps_global->home_dir, ".ldaprc", sizeof(filename));
+
+	if((text = read_file(filename, 0)) != NULL)
+	   while(done == 0 
+		&& (tls_conf = strstr(text, "TLS_CACERTDIR")) != NULL
+		&& (tls_conf == text || *(tls_conf - 1) == '\n')){
+		   tls_conf += 13;	/* 13 = strlen("TLS_CACERTDIR") */
+		   while (isspace(*tls_conf))
+		     tls_conf++;
+		   if(!strncmp(tls_conf, SMIME_SSLCERTS, strlen(SMIME_SSLCERTS)))
+		      done++;
+	    }
+
+	if(!done){
+	   STORE_S *so;
+
+	   if((so = so_get(FileStar, filename, WRITE_ACCESS)) != NULL){
+	      if(text != NULL){
+	        so_puts(so, text);
+	        so_puts(so, NEWLINE);
+	      }
+	      so_puts(so, "TLS_CACERTDIR");
+	      so_puts(so, " ");
+	      so_puts(so, SMIME_SSLCERTS); 
+	      so_puts(so, NEWLINE);
+	      so_give(&so);
+	   }
+	}
+	if(text != NULL)
+	  fs_give((void **)&text);
+    }
+#endif /* SMIME_SSLCERTS */
+
+    snprintf(tmp_20k_buf, SIZEOF_20KBUF, "%s://%s:%d", 
+		info->ldaps ? "ldaps" : "ldap", serv, info->port);
+    tmp_20k_buf[SIZEOF_20KBUF-1] = '\0';
+
+    if(ldap_initialize(&ld, tmp_20k_buf) != LDAP_SUCCESS) 
+#endif
 #else
     if((ld = ldap_open(serv, info->port)) == NULL)
 #endif
@@ -498,7 +557,11 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
     else if(!ps_global->intr_pending){
       int proto = 3, tlsmustbail = 0;
       char pwd[NETMAXPASSWD], user[NETMAXUSER];
+#ifdef _WINDOWS
       char *passwd = NULL;
+#else
+      struct berval passwd = { 0 };
+#endif
       char hostbuf[1024];
       NETMBX mb;
 #ifndef _WINDOWS
@@ -524,8 +587,7 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
       our_ldap_set_option(ld, LDAP_OPT_RESTART, LDAP_OPT_ON);
 
       /*
-       * If we need to authenticate, get the password. We are not
-       * supporting SASL authentication, just LDAP simple.
+       * If we need to authenticate, get the password
        */
       if(info->binddn && info->binddn[0]){
 	  char pmt[500];
@@ -541,9 +603,10 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
 	  if((space=strindex(hostbuf, ' ')) != NULL)
 	    *space = '\0';
 
-	  mail_valid_net_parse_work(hostbuf, &mb, "ldap");
+	  mail_valid_net_parse_work(hostbuf, &mb, info->ldaps ? "ldaps" : "ldap");
 	  mb.port = info->port;
 	  mb.tlsflag = (info->tls || info->tlsmust) ? 1 : 0;
+	  mb.sslflag = info->ldaps ? 1 : 0;
 
 try_password_again:
 
@@ -573,7 +636,12 @@ try_password_again:
 	      snprintf(pmt, sizeof(pmt), "  %s", (info->nick && *info->nick) ? info->nick : serv);
 	      mm_login_work(&mb, user, pwd, pwdtrial, pmt, info->binddn);
 	      if(pwd && pwd[0])
+#ifdef _WINDOWS
 		passwd = pwd;
+#else
+		passwd.bv_len = strlen(pwd);
+		passwd.bv_val = pwd;		
+#endif
 	  }
       }
 
@@ -583,7 +651,12 @@ try_password_again:
        * to tell the server we're v3 if the server supports v3, and if the
        * server doesn't support v3 the bind is required.
        */
-      if(tlsmustbail || ldap_simple_bind_s(ld, info->binddn, passwd) != LDAP_SUCCESS){
+      if(tlsmustbail 
+#ifdef _WINDOWS
+	|| ldap_simple_bind_s(ld, info->binddn, passwd) != LDAP_SUCCESS){
+#else
+	|| ldap_sasl_bind_s(ld, info->binddn, LDAP_SASL_SIMPLE, &passwd, NULL, NULL, NULL) != LDAP_SUCCESS){
+#endif 
 	wp_err->wp_err_occurred = 1;
 
 	ld_errnum = our_ldap_get_lderrno(ld, NULL, &ld_errstr);
@@ -606,8 +679,11 @@ try_password_again:
 
         if(we_cancel)
           cancel_busy_cue(-1);
-
+#ifdef _WINDOWS
 	ldap_unbind(ld);
+#else
+	ldap_unbind_ext(ld, NULL, NULL);
+#endif
         wp_err->error = cpystr(ebuf);
         q_status_message(SM_ORDER, 3, 5, wp_err->error);
         display_message('x');
@@ -831,11 +907,22 @@ try_password_again:
 	else{
 	  int msgid;
 	  time_t start_time;
+	  struct timeval tv = {0}, *tvp = NULL;
+
+	  memset((void *)&tv, 0, sizeof(struct timeval));
+          tv.tv_sec = info->time;
+	  tvp = &tv;
 
 	  start_time = time((time_t *)0);
 
 	  dprint((6, "ldap_lookup: calling ldap_search\n"));
+#ifdef _WINDOWS
 	  msgid = ldap_search(ld, base, info->scope, filter, NULL, 0);
+#else
+	  if(ldap_search_ext(ld, base, info->scope, filter, NULL, 0,
+			NULL, NULL, tvp, info->size, &msgid) != LDAP_SUCCESS)
+	    msgid = -1;
+#endif
 
 	  if(msgid == -1)
 	    srch_res = our_ldap_get_lderrno(ld, NULL, NULL);
@@ -863,7 +950,11 @@ try_password_again:
 	      }
 	      else if(lres == 0){  /* timeout, no results available */
 		if(intr_happened){
+#ifdef _WINDOWS
 		  ldap_abandon(ld, msgid);
+#else
+		  ldap_abandon_ext(ld, msgid, NULL, NULL);
+#endif
 		  srch_res = LDAP_PROTOCOL_ERROR;
 		  if(our_ldap_get_lderrno(ld, NULL, NULL) == LDAP_SUCCESS)
 		    our_ldap_set_lderrno(ld, LDAP_PROTOCOL_ERROR, NULL, NULL);
@@ -882,7 +973,11 @@ try_password_again:
 		  }
 		  else{
 		    if(lres == 0)
+#ifdef _WINDOWS
 		      ldap_abandon(ld, msgid);
+#else
+		      ldap_abandon_ext(ld, msgid, NULL, NULL);
+#endif
 
 		    srch_res = LDAP_TIMEOUT;
 		    if(our_ldap_get_lderrno(ld, NULL, NULL) == LDAP_SUCCESS)
@@ -898,9 +993,25 @@ try_password_again:
 		}
 	      }
 	      else{
+#ifdef _WINDOWS
 		srch_res = ldap_result2error(ld, res, 0);
 	        dprint((6, "lres=0x%x, srch_res=%d\n", lres,
 			   srch_res));
+#else
+		int err;
+		char *dn, *text, **ref;
+		LDAPControl **srv;
+
+		dn = text = NULL; ref = NULL; srv = NULL;
+		srch_res = ldap_parse_result(ld, res,
+			&err, &dn, &text, &ref, &srv, 0);
+	        dprint((6, "lres=0x%x, srch_res=%d, dn=%s, text=%s\n", lres,
+			   srch_res, dn ? dn : "", text ? text : ""));
+		if(dn) ber_memfree(dn);
+		if(text) ber_memfree(text);
+		if(ref) ber_memvfree((void **) ref);
+		if(srv) ldap_controls_free(srv);
+#endif
 	      }
 	    }while(lres == 0 &&
 		    !(intr_happened ||
@@ -925,7 +1036,11 @@ try_password_again:
 	  if(res)
 	    ldap_msgfree(res);
 	  if(ld)
+#ifdef _WINDOWS
 	    ldap_unbind(ld);
+#else
+	    ldap_unbind_ext(ld, NULL, NULL);
+#endif
 	  
 	  res = NULL; ld  = NULL;
 	}
@@ -957,8 +1072,12 @@ try_password_again:
 	  if(res)
 	    ldap_msgfree(res);
 	  if(ld)
+#ifdef _WINDOWS
 	    ldap_unbind(ld);
-	  
+#else
+	    ldap_unbind_ext(ld, NULL, NULL);
+#endif
+
 	  res = NULL; ld  = NULL;
 	}
 	else{
@@ -1043,7 +1162,11 @@ try_password_again:
 	    if(res)
 	      ldap_msgfree(res);
 	    if(ld)
+#ifdef _WINDOWS
 	      ldap_unbind(ld);
+#else
+	      ldap_unbind_ext(ld, NULL, NULL);
+#endif
 
 	    res = NULL; ld  = NULL;
 	  }
@@ -1184,41 +1307,41 @@ address_from_ldap(LDAP_CHOOSE_S *winning_e)
 	    a = ldap_next_attribute(winning_e->ld, winning_e->selected_entry, ber)){
 	    int i;
 	    char  *p;
-	    char **vals;
+	    struct berval **vals;
 
 	    dprint((9, "attribute: %s\n", a ? a : "?"));
 	    if(!ret_a->personal &&
 	       strcmp(a, winning_e->info_used->cnattr) == 0){
 		dprint((9, "Got cnattr:"));
-		vals = ldap_get_values(winning_e->ld, winning_e->selected_entry, a);
-		for(i = 0; vals[i] != NULL; i++)
+		vals = ldap_get_values_len(winning_e->ld, winning_e->selected_entry, a);
+		for(i = 0; i < ldap_count_values_len(vals); i++)
 		  dprint((9, "       %s\n",
-		         vals[i] ? vals[i] : "?"));
+		         vals[i] ? vals[i]->bv_val : "?"));
 	    
-		if(vals && vals[0])
-		  ret_a->personal = cpystr(vals[0]);
+		if(ALPINE_LDAP_can_use(vals))
+		  ret_a->personal = cpystr(vals[0]->bv_val);
 
-		ldap_value_free(vals);
+		ldap_value_free_len(vals);
 	    }
 	    else if(!ret_a->mailbox &&
 		    strcmp(a, winning_e->info_used->mailattr) == 0){
 		dprint((9, "Got mailattr:"));
-		vals = ldap_get_values(winning_e->ld, winning_e->selected_entry, a);
-		for(i = 0; vals[i] != NULL; i++)
+		vals = ldap_get_values_len(winning_e->ld, winning_e->selected_entry, a);
+		for(i = 0; i < ldap_count_values_len(vals); i++)
 		  dprint((9, "         %s\n",
-		         vals[i] ? vals[i] : "?"));
+		         vals[i] ? vals[i]->bv_val : "?"));
 		    
 		/* use first one */
-		if(vals && vals[0]){
-		    if((p = strindex(vals[0], '@')) != NULL){
+		if(ALPINE_LDAP_can_use(vals)){
+		    if((p = strindex(vals[0]->bv_val, '@')) != NULL){
 			ret_a->host = cpystr(p+1);
 			*p = '\0';
 		    }
 
-		    ret_a->mailbox = cpystr(vals[0]);
+		    ret_a->mailbox = cpystr(vals[0]->bv_val);
 		}
 
-		ldap_value_free(vals);
+		ldap_value_free_len(vals);
 	    }
 
 	    our_ldap_memfree(a);
@@ -1345,6 +1468,10 @@ break_up_ldap_server(char *serv_str)
 	if((q = srchstr(tail, "/tlsm=1")) != NULL)
 	  info->tlsmust = 1;
 
+	/* get the ldaps parameter */
+	if((q = srchstr(tail, "/ldaps=1")) != NULL)
+	  info->ldaps = 1;
+
 	/* get the search type value */
 	if((q = srchstr(tail, "/type=")) != NULL){
 	    NAMEVAL_S *v;
@@ -1569,7 +1696,11 @@ free_ldap_result_list(LDAP_SERV_RES_S **r)
 	if((*r)->res)
 	  ldap_msgfree((*r)->res);
 	if((*r)->ld)
+#ifdef _WINDOWS
 	  ldap_unbind((*r)->ld);
+#else
+	  ldap_unbind_ext((*r)->ld, NULL, NULL);
+#endif
 	if((*r)->info_used)
 	  free_ldap_server_info(&(*r)->info_used);
 	if((*r)->serv)
@@ -1794,5 +1925,24 @@ ldap_translate(char *a, LDAP_SERV_S *info_used)
     return(a);
 }
 
+char **
+berval_to_array(struct berval **v)
+{
+  char **rv = NULL;
+  int i, len;
+
+  if(v == NULL) return rv; 
+  len = ldap_count_values_len(v);
+
+  rv = fs_get((len+1)*sizeof(char *));
+  for(i = 0; i < len; i++)
+    if(ALPINE_LDAP_can_use_num(v, i))
+	rv[i] = cpystr(v[i]->bv_val);
+    else
+	rv[i] = NULL;
+  rv[len] = NULL;
+
+  return rv;
+}
 
 #endif	/* ENABLE_LDAP */
