@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: mimedesc.c 1142 2008-08-13 17:22:21Z hubert@u.washin
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -70,8 +70,9 @@ void
 describe_mime(struct mail_bodystruct *body, char *prefix, int num,
 	      int should_show, int multalt, int flags)
 {
+#define NUMXLEN 512
     PART      *part;
-    char       numx[512], string[800], *description;
+    char       numx[NUMXLEN], string[800], *description;
     int        n, named = 0, can_display_ext;
     ATTACH_S  *a;
 
@@ -222,7 +223,7 @@ describe_mime(struct mail_bodystruct *body, char *prefix, int num,
                 (description && description[0]) ? ", \"" : "",
                 (description && description[0]) ? description : "",
                 (description && description[0]) ? "\"": "");
-	string[sizeof(string)-1] =- '\0';
+	string[sizeof(string)-1] = '\0';
         a->description = cpystr(string);
         a->body        = body;
 
@@ -279,8 +280,8 @@ describe_mime(struct mail_bodystruct *body, char *prefix, int num,
         (a+1)->description = NULL;
         if(body->type == TYPEMESSAGE && body->encoding <= ENCBASE64
 	   && body->subtype && strucmp(body->subtype, "rfc822") == 0){
-	    body = body->nested.msg->body;
-	    snprintf(numx, sizeof(numx), "%.*s%d.", sizeof(numx)-20, prefix, num);
+	    body = body->nested.msg->body;	/* NUMXLEN = sizeof(numx) */
+	    snprintf(numx, sizeof(numx), "%.*s%d.", NUMXLEN-20, prefix, num);
 	    numx[sizeof(numx)-1] = '\0';
 	    describe_mime(body, numx, 1, should_show, 0, flags);
         }
