@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: msgno.c 854 2007-12-07 17:44:43Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -271,7 +271,7 @@ msgno_exclude_deleted(MAILSTREAM *stream, MSGNO_S *msgs, char *sequence)
     (void) count_flagged(stream, F_DEL);
 
     if(sequence)
-	mail_sequence (stream,sequence);
+	mail_sequence (stream,(unsigned char *) sequence);
 
     /*
      * Start with the end of the folder and work backwards so that
