@@ -2,6 +2,7 @@
  * $Id: debugtime.h 761 2007-10-23 22:35:18Z hubert@u.washington.edu $
  *
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,7 +23,7 @@
  * Exported Prototypes
  */
 #ifdef DEBUG
-char	*debug_time(int, int);
+char	*debug_time(int, int, int);
 #endif
 int	 get_time(TIMEVAL_S *);
 long	 time_diff(TIMEVAL_S *, TIMEVAL_S *);
