@@ -4,6 +4,7 @@ static char rcsid[] = "$Id: readfile.c 761 2007-10-23 22:35:18Z hubert@u.washing
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -89,6 +90,9 @@ our_copy(char *to_file, char *from_file)
      return -1;
    }
 
+   so_seek(out_cert, 0L, 0);
+   so_truncate(out_cert, 0);
+
    while(so_readc(&c, in_cert) > 0)
      so_writec(c, out_cert);
 
