@@ -2,8 +2,8 @@
  * $Id: reply.h 1074 2008-06-04 00:08:43Z hubert@u.washington.edu $
  *
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -95,13 +95,15 @@ char	   *reply_in_reply_to(ENVELOPE *);
 char	   *generate_message_id(void);
 char	   *generate_user_agent(void);
 char       *rot13(char *);
+char       *rot5n(char *);
 ADDRESS    *first_addr(ADDRESS *);
 char       *get_signature_lit(char *, int, int, int, int);
 int         sigdashes_are_present(char *);
 char	   *signature_path(char *, char *, size_t);
 char       *simple_read_remote_file(char *, char *);
 BODY	   *forward_multi_alt(MAILSTREAM *, ENVELOPE *, BODY *, long, char *, void *, gf_io_t, int);
+BODY	   *forward_multi_alt_mixed(MAILSTREAM *, ENVELOPE *, BODY *, long, char *, void *, gf_io_t, int);
 int	    same_subject(char *, char *s);
-
+void	    shorten_subject(char *);
 
 #endif /* PITH_REPLY_INCLUDED */
