@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: save.c 1204 2009-02-02 19:54:23Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2009 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -1157,6 +1157,7 @@ long save_fetch_append_cb(MAILSTREAM *stream, void *data, char **flags,
 		    snprintf(buf, sizeof(buf),
 	 "Message to save shrank: source msg # %ld may be saved incorrectly",
 			     mn_raw2m(pkg->msgmap, raw));
+		    if(F_OFF(F_IGNORE_SIZE, ps_global))
 		    q_status_message(SM_ORDER, 0, 3, buf);
 		}
 		else{
@@ -1466,15 +1467,16 @@ save_ex_output_body(MAILSTREAM *stream, long int raw, char *section,
       return(save_ex_replace_body(txtp, len, body, pc));
 
     if(body->type == TYPEMULTIPART){
-	char	  *subsect, boundary[128];
+#define BOUNDARYLEN 128
+	char	  *subsect, boundary[BOUNDARYLEN];
 	int	   n, blen;
 	PART	  *part = body->nested.part;
 	PARAMETER *param;
 
 	/* Locate supplied multipart boundary */
 	for (param = body->parameter; param; param = param->next)
-	  if (!strucmp(param->attribute, "boundary")){
-	      snprintf(boundary, sizeof(boundary), "--%.*s\015\012", sizeof(boundary)-10,
+	  if (!strucmp(param->attribute, "boundary")){	/* BOUNDARYLEN == sizeof(boundary) */
+	      snprintf(boundary, sizeof(boundary), "--%.*s\015\012", BOUNDARYLEN-10,
 		      param->value);
 	      blen = strlen(boundary);
 	      break;
@@ -1516,8 +1518,8 @@ save_ex_output_body(MAILSTREAM *stream, long int raw, char *section,
 	      return(0);
 	}
 	while ((part = part->next) != NULL);	/* until done */
-
-	snprintf(boundary, sizeof(boundary), "--%.*s--\015\012", sizeof(boundary)-10,param->value);
+						/* BOUNDARYLEN = sizeof(boundary) */
+	snprintf(boundary, sizeof(boundary), "--%.*s--\015\012", BOUNDARYLEN-10,param->value);
 	*len += blen + 2;
 	return(gf_puts(boundary, pc));
     }
@@ -1618,10 +1620,10 @@ save_ex_explain_parts(struct mail_bodystruct *body, int depth, long unsigned int
 	PART *part = body->nested.part;	/* first body part */
 
 	*len = 0;
-	if(body->description && *body->description){
+	if(body->description && *body->description){	/* MAILTMPLEN = sizeof(tmp) */
 	    snprintf(tmp, sizeof(tmp), "%*.*sA %s/%.*s%.10s%.100s%.10s segment described",
 		    depth, depth, " ", body_type_names(body->type),
-		    sizeof(tmp)-300, body->subtype ? body->subtype : "Unknown",
+		    MAILTMPLEN-300, body->subtype ? body->subtype : "Unknown",
 		    name ? " (Name=\"" : "",
 		    name ? name : "",
 		    name ? "\")" : "");
@@ -1633,11 +1635,11 @@ save_ex_explain_parts(struct mail_bodystruct *body, int depth, long unsigned int
 		    (char *) rfc1522_decode_to_utf8((unsigned char *)tmp_20k_buf,
 						    SIZEOF_20KBUF, buftmp));
 	}
-	else{
+	else{						/* MAILTMPLEN = sizeof(tmp) */
 	    snprintf(tmp, sizeof(tmp), "%*.*sA %s/%.*s%.10s%.100s%.10s segment containing:",
 		    depth, depth, " ",
 		    body_type_names(body->type),
-		    sizeof(tmp)-300, body->subtype ? body->subtype : "Unknown",
+		    MAILTMPLEN-300, body->subtype ? body->subtype : "Unknown",
 		    name ? " (Name=\"" : "",
 		    name ? name : "",
 		    name ? "\")" : "");
@@ -1656,11 +1658,11 @@ save_ex_explain_parts(struct mail_bodystruct *body, int depth, long unsigned int
 	    return(0);
 	while ((part = part->next) != NULL);	/* until done */
     }
-    else{
+    else{					/* MAILTMPLEN = sizeof(tmp) */
 	snprintf(tmp, sizeof(tmp), "%*.*sA %s/%.*s%.10s%.100s%.10s segment of about %s bytes%s",
 		depth, depth, " ",
 		body_type_names(body->type), 
-		sizeof(tmp)-300, body->subtype ? body->subtype : "Unknown",
+		MAILTMPLEN-300, body->subtype ? body->subtype : "Unknown",
 		name ? " (Name=\"" : "",
 		name ? name : "",
 		name ? "\")" : "",
@@ -1671,10 +1673,10 @@ save_ex_explain_parts(struct mail_bodystruct *body, int depth, long unsigned int
 	if(!save_ex_output_line(tmp, len, pc))
 	  return(0);
 
-	if(body->description && *body->description){
+	if(body->description && *body->description){	/* MAILTMPLEN = sizeof(tmp) */
 	    snprintf(buftmp, sizeof(buftmp), "%.75s", body->description);
 	    snprintf(tmp, sizeof(tmp), "%*.*s   described as \"%.*s\"", depth, depth, " ",
-		    sizeof(tmp)-100,
+		    MAILTMPLEN-100,
 		    (char *) rfc1522_decode_to_utf8((unsigned char *)tmp_20k_buf,
 						    SIZEOF_20KBUF, buftmp));
 	    if(save_ex_output_line(tmp, &ilen, pc))
