@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: smkeys.c 1266 2009-07-14 18:39:12Z hubert@u.washingt
 
 /*
  * ========================================================================
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -48,6 +48,239 @@ static char rcsid[] = "$Id: smkeys.c 1266 2009-07-14 18:39:12Z hubert@u.washingt
 static char     *emailstrclean(char *string);
 static int       mem_add_extra_cacerts(char *contents, X509_LOOKUP *lookup);
 int		 compare_certs_by_name(const void *data1, const void *data2);
+int		 password_policy_check(char *);
+
+int  (*pith_smime_enter_password)(char *, char *, size_t);
+
+/* test if password passes a predetermined policy.
+ * return value: 0 - does not pass; 1 - it passes 
+ */
+int
+password_policy_check(char *password)
+{
+  int rv = 1;
+  char *error;
+  char tmp[1024];
+
+  if(password == NULL || password[0] == '\0'){
+    error = _("Password cannot be blank");
+    rv = 0;     
+  } else if(strlen(password) < 8){
+    error = _("Password is too short");
+    rv = 0;
+  }
+  if(rv == 0){
+    snprintf(tmp, sizeof(tmp), "%s%s", error, _(". Enter password again"));
+    tmp[sizeof(tmp) - 1] = '\0';
+    q_status_message(SM_ORDER, 3, 3, tmp);
+  }
+  return rv;
+}
+
+
+int
+create_master_password(char *pass, size_t passlen, int first_time)
+{
+#define MAXTRIAL 3
+  int rv, trial;
+  char prompt[MAILTMPLEN];
+  char passbackup[MAILTMPLEN];
+
+  if(first_time)
+     q_status_message(SM_ORDER, 3, 3, 
+		_(" Creating a Master Password for your Password file "));
+  else
+     q_status_message(SM_ORDER, 3, 3, 
+		_(" Retrying to create a Master Password for your Password file "));
+
+  for(trial = 0; trial < MAXTRIAL; trial++){
+    snprintf(prompt, sizeof(prompt), 
+		_("Create master password \(attempt %d of %d): "), trial+1, MAXTRIAL);
+    prompt[sizeof(prompt)- 1] = '\0';
+    pass[0] = '\0';
+    do { 
+      rv = (pith_smime_enter_password)(prompt, pass, passlen);
+      if(password_policy_check(pass) == 0)
+	 pass[0] = '\0';
+    } while ((rv !=0 && rv !=1 && rv > 0) || pass[0] == '\0');
+
+    snprintf(prompt, sizeof(prompt), 
+		_("Confirm master password \(attempt %d of %d): "), trial+1, MAXTRIAL);
+    prompt[sizeof(prompt)- 1] = '\0';
+    passbackup[0] = '\0';
+    do { 
+      rv = (pith_smime_enter_password)(prompt, passbackup, sizeof(passbackup));
+    } while ((rv !=0 && rv !=1 && rv > 0) || passbackup[0] == '\0');
+    if(!strcmp(pass, passbackup))
+       break;
+    if(trial + 1 < MAXTRIAL)
+      q_status_message(SM_ORDER, 2, 2, _("Passwords do not match, try again."));
+    else{
+      q_status_message(SM_ORDER, 2, 2, _("Passwords do not match, too many failures."));
+      pass[0] = '\0';
+    }
+  }
+  return (trial < MAXTRIAL) ? 1 : 0;
+}
+
+/* 
+ * Create a self signed certificate with root name _fname_, in directory
+ * _pathdir_. If _version_ is 3, we use the _template_ file as configuration
+ * file for openssl. At this moment, we only call this function with template = NULL
+ * and version = 0, but a sensible call is 
+ * ALPINE_self_signed_certificate("/etc/ssl/openssl.cnf", 2, pathdir, fname, first_time);
+ * or so.
+ * _pathdir_ is the directory to save the file,
+ * _fname_ is the root of the name to use. Append ".key" and ".crt" to this name
+ * _first_time_ is an indicator to tell us if this is the first time we call this function
+ */
+PERSONAL_CERT *
+ALPINE_self_signed_certificate(char *template, int version, char *pathdir, char *fname)
+{
+    BIGNUM *b = NULL;
+    X509_NAME *name = NULL;
+    X509_REQ *req = NULL;
+    EVP_PKEY_CTX *pkctx;
+    BIO *out = NULL;
+    char tmp[MAXPATH+1], password[1024];
+    char *keyfile = NULL, *certfile = NULL;
+    char *extensions = NULL;
+    FILE *fp;
+    long errline = -1L;
+    PERSONAL_CERT *pc = NULL;
+    EVP_PKEY *pkey = NULL;
+    X509 *pcert = NULL;
+    CONF *req_conf = NULL;
+    static int first_time = 1;
+
+    if(pathdir == NULL)
+      return NULL;
+
+    if(template){
+       if((out = BIO_new_file(template, "r")) == NULL){
+	 q_status_message(SM_ORDER, 2, 2, _("Problem reading configuration file"));
+	 return pc;
+       }
+
+       if((req_conf = NCONF_new(NULL)) != NULL
+	&& NCONF_load_bio(req_conf, out, &errline) > 0){
+	  if((extensions = NCONF_get_string(req_conf, "req", "x509_extensions")) != NULL){
+	   X509V3_CTX ctx;
+	   X509V3_set_ctx_test(&ctx);
+	   X509V3_set_nconf(&ctx, req_conf);
+	     if (!X509V3_EXT_add_nconf(req_conf, &ctx, extensions, NULL)) {
+		q_status_message(SM_ORDER, 2, 2, _("Problem loading openssl configuration"));
+		NCONF_free(req_conf);
+		return pc;
+	     }
+          }
+       }
+       BIO_free(out);
+       out = NULL;
+    }
+
+    if(create_master_password(password, sizeof(password), first_time)
+	&& (pkctx = EVP_PKEY_CTX_new_id(EVP_PKEY_RSA, NULL)) != NULL
+	&& EVP_PKEY_keygen_init(pkctx) > 0
+	&& EVP_PKEY_CTX_set_rsa_keygen_bits(pkctx, 2048) > 0	/* RSA:2048 */
+	&& EVP_PKEY_keygen(pkctx, &pkey) > 0){
+	snprintf(tmp, sizeof(tmp), "%s.key", fname);
+	tmp[sizeof(tmp)-1] = '\0';
+	keyfile = cpystr(tmp);		
+	build_path(tmp, pathdir, keyfile, sizeof(tmp));
+	keyfile[strlen(keyfile)-4] = '\0'; /* keyfile does not have .key extension */
+	if((fp = fopen(tmp, "w")) != NULL
+	    && (out = BIO_new_fp(fp, BIO_CLOSE | BIO_FP_TEXT)) != NULL
+	    && PEM_write_bio_PrivateKey(out, pkey, EVP_des_ede3_cbc(),
+                                      NULL, 0, NULL, password)){
+	    BIO_free(out);
+	    out = NULL;
+	}
+	memset((void *)password, 0, sizeof(password));
+	if((req = X509_REQ_new()) != NULL
+	    && X509_REQ_set_version(req, 0L)){
+	    name = X509_REQ_get_subject_name(req);
+	    X509_NAME_add_entry_by_txt(name, "CN", MBSTRING_ASC, "Password File Certificate and Key Pair", -1, -1, 0);
+	    if(X509_REQ_set_pubkey(req, pkey)
+		&& (pcert = X509_new()) != NULL){
+		if(X509_set_version(pcert, version)
+		   && (b = BN_new()) != NULL	
+		   && BN_set_word(b, 65537)
+		   && BN_pseudo_rand(b, 64, 0, 0)
+		   && X509_get_serialNumber(pcert) 
+		   && BN_to_ASN1_INTEGER(b, X509_get_serialNumber(pcert)) /* set serial */
+		   && X509_set_issuer_name(pcert, X509_REQ_get_subject_name(req))
+		   && X509_set_subject_name(pcert, X509_REQ_get_subject_name(req))){
+		      X509V3_CTX ext_ctx;
+		      EVP_PKEY *tmppkey;
+
+		      X509_gmtime_adj(X509_getm_notBefore(pcert), 0);
+		      X509_time_adj_ex(X509_getm_notAfter(pcert), 1095, 0, NULL);
+
+		      if((tmppkey = X509_REQ_get0_pubkey(req)) != NULL
+		         && X509_set_pubkey(pcert, tmppkey)){
+			 if(extensions != NULL && version == 2){
+			     X509V3_set_ctx(&ext_ctx, pcert, pcert, NULL, NULL, 0);
+			     if(req_conf){	/* only if template is not null */
+			       X509V3_set_nconf(&ext_ctx, req_conf);
+			       X509V3_EXT_add_nconf(req_conf, &ext_ctx, extensions, pcert);
+			     }
+			 }
+			 EVP_PKEY_free(tmppkey);
+			 X509_sign(pcert, pkey, NULL);
+		      }
+		      BN_free(b);
+		   }
+		}
+	    }
+
+	    snprintf(tmp, sizeof(tmp), "%s.crt", fname);
+	    tmp[sizeof(tmp)-1] = '\0';
+	    certfile = cpystr(tmp);
+	    build_path(tmp, pathdir, certfile, sizeof(tmp));
+	    if((fp = fopen(tmp, "w")) != NULL
+		&&(out = BIO_new_fp(fp, BIO_FP_TEXT)) != NULL){
+	      EVP_PKEY *tpubkey = X509_REQ_get0_pubkey(req);
+	      PEM_write_bio_X509(out, pcert);
+	      BIO_flush(out);
+	      BIO_free(out);
+	      out = NULL;
+	    }
+	    if(req_conf)
+	       NCONF_free(req_conf);
+    }
+    if(keyfile && certfile && pkey && pcert){
+        pc = (PERSONAL_CERT *) fs_get(sizeof(PERSONAL_CERT));
+        memset((void *)pc, 0, sizeof(PERSONAL_CERT));
+        pc->name = keyfile;
+        pc->key  = pkey;
+        pc->cert = pcert;
+        pc->cname = certfile;
+    }
+    first_time = 0;
+    return pc;
+}
+
+CertList *
+smime_X509_to_cert_info(X509 *x, char *name)
+{
+  CertList *cert;
+  char buf[MAXPATH+1];
+
+  if(x == NULL) return NULL;
+
+  cert = fs_get(sizeof(CertList));
+  memset((void *)cert, 0, sizeof(CertList));
+  cert->x509_cert = x;
+  cert->name = name ? cpystr(name) : NULL;
+  cert->data.date_from = smime_get_date(X509_get0_notBefore(x));
+  cert->data.date_to = smime_get_date(X509_get0_notAfter(x));
+  cert->cn = smime_get_cn(x);
+  get_fingerprint(x, EVP_md5(), buf, sizeof(buf), NULL);
+  cert->data.md5 = cpystr(buf);
+
+  return cert;
+}
 
 #define SMIME_BACKUP_DIR	".backup"
 #define MAX_TRY_BACKUP		100
@@ -141,6 +374,8 @@ setup_certs_backup_by_type(WhichCerts ctype)
 				   cert->data.date_to	= cpystr(cl->data.date_to);
 				  if(cl->data.md5 != NULL)
 				   cert->data.md5	= cpystr(cl->data.md5);
+				  if(cl->cn != NULL)
+				   cert->cn  = cpystr(cl->cn);
 				  snprintf(buf2, len, "%s.%s", cl->name, cl->data.md5);
 				  buf2[sizeof(buf2)-1] = '\0';
 				  cert->name = cpystr(buf2);
@@ -196,23 +431,12 @@ setup_certs_backup_by_type(WhichCerts ctype)
 		case CACert:
 			if((in = BIO_new_file(buf2, "r"))!=0){
 			  x = PEM_read_bio_X509(in, NULL, NULL, NULL);
-			  if(x && x->cert_info){ /* for now copy this information */
-			    cert = fs_get(sizeof(CertList));
-			    memset((void *)cert, 0, sizeof(CertList));
-			    cert->x509_cert = x;
-			    cert->data.date_from = smime_get_date(x->cert_info->validity->notBefore);
-			    cert->data.date_to	 = smime_get_date(x->cert_info->validity->notAfter);
-			    get_fingerprint(x, EVP_md5(), buf, sizeof(buf), NULL);
-			    cert->data.md5	 = cpystr(buf);
-			    cert->name = cpystr(df->d_name);
+			  if(x){ /* for now copy this information */
+			    cert = smime_X509_to_cert_info(x, df->d_name);
 			    /* we will use the cert->data.md5 variable to find a backup 
 			       certificate, not the name */
-			    if(data == NULL)
-			      data = cert;
-			    else{
-			      for (cl2 = data; cl2 && cl2->next; cl2 = cl2->next);
-			      cl2->next = cert;
-			    }
+			    cert->next = data;
+			    data = cert;
 			  }
 			  BIO_free(in);
 			}
@@ -245,6 +469,23 @@ setup_certs_backup_by_type(WhichCerts ctype)
    return rv;
 }
 
+char *
+smime_get_cn(X509 *x)
+{
+   X509_NAME_ENTRY *e;
+   X509_NAME *subject;
+   char    buf[256];
+   char *rv = NULL;
+
+   subject = X509_get_subject_name(x);
+   if((e = X509_NAME_get_entry(subject, X509_NAME_entry_count(subject)-1)) != NULL){
+      X509_NAME_get_text_by_OBJ(subject, X509_NAME_ENTRY_get_object(e), buf, sizeof(buf));
+      rv = cpystr(buf);
+   }
+
+   return rv;
+}
+
 int
 compare_certs_by_name(const void *data1, const void *data2)
 {
@@ -284,9 +525,9 @@ resort_certificates(CertList **data, WhichCerts ctype)
      return;
 
    for(i = 0; cl; cl = cl->next, i++)
-      if(ctype != Private){     /* ctype == Public or ctype == CACerts */
-         for(t = s = cl->name; t = strstr(s, ".crt"); s = t+1);
-         if (s) *(s-1) = '\0';
+      if(SMHOLDERTYPE(ctype) == Directory && ctype != Private){
+           for(t = s = cl->name; (t = strstr(s, ".crt")) != NULL; s = t+1);
+           if (s) *(s-1) = '\0';
       }
    j = i;
    cll = fs_get(i*sizeof(CertList *));
@@ -295,10 +536,10 @@ resort_certificates(CertList **data, WhichCerts ctype)
    qsort((void *)cll, j, sizeof(CertList *), compare_certs_by_name);
    for(i = 0; i < j - 1; i++){
      cll[i]->next = cll[i+1];
-     if(ctype != Private)
+     if(SMHOLDERTYPE(ctype) == Directory && ctype != Private)
         cll[i]->name[strlen(cll[i]->name)]= '.';    /* restore ".crt" part */
    }
-   if(ctype != Private)
+   if(SMHOLDERTYPE(ctype) == Directory && ctype != Private)
       cll[j-1]->name[strlen(cll[j-1]->name)]= '.';    /* restore ".crt" part */
    cll[j-1]->next = NULL;
    *data = cll[0];
@@ -369,7 +610,7 @@ emailstrclean(char *string)
 
 
 char *
-smime_get_date(ASN1_GENERALIZEDTIME *tm)
+smime_get_date(const ASN1_TIME *tm)
 {
    BIO *mb = BIO_new(BIO_s_mem());
    char iobuf[4096];
@@ -380,7 +621,9 @@ smime_get_date(ASN1_GENERALIZEDTIME *tm)
    struct tm smtm;
 
    (void) BIO_reset(mb);
-   ASN1_UTCTIME_print(mb, tm);
+   if(ASN1_TIME_print(mb, tm) == 0)
+     return cpystr(_("Invalid"));
+
    (void) BIO_flush(mb);
    BIO_read(mb, iobuf, sizeof(iobuf));
 
@@ -406,7 +649,7 @@ smime_get_date(ASN1_GENERALIZEDTIME *tm)
    if(F_ON(F_DATES_TO_LOCAL,ps_global)){
       parse_date(convert_date_to_local(date), &smd);
       memset(&smtm, 0, sizeof(smtm));
-      smtm.tm_year = MIN(MAX(smd.year-1900, 0), 2000) % 100 - 1900;
+      smtm.tm_year = smd.year - 1900;
       smtm.tm_mon  = MIN(MAX(smd.month-1, 0), 11);
       smtm.tm_mday = MIN(MAX(smd.day, 1), 31);
       our_strftime(buf, sizeof(buf), "%x", &smtm);
@@ -448,12 +691,12 @@ add_certs_in_dir(X509_LOOKUP *lookup, char *path, char *ext, CertList **cdata)
 		     cert->name = cpystr(d->d_name);
 		     /* read buf into a bio and fill the CertData structure */
 		     if((in = BIO_new_file(buf, "r"))!=0){
-			x = PEM_read_bio_X509(in, NULL, NULL, NULL);
-			if(x && x->cert_info){
-			   cert->data.date_from	= smime_get_date(x->cert_info->validity->notBefore);
-			   cert->data.date_to	= smime_get_date(x->cert_info->validity->notAfter);
+			if((x = PEM_read_bio_X509(in, NULL, NULL, NULL)) != NULL){
+			   cert->data.date_from	= smime_get_date(X509_get0_notBefore(x));
+			   cert->data.date_to	= smime_get_date(X509_get0_notAfter(x));
 			   get_fingerprint(x, EVP_md5(), buf, sizeof(buf), NULL);
 			   cert->data.md5	= cpystr(buf);
+			   cert->cn = smime_get_cn(x);
 			   X509_free(x);
 			}
 			BIO_free(in);
@@ -753,11 +996,17 @@ save_cert_for(char *email, X509 *cert, WhichCerts ctype)
 	      err++;
 
 	    if(!err && ret_dir){
-		if(strlen(path) + strlen(tempfile) - strlen(ret_dir) + 1 < sizeof(path))
-		   snprintf(fpath, sizeof(fpath), "%s%c%s", 
+		if(IS_REMOTE(upath)){
+		   strncpy(fpath, rd->lf, sizeof(fpath));
+		   fpath[sizeof(fpath)-1] = '\0';
+		}
+		else{
+		   if(strlen(path) + strlen(tempfile) - strlen(ret_dir) + 1 < sizeof(path))
+		     snprintf(fpath, sizeof(fpath), "%s%c%s", 
 			path, tempfile[strlen(ret_dir)], tempfile + strlen(ret_dir) + 1);
-		else
-		   err++;
+		   else
+		     err++;
+		}
 	    }
 	    else err++;
 
@@ -840,13 +1089,32 @@ save_cert_for(char *email, X509 *cert, WhichCerts ctype)
  * The caller should free the cert.
  */
 X509 *
-get_cert_for(char *email, WhichCerts ctype)
+get_cert_for(char *email, WhichCerts ctype, int tolower)
 {
     char	certfilename[MAXPATH];
     char    	emailaddr[MAXPATH];
     X509       *cert = NULL;
     BIO	       *in;
 
+    if(ctype == Password){
+	build_path(certfilename, PATHCERTDIR(ctype), email, sizeof(certfilename));
+	strncat(certfilename, EXTCERT(Public), sizeof(certfilename)-1-strlen(certfilename));
+	certfilename[sizeof(certfilename)-1] = 0;
+
+	if((in = BIO_new_file(certfilename, "r"))!=0){
+
+	    cert = PEM_read_bio_X509(in, NULL, NULL, NULL);
+
+	    if(cert){
+		/* could check email addr in cert matches */
+	    }
+
+	    BIO_free(in);
+	}
+
+       return cert;
+    }
+
     if(!ps_global->smime)
       return cert;
 
@@ -858,7 +1126,8 @@ get_cert_for(char *email, WhichCerts ctype)
     emailaddr[sizeof(emailaddr)-1] = 0;
     
     /* clean it up (lowercase, space removal) */
-    emailstrclean(emailaddr);
+    if(tolower)
+       emailstrclean(emailaddr);
 
     if(ps_global->smime->publictype == Keychain){
 #ifdef APPLEKEYCHAIN
@@ -1027,7 +1296,7 @@ mem_to_personal_certs(char *contents)
 
 	    if(strncmp(EMAILADDRLEADER, line, strlen(EMAILADDRLEADER)) == 0){
 		name = line + strlen(EMAILADDRLEADER);
-		cert = get_cert_for(name, Public);
+		cert = get_cert_for(name, Public, 1);
 		keytext = p;
 
 		/* advance p past this record */
@@ -1051,6 +1320,7 @@ mem_to_personal_certs(char *contents)
 		    pc = (PERSONAL_CERT *) fs_get(sizeof(*pc));
 		    pc->cert = cert;
 		    pc->name = cpystr(name);
+		    pc->cname = NULL;
 		    pc->keytext = keytext;	/* a pointer into contents */
 
 		    pc->key = load_key(pc, "", SM_NORMALCERT);
@@ -1120,6 +1390,8 @@ mem_to_certlist(char *contents, WhichCerts ctype)
 	      *save_p = '\n';
 	}
     }
+    if(ret != NULL)
+       resort_certificates(&ret, ctype);
 
     return ret;
 }
@@ -1178,11 +1450,8 @@ mem_add_extra_cacerts(char *contents, X509_LOOKUP *lookup)
 		    in = BIO_new_mem_buf(certtext, q-certtext);
 		    if(in){
 			tempfile = temp_nam(NULL, "az");
-			out = NULL;
-			if(tempfile)
-			  out = BIO_new_file(tempfile, "w");
-
-			if(out){
+			out = tempfile != NULL ? BIO_new_file(tempfile, "w") : NULL;
+			if(out != NULL){
 			    while((len = BIO_read(in, iobuf, sizeof(iobuf))) > 0)
 			      BIO_write(out, iobuf, len);
 
@@ -1190,7 +1459,10 @@ mem_add_extra_cacerts(char *contents, X509_LOOKUP *lookup)
 			    if(!X509_LOOKUP_load_file(lookup, tempfile, X509_FILETYPE_PEM))
 			      failed++;
 
-			    fs_give((void **) &tempfile);
+			}
+			if(tempfile != NULL){
+			   unlink(tempfile);
+			   fs_give((void **) &tempfile);
 			}
 			  
 			BIO_free(in);
@@ -1246,25 +1518,14 @@ certlist_to_file(char *filename, CertList *certlist)
 void
 add_to_end_of_certlist(CertList **cl, char *name, X509 *cert)
 {
-    CertList *new, *clp;
+    CertList *new;
 
     if(!cl)
       return;
 
-    new = (CertList *) fs_get(sizeof(*new));
-    memset((void *) new, 0, sizeof(*new));
-    new->x509_cert = cert;
-    new->name = name ? cpystr(name) : NULL;
-
-    if(!*cl){
-	*cl = new;
-    }
-    else{
-	for(clp = (*cl); clp->next; clp = clp->next)
-	  ;
-
-	clp->next = new;
-    }
+    new = smime_X509_to_cert_info(cert, name);
+    new->next = *cl;
+    *cl = new;
 }
 
 
@@ -1284,6 +1545,9 @@ free_certlist(CertList **cl)
 	if((*cl)->name)
 	  fs_give((void **) &(*cl)->name);
 
+	if((*cl)->cn)
+	  fs_give((void **) &(*cl)->cn);
+
 	if((*cl)->x509_cert)
 	  X509_free((X509 *) (*cl)->x509_cert);
 
@@ -1298,12 +1562,11 @@ void
 free_personal_certs(PERSONAL_CERT **pc)
 {
     if(pc && *pc){
-	free_personal_certs(&(*pc)->next);
 	if((*pc)->name)
 	  fs_give((void **) &(*pc)->name);
 	
-	if((*pc)->name)
-	  fs_give((void **) &(*pc)->name);
+	if((*pc)->cname)
+	  fs_give((void **) &(*pc)->cname);
 
 	if((*pc)->cert)
 	  X509_free((*pc)->cert);
@@ -1311,6 +1574,8 @@ free_personal_certs(PERSONAL_CERT **pc)
 	if((*pc)->key)
 	  EVP_PKEY_free((*pc)->key);
 
+	free_personal_certs(&(*pc)->next);
+
 	fs_give((void **) pc);
     }
 }
