@@ -2,8 +2,8 @@
  * $Id: state.h 1074 2008-06-04 00:08:43Z hubert@u.washington.edu $
  *
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -109,6 +109,7 @@ struct pine {
 
     char         cur_folder[MAXPATH+1];
     QUOTALIST	*quota;
+    IDLIST	*id;
     char         last_unambig_folder[MAXPATH+1];
     char         last_save_folder[MAXPATH+1];
     CONTEXT_S   *last_save_context;
@@ -120,6 +121,7 @@ struct pine {
 
     char        *folders_dir;
 
+    unsigned	 signal_in_progress:1;	/* we are handling a signal */
     unsigned     mangled_footer:1; 	/* footer needs repainting */
     unsigned     mangled_header:1;	/* header needs repainting */
     unsigned     mangled_body:1;	/* body of screen needs repainting */
@@ -193,8 +195,8 @@ struct pine {
     unsigned	 force_prefer_plain:1;
     unsigned	 force_no_prefer_plain:1;
 
-    unsigned	 phone_home:1;
     unsigned     painted_body_on_startup:1;
+    unsigned	 phone_home:1;
     unsigned     painted_footer_on_startup:1;
     unsigned     open_readonly_on_startup:1;
     unsigned     exit_if_no_pinerc:1;
@@ -249,6 +251,16 @@ struct pine {
 
     short	 init_context;
 
+    struct { 
+	unsigned keep_attach:1;
+ 	unsigned strip_signature:1;
+	unsigned use_flowed:1;
+	unsigned include_header:1;
+	unsigned preserve_fields:1;
+	unsigned signature_bottom:1;
+	ACTION_S *role_chosen;
+    } reply;
+
     int         *initial_cmds;         /* cmds to execute on startup */
     int         *free_initial_cmds;    /* used to free when done */
 
@@ -271,9 +283,6 @@ struct pine {
 #endif
 #ifdef PASSFILE
                 *passfile,
-#ifdef SMIME
-		*pwdcertdir,
-#endif /* SMIME inside PASSFILE */
 #endif /* PASSFILE */
                 *pinerc,	/* Location of user's pinerc */
                 *exceptions,	/* Location of user's exceptions */
@@ -293,8 +302,6 @@ struct pine {
     SortOrder    def_sort,	/* Default sort type */
 		 sort_types[22];
 
-    int		 preserve;
-
     int          last_expire_year, last_expire_month;
 
     int		 printer_category;
@@ -355,10 +362,13 @@ struct pine {
     PRINT_S	*print;
 
 #ifdef SMIME
+    char  *smimedir;
     SMIME_STUFF_S *smime;
-#ifdef PASSFILE
-    void *pwdcert;	/* this is of type PERSONAL_CERT */
-#endif /* PASSFILE inside SMIME */
+    char *pwdcertdir;	  /* path to location of certificates for password file */
+    void *pwdcert;	  /* this is of type PERSONAL_CERT */
+    char *pwdcertcontent; /* No comment yet */
+    void *backuppassword; /* this is of type CertList */
+    void *pwdcertlist;	  /* this is of type CertList */
 #endif /* SMIME */
 
     struct variable *vars;
