@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: stream.c 1012 2008-03-26 00:44:22Z hubert@u.washingt
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -114,7 +114,7 @@ pine_mail_open(MAILSTREAM *stream, char *mailbox, long int openflags, long int *
 	   openflags & SP_USERFLDR   ? " SP_USERFLDR"   : "",
 	   openflags & SP_USEPOOL    ? " SP_USEPOOL"    : "",
 	   openflags & SP_TEMPUSE    ? " SP_TEMPUSE"    : "",
-	   debug_time(1, ps_global->debug_timestamp)));
+	   debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
     
     if(retflags)
       *retflags = 0L;
@@ -569,7 +569,7 @@ pine_mail_open(MAILSTREAM *stream, char *mailbox, long int openflags, long int *
 	       && !strcmp(stream->dtb->name, "imap")){
 		dprint((7,
 		   "pine_mail_open: cancel idle timer: TAG %08lx (%s)\n",
-		   stream->gensym, debug_time(1, ps_global->debug_timestamp)));
+		   stream->gensym, debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 	    }
 
 	    /*
@@ -1247,7 +1247,7 @@ pine_mail_close(MAILSTREAM *stream)
 
     dprint((7, "pine_mail_close: %s (%s)\n", 
 	       stream && stream->mailbox ? stream->mailbox : "(NULL)",
-	       debug_time(1, ps_global->debug_timestamp)));
+	       debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 
     if(sp_flagged(stream, SP_USEPOOL) && !sp_dead_stream(stream)){
 
@@ -1296,7 +1296,7 @@ pine_mail_close(MAILSTREAM *stream)
 	if(refcnt <= 0){
 	    dprint((7,
 	       "pine_mail_close: unlocking: start idle timer: TAG %08lx (%s)\n",
-	       stream->gensym, debug_time(1, ps_global->debug_timestamp)));
+	       stream->gensym, debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 	    sp_set_last_use_time(stream, time(0));
 
 	    /*
@@ -1350,7 +1350,7 @@ pine_mail_actually_close(MAILSTREAM *stream)
     if(!sp_closing(stream)){
 	dprint((7, "pine_mail_actually_close: %s (%s)\n", 
 		   stream && stream->mailbox ? stream->mailbox : "(NULL)",
-		   debug_time(1, ps_global->debug_timestamp)));
+		   debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 
 	sp_set_closing(stream, 1);
 	
@@ -1386,7 +1386,7 @@ maybe_kill_old_stream(MAILSTREAM *stream)
 	dprint((7,
 		"killing idle stream: %s (%s): idle timer = %ld secs\n", 
 		stream && stream->mailbox ? stream->mailbox : "(NULL)",
-		debug_time(1, ps_global->debug_timestamp),
+		debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress),
 		(long) (time(0)-sp_last_use_time(stream))));
 
 	/*
@@ -1909,9 +1909,9 @@ mail_list_internal(MAILSTREAM *s, char *r, char *p)
        && ((s && s->mailbox && *s->mailbox == '{')
 	   || (!s && ((r && *r == '{') || (p && *p == '{'))))){
 	char tmp[2*MAILTMPLEN];
-
-	snprintf(tmp, sizeof(tmp), "%.*s%.*s", sizeof(tmp)/2-1, r ? r : "",
-		sizeof(tmp)/2-1, p);
+				/* MAILTMPLEN = sizeof(tmp)/2 */
+	snprintf(tmp, sizeof(tmp), "%.*s%.*s", MAILTMPLEN-1, r ? r : "",
+		MAILTMPLEN-1, p);
 	mail_list(s, "", tmp);
     }
     else
@@ -2580,7 +2580,7 @@ sp_stream_get(char *mailbox, long unsigned int flags)
 			dprint((7,
 			       "reset idle timer1: next TAG %08lx (%s)\n",
 			       m->gensym,
-			       debug_time(1, ps_global->debug_timestamp)));
+			       debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 			sp_set_last_use_time(m, time(0));
 		    }
 
@@ -2635,7 +2635,7 @@ sp_stream_get(char *mailbox, long unsigned int flags)
 			dprint((7,
 			   "reset idle timer4: next TAG %08lx (%s)\n",
 			   m->gensym,
-			   debug_time(1, ps_global->debug_timestamp)));
+			   debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 			sp_set_last_use_time(m, time(0));
 
 			return(m);
@@ -2663,7 +2663,7 @@ sp_stream_get(char *mailbox, long unsigned int flags)
 		    dprint((7,
 			   "reset idle timer2: next TAG %08lx (%s)\n",
 			   m->gensym,
-			   debug_time(1, ps_global->debug_timestamp)));
+			   debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 		    sp_set_last_use_time(m, time(0));
 		    return(m);
 		}
@@ -2696,7 +2696,7 @@ sp_stream_get(char *mailbox, long unsigned int flags)
 			dprint((7,
 			   "reset idle timer3: next TAG %08lx (%s)\n",
 			   m->gensym,
-			   debug_time(1, ps_global->debug_timestamp)));
+			   debug_time(1, ps_global->debug_timestamp, ps_global->signal_in_progress)));
 			sp_set_last_use_time(m, time(0));
 			return(m);
 		    }
@@ -3276,13 +3276,13 @@ streams_died(void)
 		    folder = folder_name_decoded((unsigned char *)STREAMNAME(m));
 		    q_status_message1(SM_ORDER | SM_DING, 3, 3,
 			  _("MAIL FOLDER \"%s\" CLOSED DUE TO ACCESS ERROR"),
-			  short_str(pretty_fn(folder) ? pretty_fn(folder) : "?",
+			  short_str(pretty_fn((char *) folder) ? pretty_fn((char *) folder) : "?",
 				    tmp_20k_buf+1000, SIZEOF_20KBUF-1000, 35, FrontDots));
 		    dprint((6, "streams_died: locked: \"%s\"\n",
 			    folder));
 		    if(rv == 1){
 			snprintf(tmp_20k_buf, SIZEOF_20KBUF, _("Folder \"%s\" is Closed"), 
-			  short_str(pretty_fn(folder) ? pretty_fn(folder) : "?",
+			  short_str(pretty_fn((char *)folder) ? pretty_fn((char *)folder) : "?",
 			  tmp_20k_buf+1000, SIZEOF_20KBUF-1000, 35, FrontDots));
 			if(pith_opt_icon_text)
 			  (*pith_opt_icon_text)(tmp_20k_buf, IT_MCLOSED);
@@ -3293,7 +3293,7 @@ streams_died(void)
 	    else{
 		if(!sp_noticed_dead_stream(m)){
 		    sp_set_noticed_dead_stream(m, 1);
-		    folder = STREAMNAME(m);
+		    folder = (unsigned char *) STREAMNAME(m);
 		    /*
 		     * If a cached stream died and then we tried to use it
 		     * it could cause problems. We could warn about it here
