@@ -4,8 +4,8 @@ static char rcsid[] = "$Id: string.c 910 2008-01-14 22:28:38Z hubert@u.washingto
 
 /*
  * ========================================================================
+ * Copyright 2013-2017 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -1588,6 +1588,18 @@ fold(char *src, int width, int maxwidth, char *first_indent, char *indent, unsig
 		}
 	    }
 
+	    if(winner == -1 && (flags & FLD_NEXTSPC)){
+		for(i = starting_point; winner == -1 && i <= strlen(next_piece) != '\0' && i < 512; i++){
+		    endptr = utf8_count_forw_width(next_piece, i, &got_width);
+		    if(endptr && got_width == i && isspace((unsigned char) *endptr))
+		      winner = (int) i;
+		}
+		if(winner == -1){
+		   winner = got_width < 512 ? got_width : 512;
+		   endptr = NULL;
+		}
+	    }
+
 	    if(winner == -1){ /* if no good folding spot, fold at width */
 		winner = starting_point;
 		endptr = NULL;
@@ -1652,6 +1664,18 @@ fold(char *src, int width, int maxwidth, char *first_indent, char *indent, unsig
 		}
 	    }
 
+	    if(winner == -1 && (flags & FLD_NEXTSPC)){
+		for(i = starting_point; winner == -1 && i <= strlen(next_piece) && i < 512; i++){
+		    endptr = utf8_count_forw_width(next_piece, i, &got_width);
+		    if(endptr && got_width == i && isspace((unsigned char) *endptr))
+		      winner = (int) i;
+		}
+		if(winner == -1){
+		   winner = got_width < 512 ? got_width : 512;
+		   endptr = NULL;
+		}
+	    }
+
 	    if(winner == -1){ /* if no good folding spot, fold at width */
 		winner = starting_point;
 		endptr = NULL;
@@ -2889,3 +2913,73 @@ free_strlist(STRLIST_S **strp)
 	fs_give((void **) strp);
     }
 }
+
+void
+convert_decimal_to_roman (char *rn, size_t len, long n, char l)
+{
+  char symbols[7];
+  int amo[7];
+  int i, j, k;
+
+  rn[0] = '\0';
+  if(n >= 4000L || n <= 0L)
+    return;
+
+  symbols[0] = l + 'm' - 'i';
+  symbols[1] = l + 'd' - 'i';
+  symbols[2] = l + 'c' - 'i';
+  symbols[3] = l + 'l' - 'i';
+  symbols[4] = l + 'x' - 'i';
+  symbols[5] = l + 'v' - 'i';
+  symbols[6] = l;
+  
+  amo[0] = n/1000; n -= amo[0]*1000;
+  amo[1] = n/500;  n -= amo[1]*500;
+  amo[2] = n/100;  n -= amo[2]*100;
+  amo[3] = n/50;   n -= amo[3]*50;
+  amo[4] = n/10;   n -= amo[4]*10;
+  amo[5] = n/5;    n -= amo[5]*5;
+  amo[6] = n;
+
+  for(i = 0, j = 0; i < len && j < strlen(symbols); j++){
+     if(amo[j] < 4){
+	if(amo[j+1] != 4){
+	  for(k = 0; k < amo[j]; k++)
+	    rn[i++] = symbols[j];
+	}
+     } else {
+	if(amo[j-1] == 0){
+	  rn[i++] = symbols[j];
+	  rn[i++] = symbols[j-1];
+	} else {
+	  rn[i++] = symbols[j];
+	  rn[i++] = symbols[j-2];
+	}
+     }
+  }
+  if(i < len) rn[i] = '\0';
+  rn[len-1] = '\0';
+}
+
+void
+convert_decimal_to_alpha (char *rn, size_t len, long n, char l)
+{
+  int amo[16];
+  int i;
+
+  rn[0] = '\0';
+
+  if(n < 0)
+    return;
+
+  for(i = 0; i < sizeof(amo) && n > 0; i++){
+     amo[i] = n % 26; 
+          n = (n - amo[i])/26;
+  }
+  amo[i] = -1;
+
+  for(i = 0; i < len && amo[i] >= 0; i++)
+     rn[i] = l + amo[i] - 1;
+  if(i < len) rn[i] = '\0';
+  rn[len-1] = '\0';
+}
