@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: takeaddr.c 1012 2008-03-26 00:44:22Z hubert@u.washin
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2015 Eduardo Chappa
+ * Copyright 2013-2017 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -1804,7 +1804,7 @@ detach_vcard_att(MAILSTREAM *stream, long int msgno, struct mail_bodystruct *bod
     if(dtext)
       fs_give((void **)&dtext);
 
-    res[count] = '\0';
+    res[count] = NULL;
     return(res);
 }
 
@@ -1824,6 +1824,8 @@ grab_addrs_from_body(MAILSTREAM *stream, long int msgno,
     gf_io_t    pc;
     SourceType src = CharStar;
     int        added = 0, failure;
+    char      *partno = "1";
+    ATTACH_S  *a = ps_global->atmts;
 
     dprint((9, "\n - grab_addrs_from_body - \n"));
 
@@ -1848,7 +1850,12 @@ grab_addrs_from_body(MAILSTREAM *stream, long int msgno,
 
     gf_set_so_writec(&pc, so);
 
-    failure = !get_body_part_text(stream, body, msgno, "1", 0L, pc,
+    if(body->type == TYPEMULTIPART 
+	&& strucmp(body->subtype, "ALTERNATIVE") == 0
+	&& a && a+1 && (a+1)->shown) 
+	partno = "2";
+
+    failure = !get_body_part_text(stream, body, msgno, partno, 0L, pc,
 				  NULL, NULL, GBPT_NONE);
 
     gf_clear_so_writec(so);
